@@ -22,6 +22,7 @@
 package org.jboss.ha.framework.server;
 
 
+import java.lang.reflect.Constructor;
 import java.lang.reflect.Method;
 import java.net.InetAddress;
 import java.rmi.dgc.VMID;
@@ -46,7 +47,6 @@ import org.jboss.system.server.ServerConfigUtil;
 import org.jgroups.Channel;
 import org.jgroups.JChannel;
 import org.jgroups.Version;
-import org.jgroups.debug.Debugger;
 import org.jgroups.jmx.JChannelFactoryMBean;
 import org.jgroups.jmx.JmxConfigurator;
 import org.w3c.dom.Attr;
@@ -61,7 +61,7 @@ import org.w3c.dom.NodeList;
  *
  *   @author <a href="mailto:bill@burkecentral.com">Bill Burke</a>.
  *   @author <a href="mailto:sacha.labourey@cogito-info.ch">Sacha Labourey</a>.
- *   @version $Revision: 62947 $
+ *   @version $Revision: 67538 $
  */
 public class ClusterPartition
    extends ServiceMBeanSupport 
@@ -97,7 +97,7 @@ public class ClusterPartition
    protected JChannelFactoryMBean multiplexer = null;
    protected String stackName = null;
    protected org.jgroups.JChannel channel;
-   protected Debugger debugger=null;
+   protected Object debugger=null;
    protected boolean use_debugger=false;
 
    protected String nodeName = null;
@@ -343,8 +343,7 @@ public class ClusterPartition
       
       if(use_debugger && debugger == null)
       {
-         debugger=new Debugger(channel);
-         debugger.start();
+         startChannelDebugger();
       }
       channel.setOpt(Channel.AUTO_RECONNECT, Boolean.TRUE);
       channel.setOpt(Channel.AUTO_GETSTATE, Boolean.TRUE);
@@ -529,8 +528,20 @@ public class ClusterPartition
    {
       if(debugger == null)
       {
-         debugger=new Debugger(this.channel, accumulative);
-         debugger.start();
+         try
+         {
+            Class clazz = getClass().getClassLoader().loadClass("org.jgroups.debug.Debugger");
+            Constructor ctor = clazz.getConstructor(new Class[]
+            {JChannel.class, boolean.class});
+            debugger = ctor.newInstance(new Object[]
+            {channel, Boolean.valueOf(accumulative)});
+            Method start = clazz.getDeclaredMethod("start", new Class[] {});
+            start.invoke(debugger, new Object[] {});
+         }
+         catch (Exception e)
+         {
+            log.error("Unable to start Debugger", e);
+         }
       }
    }
 
@@ -538,7 +549,16 @@ public class ClusterPartition
    {
       if(debugger != null)
       {
-         // debugger.stop(); // uncomment when new JGroups version is available
+         Class clazz = debugger.getClass();
+         try
+         {
+            Method stop = clazz.getDeclaredMethod("stop", new Class[] {});
+            stop.invoke(debugger, new Object[]{});
+         }
+         catch (Exception e)
+         {
+            log.error("Unable to cleanly stop Debugger", e);
+         }
          debugger=null;
       }
    }
