@@ -53,7 +53,7 @@ import org.jboss.ha.framework.interfaces.HAPartition;
  * @author  <a href="mailto:bill@burkecentral.com">Bill Burke</a>.
  * @author  <a href="mailto:sacha.labourey@cogito-info.ch">Sacha Labourey</a>.
  * @author  Scott.stark@jboss.org
- * @version $Revision: 61770 $
+ * @version $Revision: 69143 $
  */
 public class DistributedReplicantManagerImpl
    implements DistributedReplicantManagerImplMBean,
@@ -104,12 +104,14 @@ public class DistributedReplicantManagerImpl
       this.mbeanserver = server;
       this.log = Logger.getLogger(DistributedReplicantManagerImpl.class.getName() + "." + partition.getPartitionName());
       this.trace = log.isTraceEnabled();
+      // JBAS-5068 Create the handler early so we don't risk NPEs
+      this.asynchHandler = new AsynchEventHandler(this, "AsynchKeyChangeHandler");
    }
 
    // Public --------------------------------------------------------
    
    public void init() throws Exception
-   {
+   {  
       log.debug("registerRPCHandler");
       partition.registerRPCHandler(SERVICE_NAME, this);
       log.debug("subscribeToStateTransferEvents");
@@ -131,9 +133,7 @@ public class DistributedReplicantManagerImpl
    public void start() throws Exception
    {
       this.nodeName = this.partition.getNodeName ();
-      
-      // Create the asynch listener handler thread
-      asynchHandler = new AsynchEventHandler(this, "AsynchKeyChangeHandler");
+            
       asynchHandler.start();
 
       partitionNameKnown.release (); // partition name is now known!
