@@ -45,10 +45,15 @@ import org.jboss.remoting.CannotConnectException;
 import org.jboss.remoting.Client;
 import org.jboss.remoting.InvokerLocator;
 import org.jboss.remoting.serialization.IMarshalledValue;
+import org.jboss.tm.TransactionPropagationContextFactory;
+import org.jboss.tm.TransactionPropagationContextUtil;
 
 
 /**
+ * Unified invoker implementation for InvokerProxyHA
+ * 
  * @author <a href="mailto:tom.elrod@jboss.com">Tom Elrod</a>
+ * @author <a href="mailto:galder.zamarreno@jboss.com">Galder Zamarreno</a>
  */
 public class UnifiedInvokerHAProxy 
    extends UnifiedInvokerProxy
@@ -64,11 +69,19 @@ public class UnifiedInvokerHAProxy
 
    public static final WeakHashMap txFailoverAuthorizations = new WeakHashMap();
 
+   /** Trace level logging flag only set when the proxy is created or read from JNDI */
+   private static boolean trace = false;
 
    public UnifiedInvokerHAProxy()
    {
       super();
-      log.debug("UnifiedInvokerHAProxy constructor called with no arguments.");
+      trace = log.isTraceEnabled();
+
+      if (trace)
+      {
+         log.trace("UnifiedInvokerHAProxy constructor called with no arguments.");
+      }
+      
       setSubSystem("invokerha");
    }
 
@@ -81,34 +94,51 @@ public class UnifiedInvokerHAProxy
       this.familyClusterInfo = ClusteringTargetsRepository.initTarget(proxyFamilyName, targets, viewId);
       this.loadBalancePolicy = policy;
       this.proxyFamilyName = proxyFamilyName;
+      trace = log.isTraceEnabled();
 
       setSubSystem("invokerha");
    }
 
    public boolean txContextAllowsFailover(Invocation invocation)
    {
-      javax.transaction.Transaction tx = invocation.getTransaction();
-      if(tx != null)
+      Object tpc = getTransactionPropagationContext();
+      if(tpc != null)
       {
-         synchronized(tx)
+         synchronized(tpc)
          {
-            return ! txFailoverAuthorizations.containsKey(tx);
+            if (trace)
+            {
+               log.trace("checking tx failover authorisation map with tpc " + tpc);
+            }
+
+            /* if the map contains the tpc, then we can't allow a failover */
+            return ! txFailoverAuthorizations.containsKey(tpc);
          }
       }
-      else
-      {
-         return true;
-      }
+      
+      return true;
    }
 
    public void invocationHasReachedAServer(Invocation invocation)
    {
-      javax.transaction.Transaction tx = invocation.getTransaction();
-      if(tx != null)
+      Object tpc = getTransactionPropagationContext();
+      if(tpc != null)
       {
-         synchronized(tx)
+         synchronized(tpc)
          {
-            txFailoverAuthorizations.put(tx, null);
+            if (trace)
+            {
+               log.trace("after reaching the server, transaction propagation context (tpc) is " + tpc);
+            }
+            
+            Object stickyTarget = invocation.getTransientValue("TX_STICKY_TARGET");
+            
+            if (trace && stickyTarget != null)
+            {
+               log.trace("remember transaction bound target[" + stickyTarget + "] for tpc[" + tpc + "]");
+            }
+            
+            txFailoverAuthorizations.put(tpc, stickyTarget);
          }
       }
    }
@@ -139,7 +169,7 @@ public class UnifiedInvokerHAProxy
     * @return
     * @throws MalformedURLException
     */
-   protected Client getClient(Invocation invocationBasedRouting) throws MalformedURLException
+   protected synchronized Client getClient(Invocation invocationBasedRouting) throws MalformedURLException
    {
       Object target = loadBalancePolicy.chooseTarget(familyClusterInfo, invocationBasedRouting);
       InvokerLocator targetLocator = (InvokerLocator) target;
@@ -165,6 +195,9 @@ public class UnifiedInvokerHAProxy
       //
       int failoverCounter = 0;
       invocation.setValue("FAILOVER_COUNTER", new Integer(failoverCounter), PayloadKey.AS_IS);
+      
+      // If transaction sticky, put chosen target
+      putIfExistsTransactionTarget(invocation, getTransactionPropagationContext());
 
       Object response = null;
       Exception lastException = null;
@@ -178,12 +211,18 @@ public class UnifiedInvokerHAProxy
          {
             invocation.setValue("CLUSTER_VIEW_ID", new Long(this.familyClusterInfo.getCurrentViewId()));
 
-            log.debug("Client cluster view id: " + familyClusterInfo.getCurrentViewId());
-            log.debug(printPossibleTargets());
+            if (trace)
+            {
+               log.trace("Client cluster view id: " + familyClusterInfo.getCurrentViewId());
+               log.trace(printPossibleTargets());
+            }
 
             Client clientInstance = getClient(invocation);
 
-            log.debug("Making invocation on " + clientInstance.getInvoker().getLocator());
+            if (trace)
+            {
+               log.trace("Making invocation on " + clientInstance.getInvoker().getLocator());
+            }
 
             response = clientInstance.invoke(invocation, null);
 
@@ -191,7 +230,10 @@ public class UnifiedInvokerHAProxy
 
             if(response instanceof Exception)
             {
-               log.debug("Invocation returened exception: " + response);
+               if (trace)
+               {
+                  log.trace("Invocation returned exception: " + response);
+               }
                if(response instanceof GenericClusteringException)
                {
                   GenericClusteringException gcex = (GenericClusteringException) response;
@@ -222,7 +264,10 @@ public class UnifiedInvokerHAProxy
                      failoverCounter++;
                      invocation.setValue("FAILOVER_COUNTER", new Integer(failoverCounter), PayloadKey.AS_IS);
 
-                     log.debug("Received GenericClusteringException where request was not completed.  Will retry.");
+                     if (trace)
+                     {
+                        log.trace("Received GenericClusteringException where request was not completed.  Will retry.");
+                     }
 
                      continue;
                   }
@@ -256,6 +301,8 @@ public class UnifiedInvokerHAProxy
             {
                updateClusterInfo(haResponse.newReplicants, haResponse.currentViewId);
             }
+            
+            invocationHasReachedAServer(invocation);
 
             response = haResponse.response;
             return response;
@@ -263,7 +310,11 @@ public class UnifiedInvokerHAProxy
          }
          catch(CannotConnectException cncEx)
          {
-            log.debug("Invocation failed: CannotConnectException - " + cncEx, cncEx);
+            if (trace)
+            {
+               log.trace("Invocation failed: CannotConnectException - " + cncEx, cncEx);
+            }
+            
             removeDeadTarget(getLocator());
             resetView();
             failoverAuthorized = txContextAllowsFailover(invocation);
@@ -300,7 +351,10 @@ public class UnifiedInvokerHAProxy
                failoverCounter++;
                invocation.setValue("FAILOVER_COUNTER", new Integer(failoverCounter), PayloadKey.AS_IS);
 
-               log.debug("Received GenericClusteringException where request was not completed.  Will retry.");
+               if (trace)
+               {
+                  log.trace("Received GenericClusteringException where request was not completed.  Will retry.");
+               }
             }
             else
             {
@@ -310,7 +364,10 @@ public class UnifiedInvokerHAProxy
          }
          catch(RemoteException aex)
          {
-            log.debug("Invocation failed: RemoteException - " + aex, aex);
+            if (trace)
+            {
+               log.trace("Invocation failed: RemoteException - " + aex, aex);
+            }
 
             // per Jira issue JBREM-61
             if(isStrictRMIException())
@@ -324,7 +381,10 @@ public class UnifiedInvokerHAProxy
          }
          catch(Throwable throwable)
          {
-            log.debug("Invocation failed: " + throwable, throwable);
+            if (trace)
+            {
+               log.trace("Invocation failed: " + throwable, throwable);
+            }
 
             // this is somewhat of a hack as remoting throws throwable,
             // so will let Exception types bubble up, but if Throwable type,
@@ -376,7 +436,10 @@ public class UnifiedInvokerHAProxy
          if(this.familyClusterInfo != null)
          {
             familyClusterInfo.removeDeadTarget(locator);
-            log.debug("Removed " + locator + " from target list.");
+            if (trace)
+            {
+               log.trace("Removed " + locator + " from target list.");
+            }
          }
       }
    }
@@ -387,11 +450,14 @@ public class UnifiedInvokerHAProxy
       if(familyClusterInfo != null)
       {
          familyClusterInfo.updateClusterInfo(newReplicants, currentViewId);
-         log.debug("Updating cluster info.  New view id: " + currentViewId);
-         log.debug("New cluster target list is:");
-         for(int x = 0; x < newReplicants.size(); x++)
+         if (trace)
          {
-            log.debug(newReplicants.get(x));
+            log.trace("Updating cluster info.  New view id: " + currentViewId);
+            log.trace("New cluster target list is:");
+            for(int x = 0; x < newReplicants.size(); x++)
+            {
+               log.trace(newReplicants.get(x));
+            }
          }
       }
    }
@@ -448,7 +514,54 @@ public class UnifiedInvokerHAProxy
          default:
             throw new StreamCorruptedException("Unknown version seen: " + version);
       }
+      
+      trace = log.isTraceEnabled();
+      if(trace)
+      {
+         log.trace("Init, clusterInfo: "+familyClusterInfo+", policy="+loadBalancePolicy);
+      }
    }
 
+   /**
+    * Before invocation, access to transaction propagation context is needed
+    * to find out whether the invocation is part of an on going transaction and 
+    * might need it's target being sticky to this tx. 
+    */
+   protected Object getTransactionPropagationContext()
+   {
+      TransactionPropagationContextFactory tpcFactory = TransactionPropagationContextUtil.getTPCFactoryClientSide();
+      return (tpcFactory == null) ? null : tpcFactory.getTransactionPropagationContext();
+   }
+   
+   /**
+    * Called at the beginning of the invocation to check whether the current tpc
+    * is already present in the tx failover map. If it is, get the chosen 
+    * target associated to it and add it to the invocation transient payload so 
+    * that the load balance policy can choose the right target.
+    */
+   protected void putIfExistsTransactionTarget(Invocation invocation, Object tpc)
+   {
+      if (tpc != null)
+      {
+         synchronized (tpc)
+         {
+            if (trace)
+            {
+               log.trace("in the proxy, transaction propagation context (tpc) is " + tpc);
+            }
 
+            Object stickyTarget = txFailoverAuthorizations.get(tpc);
+               
+            if (stickyTarget != null)
+            {
+               if (trace)
+               {
+                  log.trace("put transaction bound target [" + stickyTarget + "] into transient payload");
+               }
+               
+               invocation.getTransientPayload().put("TX_STICKY_TARGET", stickyTarget);
+            }
+         }
+      }
+   }
 }
\ No newline at end of file
