@@ -34,6 +34,8 @@ import java.util.HashSet;
 import java.util.Iterator;
 import java.util.Properties;
 import java.util.Set;
+import java.util.concurrent.TimeUnit;
+import java.util.concurrent.locks.ReentrantLock;
 
 import javax.resource.ResourceException;
 import javax.resource.spi.ConnectionEvent;
@@ -49,53 +51,98 @@ import org.jboss.resource.JBossResourceException;
 
 import EDU.oswego.cs.dl.util.concurrent.SynchronizedBoolean;
 
+import org.jboss.resource.statistic.JBossConnectionStatistics;
+
 /**
  * BaseWrapperManagedConnection
  *
  * @author <a href="mailto:d_jencks@users.sourceforge.net">David Jencks</a>
  * @author <a href="mailto:adrian@jboss.com">Adrian Brock</a>
- * @version $Revision: 57189 $
+ * @author <a href="mailto:weston.price@jboss.com>Weston Price</a>
+ * @version $Revision: 75426 $
  */
 
-public abstract class BaseWrapperManagedConnection
-   implements  ManagedConnection
+public abstract class BaseWrapperManagedConnection implements ManagedConnection , JBossConnectionStatistics
 {
+   private static final WrappedConnectionFactory wrappedConnectionFactory;
+   
    protected final BaseWrapperManagedConnectionFactory mcf;
+
    protected final Connection con;
+
    protected final Properties props;
+
    private final int transactionIsolation;
+
    private final boolean readOnly;
 
+   private ReentrantLock lock = new ReentrantLock();
+
    private final Collection cels = new ArrayList();
+
    private final Set handles = new HashSet();
+
    private PreparedStatementCache psCache = null;
 
    protected final Object stateLock = new Object();
-   
-   protected boolean inManagedTransaction = false;
-   protected SynchronizedBoolean inLocalTransaction = new SynchronizedBoolean(false);
-   protected boolean jdbcAutoCommit = true;
-   protected boolean underlyingAutoCommit = true;
-   protected boolean jdbcReadOnly;
-   protected boolean underlyingReadOnly;
-   protected int jdbcTransactionIsolation;
-   protected boolean destroyed = false;
 
-   public BaseWrapperManagedConnection (final BaseWrapperManagedConnectionFactory mcf,
-                                  final Connection con,
-                                  final Properties props,
-                                  final int transactionIsolation,
-                                  final int psCacheSize)
-      throws SQLException
+   protected boolean inManagedTransaction = false;
+
+   protected SynchronizedBoolean inLocalTransaction = new SynchronizedBoolean(false);
+
+   protected boolean jdbcAutoCommit = true;
+
+   protected boolean underlyingAutoCommit = true;
+
+   protected boolean jdbcReadOnly;
+
+   protected boolean underlyingReadOnly;
+
+   protected int jdbcTransactionIsolation;
+
+   protected boolean destroyed = false;
+   
+   static
+   {
+      Class connectionFactory = null;
+      try
+      {
+         connectionFactory = BaseWrapperManagedConnection.class.forName("org.jboss.resource.adapter.jdbc.jdk6.WrappedConnectionFactoryJDK6");
+      }
+      catch (ClassNotFoundException e)
+      {
+         try
+         {
+            connectionFactory = BaseWrapperManagedConnection.class.forName("org.jboss.resource.adapter.jdbc.jdk5.WrappedConnectionFactoryJDK5");
+         }
+         catch (ClassNotFoundException ex)
+         {
+            throw new RuntimeException("Unabled to load wrapped connection factory", ex);
+         }
+      }
+      try
+      {
+         wrappedConnectionFactory = (WrappedConnectionFactory) connectionFactory.newInstance();
+      }
+      catch (Exception e)
+      {
+         throw new RuntimeException("Error initializign connection factory", e);
+      }
+   }
+
+   public BaseWrapperManagedConnection(final BaseWrapperManagedConnectionFactory mcf, final Connection con,
+         final Properties props, final int transactionIsolation, final int psCacheSize) throws SQLException
    {
       this.mcf = mcf;
       this.con = con;
       this.props = props;
+
       if (psCacheSize > 0)
-          psCache = new PreparedStatementCache(psCacheSize);
+         psCache = new PreparedStatementCache(psCacheSize);
 
       if (transactionIsolation == -1)
          this.transactionIsolation = con.getTransactionIsolation();
+
       else
       {
          this.transactionIsolation = transactionIsolation;
@@ -142,8 +189,8 @@ public abstract class BaseWrapperManagedConnection
    {
       if (!(handle instanceof WrappedConnection))
          throw new JBossResourceException("Wrong kind of connection handle to associate" + handle);
-      ((WrappedConnection)handle).setManagedConnection(this);
-      synchronized(handles)
+      ((WrappedConnection) handle).setManagedConnection(this);
+      synchronized (handles)
       {
          handles.add(handle);
       }
@@ -170,9 +217,9 @@ public abstract class BaseWrapperManagedConnection
    {
       synchronized (handles)
       {
-         for (Iterator i = handles.iterator(); i.hasNext(); )
+         for (Iterator i = handles.iterator(); i.hasNext();)
          {
-            WrappedConnection lc = (WrappedConnection)i.next();
+            WrappedConnection lc = (WrappedConnection) i.next();
             lc.setManagedConnection(null);
          }
          handles.clear();
@@ -195,12 +242,46 @@ public abstract class BaseWrapperManagedConnection
             }
          }
       }
+      // I'm recreating the lock object when we return to the pool
+      // because it looks too nasty to expect the connection handle
+      // to unlock properly in certain race conditions
+      // where the dissociation of the managed connection is "random".
+      lock = new ReentrantLock();
+   }
+
+   protected void lock()
+   {
+      lock.lock();
+   }
+
+   protected void tryLock() throws SQLException
+   {
+      int tryLock = mcf.getUseTryLock();
+      if (tryLock <= 0)
+      {
+         lock();
+         return;
+      }
+      try
+      {
+         if (lock.tryLock(tryLock, TimeUnit.SECONDS) == false)
+            throw new SQLException("Unable to obtain lock in " + tryLock + " seconds: " + this);
+      }
+      catch (InterruptedException e)
+      {
+         throw new SQLException("Interrupted attempting lock: " + this);
+      }
+   }
+   
+   protected void unlock()
+   {
+      lock.unlock();
    }
 
    public Object getConnection(Subject subject, ConnectionRequestInfo cri) throws ResourceException
    {
       checkIdentity(subject, cri);
-      WrappedConnection lc =  new WrappedConnection(this);
+      WrappedConnection lc = wrappedConnectionFactory.createWrappedConnection(this);
       synchronized (handles)
       {
          handles.add(lc);
@@ -217,6 +298,16 @@ public abstract class BaseWrapperManagedConnection
 
       cleanup();
       try
+      {
+         // See JBAS-5678
+         if (underlyingAutoCommit == false)
+            con.rollback();
+      }
+      catch (SQLException ignored)
+      {
+         getLog().trace("Ignored error during rollback: ", ignored);
+      }
+      try
       {
          con.close();
       }
@@ -241,6 +332,12 @@ public abstract class BaseWrapperManagedConnection
       }
    }
 
+   public Properties getProperties()
+   {
+      return this.props;
+
+   }
+
    void closeHandle(WrappedConnection handle)
    {
       synchronized (stateLock)
@@ -249,30 +346,33 @@ public abstract class BaseWrapperManagedConnection
             return;
       }
 
-      synchronized(handles)
+      synchronized (handles)
       {
          handles.remove(handle);
       }
       ConnectionEvent ce = new ConnectionEvent(this, ConnectionEvent.CONNECTION_CLOSED);
       ce.setConnectionHandle(handle);
       Collection copy = null;
-      synchronized(cels)
+      synchronized (cels)
       {
          copy = new ArrayList(cels);
       }
-      for (Iterator i = copy.iterator(); i.hasNext(); )
+      for (Iterator i = copy.iterator(); i.hasNext();)
       {
-         ConnectionEventListener cel = (ConnectionEventListener)i.next();
+         ConnectionEventListener cel = (ConnectionEventListener) i.next();
          cel.connectionClosed(ce);
       }
    }
 
-   void connectionError(Throwable t)
+   Throwable connectionError(Throwable t)
    {
       if (t instanceof SQLException == false || mcf.isExceptionFatal((SQLException) t))
          broadcastConnectionError(t);
+
+      return t;
    }
 
+
    protected void broadcastConnectionError(Throwable e)
    {
       synchronized (stateLock)
@@ -285,7 +385,7 @@ public abstract class BaseWrapperManagedConnection
             return;
          }
       }
-      
+
       Exception ex = null;
       if (e instanceof Exception)
          ex = (Exception) e;
@@ -293,13 +393,13 @@ public abstract class BaseWrapperManagedConnection
          ex = new ResourceAdapterInternalException("Unexpected error", e);
       ConnectionEvent ce = new ConnectionEvent(this, ConnectionEvent.CONNECTION_ERROR_OCCURRED, ex);
       Collection copy = null;
-      synchronized(cels)
+      synchronized (cels)
       {
          copy = new ArrayList(cels);
       }
-      for (Iterator i = copy.iterator(); i.hasNext(); )
+      for (Iterator i = copy.iterator(); i.hasNext();)
       {
-         ConnectionEventListener cel = (ConnectionEventListener)i.next();
+         ConnectionEventListener cel = (ConnectionEventListener) i.next();
          try
          {
             cel.connectionErrorOccurred(ce);
@@ -311,8 +411,7 @@ public abstract class BaseWrapperManagedConnection
       }
    }
 
-   Connection getConnection()
-      throws SQLException
+   Connection getConnection() throws SQLException
    {
       if (con == null)
          throw new SQLException("Connection has been destroyed!!!");
@@ -323,7 +422,8 @@ public abstract class BaseWrapperManagedConnection
    {
       if (psCache != null)
       {
-         PreparedStatementCache.Key key = new PreparedStatementCache.Key(sql, PreparedStatementCache.Key.PREPARED_STATEMENT, resultSetType, resultSetConcurrency);
+         PreparedStatementCache.Key key = new PreparedStatementCache.Key(sql,
+               PreparedStatementCache.Key.PREPARED_STATEMENT, resultSetType, resultSetConcurrency);
          CachedPreparedStatement cachedps = (CachedPreparedStatement) psCache.get(key);
          if (cachedps != null)
          {
@@ -335,7 +435,7 @@ public abstract class BaseWrapperManagedConnection
          else
          {
             PreparedStatement ps = doPrepareStatement(sql, resultSetType, resultSetConcurrency);
-            cachedps = new CachedPreparedStatement(ps);
+            cachedps = wrappedConnectionFactory.createCachedPreparedStatement(ps);
             psCache.insert(key, cachedps);
          }
          return cachedps;
@@ -353,8 +453,9 @@ public abstract class BaseWrapperManagedConnection
    {
       if (psCache != null)
       {
-         PreparedStatementCache.Key key = new PreparedStatementCache.Key(sql, PreparedStatementCache.Key.CALLABLE_STATEMENT, resultSetType, resultSetConcurrency);
-         CachedCallableStatement cachedps = (CachedCallableStatement)psCache.get(key);
+         PreparedStatementCache.Key key = new PreparedStatementCache.Key(sql,
+               PreparedStatementCache.Key.CALLABLE_STATEMENT, resultSetType, resultSetConcurrency);
+         CachedCallableStatement cachedps = (CachedCallableStatement) psCache.get(key);
          if (cachedps != null)
          {
             if (canUse(cachedps))
@@ -365,7 +466,7 @@ public abstract class BaseWrapperManagedConnection
          else
          {
             CallableStatement cs = doPrepareCall(sql, resultSetType, resultSetConcurrency);
-            cachedps = new CachedCallableStatement(cs);
+            cachedps = wrappedConnectionFactory.createCachedCallableStatement(cs);
             psCache.insert(key, cachedps);
          }
          return cachedps;
@@ -378,29 +479,28 @@ public abstract class BaseWrapperManagedConnection
    {
       return con.prepareCall(sql, resultSetType, resultSetConcurrency);
    }
-   
+
    boolean canUse(CachedPreparedStatement cachedps)
    {
       // Nobody is using it so we are ok
       if (cachedps.isInUse() == false)
          return true;
-      
+
       // Cannot reuse prepared statements in auto commit mode
       // if will close the previous usage of the PS
       if (underlyingAutoCommit == true)
          return false;
-      
+
       // We have been told not to share
       return mcf.sharePS;
    }
-   
+
    protected Logger getLog()
    {
       return mcf.log;
    }
 
-   private void checkIdentity(Subject subject, ConnectionRequestInfo cri)
-      throws ResourceException
+   private void checkIdentity(Subject subject, ConnectionRequestInfo cri) throws ResourceException
    {
       Properties newProps = mcf.getConnectionProperties(subject, cri);
       if (!props.equals(newProps))
@@ -434,7 +534,7 @@ public abstract class BaseWrapperManagedConnection
       if (jdbcAutoCommit == false && inLocalTransaction.set(true) == false)
       {
          ArrayList copy;
-         synchronized(cels)
+         synchronized (cels)
          {
             copy = new ArrayList(cels);
          }
@@ -452,7 +552,7 @@ public abstract class BaseWrapperManagedConnection
             }
          }
       }
-      
+
       checkState();
    }
 
@@ -471,7 +571,7 @@ public abstract class BaseWrapperManagedConnection
 
    boolean isJdbcAutoCommit()
    {
-      return inManagedTransaction? false: jdbcAutoCommit;
+      return inManagedTransaction ? false : jdbcAutoCommit;
    }
 
    void setJdbcAutoCommit(final boolean jdbcAutoCommit) throws SQLException
@@ -486,7 +586,7 @@ public abstract class BaseWrapperManagedConnection
       if (jdbcAutoCommit && inLocalTransaction.set(false))
       {
          ArrayList copy;
-         synchronized(cels)
+         synchronized (cels)
          {
             copy = new ArrayList(cels);
          }
@@ -549,7 +649,7 @@ public abstract class BaseWrapperManagedConnection
       if (inLocalTransaction.set(false))
       {
          ArrayList copy;
-         synchronized(cels)
+         synchronized (cels)
          {
             copy = new ArrayList(cels);
          }
@@ -583,7 +683,7 @@ public abstract class BaseWrapperManagedConnection
       if (inLocalTransaction.set(false))
       {
          ArrayList copy;
-         synchronized(cels)
+         synchronized (cels)
          {
             copy = new ArrayList(cels);
          }
@@ -624,15 +724,23 @@ public abstract class BaseWrapperManagedConnection
    {
       return mcf.isTransactionQueryTimeout;
    }
-   
+
    int getQueryTimeout()
    {
       return mcf.getQueryTimeout();
    }
-   
+
    protected void checkException(SQLException e) throws ResourceException
    {
       connectionError(e);
       throw new JBossResourceException("SQLException", e);
    }
+   
+   public Object listConnectionStats()
+   {
+	   if(psCache != null)
+		   return psCache.toString();
+	   else
+		   return "-1"; //-1 indicates NoCache
+   }
 }
