@@ -30,6 +30,7 @@ import java.sql.Blob;
 import java.sql.CallableStatement;
 import java.sql.Clob;
 import java.sql.Date;
+import java.sql.PreparedStatement;
 import java.sql.Ref;
 import java.sql.SQLException;
 import java.sql.Time;
@@ -42,9 +43,9 @@ import java.util.Map;
  * 
  * @author <a href="mailto:d_jencks@users.sourceforge.net">David Jencks</a>
  * @author <a href="mailto:adrian@jboss.com">Adrian Brock</a>
- * @version $Revision: 57189 $
+ * @version $Revision: 75426 $
  */
-public class WrappedCallableStatement extends WrappedPreparedStatement implements CallableStatement
+public abstract class WrappedCallableStatement extends WrappedPreparedStatement implements CallableStatement
 {
    private final CallableStatement cs;
 
@@ -54,6 +55,11 @@ public class WrappedCallableStatement extends WrappedPreparedStatement implement
       this.cs = cs;
    }
 
+   public CallableStatement getUnderlyingStatement() throws SQLException
+   {
+      return (CallableStatement) super.getUnderlyingStatement();
+   }
+
    public Object getObject(int parameterIndex) throws SQLException
    {
       checkState();
@@ -1083,4 +1089,9 @@ public class WrappedCallableStatement extends WrappedPreparedStatement implement
          throw checkException(t);
       }
    }
+
+   protected CallableStatement getWrappedObject() throws SQLException
+   {
+      return (CallableStatement) super.getWrappedObject();
+   }
 }
