@@ -42,9 +42,9 @@ import org.jboss.util.NestedSQLException;
  * 
  * @author <a href="mailto:d_jencks@users.sourceforge.net">David Jencks</a>
  * @author <a href="mailto:adrian@jboss.com">Adrian Brock</a>
- * @version $Revision: 57189 $
+ * @version $Revision: 75426 $
  */
-public class WrappedConnection implements Connection
+public abstract class WrappedConnection extends JBossWrapper implements Connection
 {
    private static final Logger log = Logger.getLogger(WrappedConnection.class);
 
@@ -52,7 +52,7 @@ public class WrappedConnection implements Connection
 
    private WrapperDataSource dataSource;
    
-   private HashMap statements;
+   private HashMap<WrappedStatement, Throwable> statements;
 
    private boolean closed = false;
 
@@ -72,6 +72,24 @@ public class WrappedConnection implements Connection
          trackStatements = mc.getTrackStatements();
    }
 
+   protected void lock() throws SQLException
+   {
+      BaseWrapperManagedConnection mc = this.mc;
+      if (mc != null)
+         mc.tryLock();
+      else
+         throw new SQLException("Connection is not associated with a managed connection." + this);
+   }
+
+   protected void unlock()
+   {
+      BaseWrapperManagedConnection mc = this.mc;
+      if (mc != null)
+         mc.unlock();
+      // We recreate the lock when returned to the pool
+      // so missing the unlock after disassociation is not important
+   }
+
    public WrapperDataSource getDataSource()
    {
       return dataSource;
@@ -84,8 +102,16 @@ public class WrappedConnection implements Connection
    
    public void setReadOnly(boolean readOnly) throws SQLException
    {
-      checkStatus();
-      mc.setJdbcReadOnly(readOnly);
+      lock();
+      try
+      {
+         checkStatus();
+         mc.setJdbcReadOnly(readOnly);
+      }
+      finally
+      {
+         unlock();
+      }
    }
 
    public boolean isReadOnly() throws SQLException
@@ -105,13 +131,13 @@ public class WrappedConnection implements Connection
             {
                if (statements != null)
                {
-                  for (Iterator i = statements.entrySet().iterator(); i.hasNext(); )
+                  for (Iterator<Map.Entry<WrappedStatement, Throwable>> i = statements.entrySet().iterator(); i.hasNext(); )
                   {
-                     Map.Entry entry = (Map.Entry) i.next();
-                     WrappedStatement ws = (WrappedStatement) entry.getKey();
+                     Map.Entry<WrappedStatement, Throwable> entry = i.next();
+                     WrappedStatement ws = entry.getKey();
                      if (trackStatements == BaseWrapperManagedConnectionFactory.TRACK_STATEMENTS_TRUE_INT)
                      {
-                        Throwable stackTrace = (Throwable) entry.getValue();
+                        Throwable stackTrace = entry.getValue();
                         log.warn("Closing a statement you left open, please do your own housekeeping", stackTrace);
                      }
                      try
@@ -137,394 +163,676 @@ public class WrappedConnection implements Connection
       return closed;
    }
 
+   protected abstract WrappedStatement wrapStatement(Statement statement);
+   
    public Statement createStatement() throws SQLException
    {
-      checkTransaction();
+      lock();
       try
       {
-         return new WrappedStatement(this, mc.getConnection().createStatement());
+         checkTransaction();
+         try
+         {
+            return wrapStatement(mc.getConnection().createStatement());
+         }
+         catch (Throwable t)
+         {
+            throw checkException(t);
+         }
       }
-      catch (Throwable t)
+      finally
       {
-         throw checkException(t);
+         unlock();
       }
    }
 
    public Statement createStatement(int resultSetType, int resultSetConcurrency) throws SQLException
    {
-      checkTransaction();
+      lock();
       try
       {
-         return new WrappedStatement(this, mc.getConnection().createStatement(resultSetType, resultSetConcurrency));
+         checkTransaction();
+         try
+         {
+            return wrapStatement(mc.getConnection().createStatement(resultSetType, resultSetConcurrency));
+         }
+         catch (Throwable t)
+         {
+            throw checkException(t);
+         }
       }
-      catch (Throwable t)
+      finally
       {
-         throw checkException(t);
+         unlock();
       }
    }
 
    public Statement createStatement(int resultSetType, int resultSetConcurrency, int resultSetHoldability)
          throws SQLException
    {
-
-      checkTransaction();
+      lock();
       try
       {
-         return new WrappedStatement(this, mc.getConnection()
-               .createStatement(resultSetType, resultSetConcurrency, resultSetHoldability));
+         checkTransaction();
+         try
+         {
+            return wrapStatement(mc.getConnection()
+                  .createStatement(resultSetType, resultSetConcurrency, resultSetHoldability));
+         }
+         catch (Throwable t)
+         {
+            throw checkException(t);
+         }
       }
-      catch (Throwable t)
+      finally
       {
-         throw checkException(t);
+         unlock();
       }
    }
 
+   protected abstract WrappedPreparedStatement wrapPreparedStatement(PreparedStatement statement);
+
    public PreparedStatement prepareStatement(String sql) throws SQLException
    {
-      checkTransaction();
+      lock();
       try
       {
-         return new WrappedPreparedStatement(this, mc.prepareStatement(sql, ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY));
+         checkTransaction();
+         try
+         {
+            return wrapPreparedStatement(mc.prepareStatement(sql, ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY));
+         }
+         catch (Throwable t)
+         {
+            throw checkException(t);
+         }
       }
-      catch (Throwable t)
+      finally
       {
-         throw checkException(t);
+         unlock();
       }
    }
 
    public PreparedStatement prepareStatement(String sql, int resultSetType, int resultSetConcurrency)
          throws SQLException
    {
-      checkTransaction();
+      lock();
       try
       {
-         return new WrappedPreparedStatement(this, mc.prepareStatement(sql, resultSetType, resultSetConcurrency));
+         checkTransaction();
+         try
+         {
+            return wrapPreparedStatement(mc.prepareStatement(sql, resultSetType, resultSetConcurrency));
+         }
+         catch (Throwable t)
+         {
+            throw checkException(t);
+         }
       }
-      catch (Throwable t)
+      finally
       {
-         throw checkException(t);
+         unlock();
       }
    }
 
    public PreparedStatement prepareStatement(String sql, int resultSetType, int resultSetConcurrency,
          int resultSetHoldability) throws SQLException
    {
-      checkTransaction();
+      lock();
       try
       {
-         return new WrappedPreparedStatement(this, mc.getConnection()
-               .prepareStatement(sql, resultSetType, resultSetConcurrency, resultSetHoldability));
+         checkTransaction();
+         try
+         {
+            return wrapPreparedStatement(mc.getConnection()
+                  .prepareStatement(sql, resultSetType, resultSetConcurrency, resultSetHoldability));
+         }
+         catch (Throwable t)
+         {
+            throw checkException(t);
+         }
       }
-      catch (Throwable t)
+      finally
       {
-         throw checkException(t);
+         unlock();
       }
    }
 
    public PreparedStatement prepareStatement(String sql, int autoGeneratedKeys) throws SQLException
    {
-      checkTransaction();
+      lock();
       try
       {
-         return new WrappedPreparedStatement(this, mc.getConnection().prepareStatement(sql, autoGeneratedKeys));
+         checkTransaction();
+         try
+         {
+            return wrapPreparedStatement(mc.getConnection().prepareStatement(sql, autoGeneratedKeys));
+         }
+         catch (Throwable t)
+         {
+            throw checkException(t);
+         }
       }
-      catch (Throwable t)
+      finally
       {
-         throw checkException(t);
+         unlock();
       }
    }
 
    public PreparedStatement prepareStatement(String sql, int[] columnIndexes) throws SQLException
    {
-      checkTransaction();
+      lock();
       try
       {
-         return new WrappedPreparedStatement(this, mc.getConnection().prepareStatement(sql, columnIndexes));
+         checkTransaction();
+         try
+         {
+            return wrapPreparedStatement(mc.getConnection().prepareStatement(sql, columnIndexes));
+         }
+         catch (Throwable t)
+         {
+            throw checkException(t);
+         }
       }
-      catch (Throwable t)
+      finally
       {
-         throw checkException(t);
+         unlock();
       }
    }
 
    public PreparedStatement prepareStatement(String sql, String[] columnNames) throws SQLException
    {
-
-      checkTransaction();
+      lock();
       try
       {
-         return new WrappedPreparedStatement(this, mc.getConnection().prepareStatement(sql, columnNames));
+         checkTransaction();
+         try
+         {
+            return wrapPreparedStatement(mc.getConnection().prepareStatement(sql, columnNames));
+         }
+         catch (Throwable t)
+         {
+            throw checkException(t);
+         }
       }
-      catch (Throwable t)
+      finally
       {
-         throw checkException(t);
+         unlock();
       }
    }
 
+   protected abstract WrappedCallableStatement wrapCallableStatement(CallableStatement statement);
+
    public CallableStatement prepareCall(String sql) throws SQLException
    {
-      checkTransaction();
+      lock();
       try
       {
-         return new WrappedCallableStatement(this, mc.prepareCall(sql, ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY));
+         checkTransaction();
+         try
+         {
+            return wrapCallableStatement(mc.prepareCall(sql, ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY));
+         }
+         catch (Throwable t)
+         {
+            throw checkException(t);
+         }
       }
-      catch (Throwable t)
+      finally
       {
-         throw checkException(t);
+         unlock();
       }
    }
 
    public CallableStatement prepareCall(String sql, int resultSetType, int resultSetConcurrency) throws SQLException
    {
-      checkTransaction();
+      lock();
       try
       {
-         return new WrappedCallableStatement(this, mc.prepareCall(sql, resultSetType, resultSetConcurrency));
+         checkTransaction();
+         try
+         {
+            return wrapCallableStatement(mc.prepareCall(sql, resultSetType, resultSetConcurrency));
+         }
+         catch (Throwable t)
+         {
+            throw checkException(t);
+         }
       }
-      catch (Throwable t)
+      finally
       {
-         throw checkException(t);
+         unlock();
       }
    }
 
    public CallableStatement prepareCall(String sql, int resultSetType, int resultSetConcurrency,
          int resultSetHoldability) throws SQLException
    {
-
-      checkTransaction();
+      lock();
       try
       {
-         return new WrappedCallableStatement(this, mc.getConnection()
-               .prepareCall(sql, resultSetType, resultSetConcurrency, resultSetHoldability));
+         checkTransaction();
+         try
+         {
+            return wrapCallableStatement(mc.getConnection()
+                  .prepareCall(sql, resultSetType, resultSetConcurrency, resultSetHoldability));
+         }
+         catch (Throwable t)
+         {
+            throw checkException(t);
+         }
       }
-      catch (Throwable t)
+      finally
       {
-         throw checkException(t);
+         unlock();
       }
    }
 
    public String nativeSQL(String sql) throws SQLException
    {
-      checkTransaction();
+      lock();
       try
       {
-         return mc.getConnection().nativeSQL(sql);
+         checkTransaction();
+         try
+         {
+            return mc.getConnection().nativeSQL(sql);
+         }
+         catch (Throwable t)
+         {
+            throw checkException(t);
+         }
       }
-      catch (Throwable t)
+      finally
       {
-         throw checkException(t);
+         unlock();
       }
    }
 
    public void setAutoCommit(boolean autocommit) throws SQLException
    {
-      checkStatus();
-      mc.setJdbcAutoCommit(autocommit);
+      lock();
+      try
+      {
+         checkStatus();
+         mc.setJdbcAutoCommit(autocommit);
+      }
+      finally
+      {
+         unlock();
+      }
    }
 
    public boolean getAutoCommit() throws SQLException
    {
-      checkStatus();
-      return mc.isJdbcAutoCommit();
+      lock();
+      try
+      {
+         checkStatus();
+         return mc.isJdbcAutoCommit();
+      }
+      finally
+      {
+         unlock();
+      }
    }
 
    public void commit() throws SQLException
    {
-      checkTransaction();
-      mc.jdbcCommit();
+      lock();
+      try
+      {
+         checkTransaction();
+         mc.jdbcCommit();
+      }
+      finally
+      {
+         unlock();
+      }
    }
 
    public void rollback() throws SQLException
    {
-      checkTransaction();
-      mc.jdbcRollback();
+      lock();
+      try
+      {
+         checkTransaction();
+         mc.jdbcRollback();
+      }
+      finally
+      {
+         unlock();
+      }
    }
 
    public void rollback(Savepoint savepoint) throws SQLException
    {
-      checkTransaction();
-      mc.jdbcRollback(savepoint);
+      lock();
+      try
+      {
+         checkTransaction();
+         mc.jdbcRollback(savepoint);
+      }
+      finally
+      {
+         unlock();
+      }
    }
 
    public DatabaseMetaData getMetaData() throws SQLException
    {
-      checkTransaction();
+      lock();
       try
       {
-         return mc.getConnection().getMetaData();
+         checkTransaction();
+         try
+         {
+            return mc.getConnection().getMetaData();
+         }
+         catch (Throwable t)
+         {
+            throw checkException(t);
+         }
       }
-      catch (Throwable t)
+      finally
       {
-         throw checkException(t);
+         unlock();
       }
    }
 
    public void setCatalog(String catalog) throws SQLException
    {
-      checkTransaction();
+      lock();
       try
       {
-         mc.getConnection().setCatalog(catalog);
+         checkTransaction();
+         try
+         {
+            mc.getConnection().setCatalog(catalog);
+         }
+         catch (Throwable t)
+         {
+            throw checkException(t);
+         }
       }
-      catch (Throwable t)
+      finally
       {
-         throw checkException(t);
+         unlock();
       }
    }
+
    public String getCatalog() throws SQLException
    {
-      checkTransaction();
+      lock();
       try
       {
-         return mc.getConnection().getCatalog();
+         checkTransaction();
+         try
+         {
+            return mc.getConnection().getCatalog();
+         }
+         catch (Throwable t)
+         {
+            throw checkException(t);
+         }
       }
-      catch (Throwable t)
+      finally
       {
-         throw checkException(t);
+         unlock();
       }
    }
 
    public void setTransactionIsolation(int isolationLevel) throws SQLException
    {
-      checkStatus();
-      mc.setJdbcTransactionIsolation(isolationLevel);
+      lock();
+      try
+      {
+         checkStatus();
+         mc.setJdbcTransactionIsolation(isolationLevel);
+      }
+      finally
+      {
+         unlock();
+      }
    }
 
    public int getTransactionIsolation() throws SQLException
    {
-      checkStatus();
-      return mc.getJdbcTransactionIsolation();
+      lock();
+      try
+      {
+         checkStatus();
+         return mc.getJdbcTransactionIsolation();
+      }
+      finally
+      {
+         unlock();
+      }
    }
 
    public SQLWarning getWarnings() throws SQLException
    {
-      checkTransaction();
+      lock();
       try
       {
-         return mc.getConnection().getWarnings();
+         checkTransaction();
+         try
+         {
+            return mc.getConnection().getWarnings();
+         }
+         catch (Throwable t)
+         {
+            throw checkException(t);
+         }
       }
-      catch (Throwable t)
+      finally
       {
-         throw checkException(t);
+         unlock();
       }
    }
 
    public void clearWarnings() throws SQLException
    {
-      checkTransaction();
+      lock();
       try
       {
-         mc.getConnection().clearWarnings();
+         checkTransaction();
+         try
+         {
+            mc.getConnection().clearWarnings();
+         }
+         catch (Throwable t)
+         {
+            throw checkException(t);
+         }
       }
-      catch (Throwable t)
+      finally
       {
-         throw checkException(t);
+         unlock();
       }
    }
 
+   @SuppressWarnings("unchecked")
    public Map getTypeMap() throws SQLException
    {
-      checkTransaction();
+      lock();
       try
       {
-         return mc.getConnection().getTypeMap();
+         checkTransaction();
+         try
+         {
+            return mc.getConnection().getTypeMap();
+         }
+         catch (Throwable t)
+         {
+            throw checkException(t);
+         }
       }
-      catch (Throwable t)
+      finally
       {
-         throw checkException(t);
+         unlock();
       }
    }
 
+   @SuppressWarnings("unchecked")
    public void setTypeMap(Map typeMap) throws SQLException
    {
-      checkTransaction();
+      lock();
       try
       {
-         mc.getConnection().setTypeMap(typeMap);
+         checkTransaction();
+         try
+         {
+            mc.getConnection().setTypeMap(typeMap);
+         }
+         catch (Throwable t)
+         {
+            throw checkException(t);
+         }
       }
-      catch (Throwable t)
+      finally
       {
-         throw checkException(t);
+         unlock();
       }
    }
 
    public void setHoldability(int holdability) throws SQLException
    {
-      checkTransaction();
+      lock();
       try
       {
-         mc.getConnection().setHoldability(holdability);
+         checkTransaction();
+         try
+         {
+            mc.getConnection().setHoldability(holdability);
+         }
+         catch (Throwable t)
+         {
+            throw checkException(t);
+         }
       }
-      catch (Throwable t)
+      finally
       {
-         throw checkException(t);
+         unlock();
       }
    }
 
    public int getHoldability() throws SQLException
    {
-      checkTransaction();
+      lock();
       try
       {
-         return mc.getConnection().getHoldability();
+         checkTransaction();
+         try
+         {
+            return mc.getConnection().getHoldability();
+         }
+         catch (Throwable t)
+         {
+            throw checkException(t);
+         }
       }
-      catch (Throwable t)
+      finally
       {
-         throw checkException(t);
+         unlock();
       }
    }
 
    public Savepoint setSavepoint() throws SQLException
    {
-      checkTransaction();
+      lock();
       try
       {
-         return mc.getConnection().setSavepoint();
+         checkTransaction();
+         try
+         {
+            return mc.getConnection().setSavepoint();
+         }
+         catch (Throwable t)
+         {
+            throw checkException(t);
+         }
       }
-      catch (Throwable t)
+      finally
       {
-         throw checkException(t);
+         unlock();
       }
    }
 
    public Savepoint setSavepoint(String name) throws SQLException
    {
-      checkTransaction();
+      lock();
       try
       {
-         return mc.getConnection().setSavepoint(name);
+         checkTransaction();
+         try
+         {
+            return mc.getConnection().setSavepoint(name);
+         }
+         catch (Throwable t)
+         {
+            throw checkException(t);
+         }
       }
-      catch (Throwable t)
+      finally
       {
-         throw checkException(t);
+         unlock();
       }
    }
 
    public void releaseSavepoint(Savepoint savepoint) throws SQLException
    {
-      checkTransaction();
+      lock();
       try
       {
-         mc.getConnection().releaseSavepoint(savepoint);
+         checkTransaction();
+         try
+         {
+            mc.getConnection().releaseSavepoint(savepoint);
+         }
+         catch (Throwable t)
+         {
+            throw checkException(t);
+         }
       }
-      catch (Throwable t)
+      finally
       {
-         throw checkException(t);
+         unlock();
       }
    }
 
    public Connection getUnderlyingConnection() throws SQLException
    {
-      checkTransaction();
-      return mc.getConnection();
+      lock();
+      try
+      {
+         checkTransaction();
+         return mc.getConnection();
+      }
+      finally
+      {
+         unlock();
+      }
    }
 
-   void checkTransaction() throws SQLException
+   protected Connection getWrappedObject() throws SQLException
+   {
+      return getUnderlyingConnection();
+   }
+
+   protected void checkTransaction() throws SQLException
    {
       checkStatus();
       mc.checkTransaction();
    }
 
+   void checkTransactionActive() throws SQLException
+   {
+      if (dataSource == null)
+         return;
+      dataSource.checkTransactionActive();
+   }
+
    /**
     * The checkStatus method checks that the handle has not been closed and
     * that it is associated with a managed connection.
@@ -537,6 +845,7 @@ public class WrappedConnection implements Connection
          throw new SQLException("Connection handle has been closed and is unusable");
       if (mc == null)
          throw new SQLException("Connection handle is not currently associated with a ManagedConnection");
+      checkTransactionActive();
    }
 
    /**
@@ -544,17 +853,26 @@ public class WrappedConnection implements Connection
     * the ManagedConnection of the error. Subclasses may override this to
     * filter exceptions based on their severity.
     * 
-    * @param e a <code>SQLException</code> value
-    * @exception Exception if an error occurs
+    * @param t a throwable
+    * @return the sql exception
+    * @exception SQLException if an error occurs
     */
    protected SQLException checkException(Throwable t) throws SQLException
    {
+      Throwable result = null;
+
       if (mc != null)
-         mc.connectionError(t);
-      if (t instanceof SQLException)
-         throw (SQLException) t;
+         result = mc.connectionError(t);
+
+      if (result instanceof SQLException)
+      {
+         throw (SQLException) result;
+      }
       else
-         throw new NestedSQLException("Error", t);
+      {
+         throw new NestedSQLException("Error", result);
+      }
+      
    }
 
    int getTrackStatements()
@@ -570,7 +888,7 @@ public class WrappedConnection implements Connection
       synchronized (this)
       {
          if (statements == null)
-            statements = new HashMap();
+            statements = new HashMap<WrappedStatement, Throwable>();
          
          if (trackStatements == BaseWrapperManagedConnectionFactory.TRACK_STATEMENTS_TRUE_INT)
             statements.put(ws, new Throwable("STACKTRACE"));
