@@ -39,15 +39,15 @@ import java.sql.Time;
 import java.sql.Timestamp;
 import java.util.Calendar;
 import java.util.Map;
+import java.util.concurrent.atomic.AtomicBoolean;
 
 /**
  * A wrapper for a result set
  * 
  * @author <a href="mailto:adrian@jboss.com">Adrian Brock</a>
- * @author <a href="weston.price@jboss.com">Weston Price</a>
- * @version $Revision: 57189 $
+ * @version $Revision: 75426 $
  */
-public class WrappedResultSet implements ResultSet
+public abstract class WrappedResultSet extends JBossWrapper implements ResultSet
 {
    /** The wrapped statement */
    private WrappedStatement statement;
@@ -56,10 +56,7 @@ public class WrappedResultSet implements ResultSet
    private ResultSet resultSet;
 
    /** Whether we are closed */
-   private boolean closed = false;
-
-   /** The state lock */
-   private Object lock = new Object();
+   private AtomicBoolean closed = new AtomicBoolean(false);
    
    /** 
     * Create a new wrapped result set
@@ -102,8 +99,16 @@ public class WrappedResultSet implements ResultSet
 
    public ResultSet getUnderlyingResultSet() throws SQLException
    {
-      checkState();
-      return resultSet;
+      statement.lock();
+      try
+      {
+         checkTransaction();
+         return resultSet;
+      }
+      finally
+      {
+         statement.unlock();
+      }
    }
 
    public boolean absolute(int row) throws SQLException
@@ -173,27 +178,32 @@ public class WrappedResultSet implements ResultSet
 
    public void close() throws SQLException
    {
-      synchronized (lock)
-      {
-         if (closed)
-           return;
-         closed = true;
-      
-      }
+      if (closed.get())
+         return;
+ 
+      closed.set(true);
       statement.unregisterResultSet(this);
       internalClose();
    }
 
    public void deleteRow() throws SQLException
    {
-      checkState();
+      statement.lock();
       try
       {
-         resultSet.deleteRow();
+         checkTransaction();
+         try
+         {
+            resultSet.deleteRow();
+         }
+         catch (Throwable t)
+         {
+            throw checkException(t);
+         }
       }
-      catch (Throwable t)
+      finally
       {
-         throw checkException(t);
+         statement.unlock();
       }
    }
 
@@ -288,9 +298,7 @@ public class WrappedResultSet implements ResultSet
       }
    }
 
-   /**
-    * @deprecated
-    */
+   @Deprecated
    public BigDecimal getBigDecimal(int columnIndex, int scale) throws SQLException
    {
       checkState();
@@ -317,9 +325,7 @@ public class WrappedResultSet implements ResultSet
       }
    }
 
-   /**
-    * @deprecated
-    */
+   @Deprecated
    public BigDecimal getBigDecimal(String columnName, int scale) throws SQLException
    {
       checkState();
@@ -749,6 +755,7 @@ public class WrappedResultSet implements ResultSet
       }
    }
 
+   @SuppressWarnings("unchecked")
    public Object getObject(int i, Map map) throws SQLException
    {
       checkState();
@@ -775,6 +782,7 @@ public class WrappedResultSet implements ResultSet
       }
    }
 
+   @SuppressWarnings("unchecked")
    public Object getObject(String colName, Map map) throws SQLException
    {
       checkState();
@@ -1002,9 +1010,7 @@ public class WrappedResultSet implements ResultSet
       }
    }
 
-   /**
-    * @deprecated
-    */
+   @Deprecated
    public InputStream getUnicodeStream(int columnIndex) throws SQLException
    {
       checkState();
@@ -1018,9 +1024,7 @@ public class WrappedResultSet implements ResultSet
       }
    }
 
-   /**
-    * @deprecated
-    */
+   @Deprecated
    public InputStream getUnicodeStream(String columnName) throws SQLException
    {
       try
@@ -1074,14 +1078,22 @@ public class WrappedResultSet implements ResultSet
 
    public void insertRow() throws SQLException
    {
-      checkState();
+      statement.lock();
       try
       {
-         resultSet.insertRow();
+         checkTransaction();
+         try
+         {
+            resultSet.insertRow();
+         }
+         catch (Throwable t)
+         {
+            throw checkException(t);
+         }
       }
-      catch (Throwable t)
+      finally
       {
-         throw checkException(t);
+         statement.unlock();
       }
    }
 
@@ -1789,14 +1801,22 @@ public class WrappedResultSet implements ResultSet
 
    public void updateRow() throws SQLException
    {
-      checkState();
+      statement.lock();
       try
       {
-         resultSet.updateRow();
+         checkTransaction();
+         try
+         {
+            resultSet.updateRow();
+         }
+         catch (Throwable t)
+         {
+            throw checkException(t);
+         }
       }
-      catch (Throwable t)
+      finally
       {
-         throw checkException(t);
+         statement.unlock();
       }
    }
 
@@ -1917,26 +1937,31 @@ public class WrappedResultSet implements ResultSet
       }
    }
 
-   SQLException checkException(Throwable t) throws SQLException
+   protected ResultSet getWrappedObject() throws SQLException
+   {
+      return getUnderlyingResultSet();
+   }
+
+   protected SQLException checkException(Throwable t) throws SQLException
    {
       throw statement.checkException(t);
    }
    
    void internalClose() throws SQLException
    {
-      synchronized (lock)
-      {
-         closed = true;
-      }
+      closed.set(true);
       resultSet.close();
    }
 
    void checkState() throws SQLException
    {
-      synchronized (lock)
-      {
-         if (closed)
-            throw new SQLException("The result set is closed.");
-      }
+      if (closed.get())
+        throw new SQLException("The result set is closed.");
    }
-}
\ No newline at end of file
+
+   protected void checkTransaction() throws SQLException
+   {
+      checkState();
+      statement.checkTransactionActive();
+   }
+}
