@@ -27,12 +27,15 @@ import java.util.HashSet;
 import java.util.Iterator;
 import java.util.Set;
 import java.util.Vector;
+import java.util.concurrent.TimeUnit;
+import java.util.concurrent.locks.ReentrantLock;
 
 import javax.jms.Connection;
 import javax.jms.ExceptionListener;
 import javax.jms.JMSException;
 import javax.jms.QueueConnection;
 import javax.jms.QueueSession;
+import javax.jms.ResourceAllocationException;
 import javax.jms.Session;
 import javax.jms.TopicConnection;
 import javax.jms.TopicSession;
@@ -128,7 +131,7 @@ import org.jboss.resource.JBossResourceException;
  * @author <a href="mailto:peter.antman@tim.se">Peter Antman</a>.
  * @author <a href="mailto:jason@planet57.com">Jason Dillon</a>
  * @author <a href="mailto:adrian@jboss.com">Adrian Brock</a>
- * @version $Revision: 57189 $
+ * @version $Revision: 73442 $
  */
 public class JmsManagedConnection
    implements ManagedConnection, ExceptionListener
@@ -141,6 +144,8 @@ public class JmsManagedConnection
    private String pwd;
    private boolean isDestroyed;
 
+   private ReentrantLock lock = new ReentrantLock();
+   
    // Physical JMS connection stuff
    private Connection con;
    private Session session;
@@ -181,7 +186,21 @@ public class JmsManagedConnection
       this.user = user;
       this.pwd = pwd;
 
-      setup();
+      try
+      {
+         setup();
+      }
+      catch (Throwable t)
+      {
+         try
+         {
+            destroy();
+         }
+         catch (Throwable ignored)
+         {
+         }
+         JBossResourceException.rethrowAsResourceException("Error creating connection/session", t);
+      }
    }
 
    //---- ManagedConnection API ----
@@ -264,7 +283,7 @@ public class JmsManagedConnection
     */
    public void destroy() throws ResourceException
    {
-      if (isDestroyed) return;
+      if (isDestroyed || con == null) return;
 
       isDestroyed = true;
 
@@ -287,21 +306,24 @@ public class JmsManagedConnection
          {
             if (info.getType() == JmsConnectionFactory.TOPIC)
             {
-               topicSession.close();
-               if (xaTransacted) {
+               if (topicSession != null)
+                  topicSession.close();
+               if (xaTransacted && xaTopicSession != null) {
                   xaTopicSession.close();
                }
             }
             else if (info.getType() == JmsConnectionFactory.QUEUE)
             {
-               queueSession.close();
-               if (xaTransacted)
+               if (queueSession != null)
+                  queueSession.close();
+               if (xaTransacted && xaQueueSession != null)
                   xaQueueSession.close();
             }
             else
             {
-               session.close();
-               if (xaTransacted)
+               if (session != null)
+                  session.close();
+               if (xaTransacted && xaSession != null)
                   xaSession.close();
             }
          }
@@ -311,10 +333,9 @@ public class JmsManagedConnection
          }
          con.close();
       }
-      catch (JMSException e)
+      catch (Throwable e)
       {
-         throw new JBossResourceException
-            ("Could not properly close the session and connection", e);
+         throw new JBossResourceException("Could not properly close the session and connection", e);
       }
    }
 
@@ -332,6 +353,12 @@ public class JmsManagedConnection
 
       // destory handles
       destroyHandles();
+
+      // I'm recreating the lock object when we return to the pool
+      // because it looks too nasty to expect the connection handle
+      // to unlock properly in certain race conditions
+      // where the dissociation of the managed connection is "random".
+      lock = new ReentrantLock();
    }
 
    /**
@@ -360,6 +387,35 @@ public class JmsManagedConnection
             ("ManagedConnection in an illegal state");
    }
 
+   protected void lock()
+   {
+      lock.lock();
+   }
+
+   protected void tryLock() throws JMSException
+   {
+      int tryLock = mcf.getUseTryLock();
+      if (tryLock <= 0)
+      {
+         lock();
+         return;
+      }
+      try
+      {
+         if (lock.tryLock(tryLock, TimeUnit.SECONDS) == false)
+            throw new ResourceAllocationException("Unable to obtain lock in " + tryLock + " seconds: " + this);
+      }
+      catch (InterruptedException e)
+      {
+         throw new ResourceAllocationException("Interrupted attempting lock: " + this);
+      }
+   }
+   
+   protected void unlock()
+   {
+      lock.unlock();
+   }
+
    /**
     * Add a connection event listener.
     *
@@ -412,6 +468,7 @@ public class JmsManagedConnection
       if (log.isTraceEnabled())
          log.trace("XAResource=" + xaResource);
 
+      xaResource = new JmsXAResource(this, xaResource);
       return xaResource;
    }
 
