@@ -22,6 +22,7 @@
 package org.jboss.resource.adapter.jms;
 
 import java.io.Serializable;
+import java.sql.SQLException;
 import java.util.HashSet;
 import java.util.Iterator;
 
@@ -60,7 +61,7 @@ import org.jboss.logging.Logger;
  * @author <a href="mailto:peter.antman@tim.se">Peter Antman </a>.
  * @author <a href="mailto:jason@planet57.com">Jason Dillon </a>.
  * @author <a href="mailto:adrian@jboss.com">Adrian Brock</a>
- * @version $Revision: 63350 $
+ * @version $Revision: 73002 $
  */
 public class JmsSession implements Session, QueueSession, TopicSession
 {
@@ -73,7 +74,7 @@ public class JmsSession implements Session, QueueSession, TopicSession
    private JmsConnectionRequestInfo info;
 
    /** The session factory for this session */
-   private JmsSessionFactory sf;
+   private JmsSessionFactoryImpl sf;
    
    /** The message consumers */
    private HashSet consumers = new HashSet();
@@ -97,11 +98,29 @@ public class JmsSession implements Session, QueueSession, TopicSession
          log.trace("new JmsSession " + this + " mc=" + mc + " cri=" + info);
    }
 
-   public void setJmsSessionFactory(JmsSessionFactory sf)
+   public void setJmsSessionFactory(JmsSessionFactoryImpl sf)
    {
       this.sf = sf;
    }
    
+   protected void lock() throws JMSException
+   {
+      JmsManagedConnection mc = this.mc;
+      if (mc != null)
+         mc.tryLock();
+      else
+         throw new IllegalStateException("Connection is not associated with a managed connection. " + this);
+   }
+
+   protected void unlock()
+   {
+      JmsManagedConnection mc = this.mc;
+      if (mc != null)
+         mc.unlock();
+      // We recreate the lock when returned to the pool
+      // so missing the unlock after disassociation is not important
+   }
+   
    /**
     * Ensure that the session is opened.
     * 
@@ -115,12 +134,20 @@ public class JmsSession implements Session, QueueSession, TopicSession
       if (mc == null)
          throw new IllegalStateException("The session is closed");
       
+      checkTransactionActive();
+      
       Session session = mc.getSession();
       if (trace)
          log.trace("getSession " + session + " for " + this);
       return session;
    }
 
+   void checkTransactionActive() throws IllegalStateException
+   {
+      if (sf != null)
+         sf.checkTransactionActive();
+   }
+   
    // ---- Session API
 
    public BytesMessage createBytesMessage() throws JMSException
@@ -240,32 +267,56 @@ public class JmsSession implements Session, QueueSession, TopicSession
    // FIXME - is this really OK, probably not
    public void commit() throws JMSException
    {
-      Session session = getSession();
-      if (info.isTransacted() == false)
-         throw new IllegalStateException("Session is not transacted");
-      if (trace)
-         log.trace("Commit session " + this);
-      session.commit();
+      lock();
+      try
+      {
+         Session session = getSession();
+         if (info.isTransacted() == false)
+            throw new IllegalStateException("Session is not transacted");
+         if (trace)
+            log.trace("Commit session " + this);
+         session.commit();
+      }
+      finally
+      {
+         unlock();
+      }
    }
 
    public void rollback() throws JMSException
    {
-      Session session = getSession();
-      if (info.isTransacted() == false)
-         throw new IllegalStateException("Session is not transacted");
-      if (trace)
-         log.trace("Rollback session " + this);
-      session.rollback();
+      lock();
+      try
+      {
+         Session session = getSession();
+         if (info.isTransacted() == false)
+            throw new IllegalStateException("Session is not transacted");
+         if (trace)
+            log.trace("Rollback session " + this);
+         session.rollback();
+      }
+      finally
+      {
+         unlock();
+      }
    }
 
    public void recover() throws JMSException
    {
-      Session session = getSession();
-      if (info.isTransacted())
-         throw new IllegalStateException("Session is transacted");
-      if (trace)
-         log.trace("Recover session " + this);
-      session.recover();
+      lock();
+      try
+      {
+         Session session = getSession();
+         if (info.isTransacted())
+            throw new IllegalStateException("Session is transacted");
+         if (trace)
+            log.trace("Recover session " + this);
+         session.recover();
+      }
+      finally
+      {
+         unlock();
+      }
    }
 
    // --- TopicSession API
@@ -288,28 +339,44 @@ public class JmsSession implements Session, QueueSession, TopicSession
 
    public TopicSubscriber createSubscriber(Topic topic) throws JMSException
    {
-      TopicSession session = getTopicSession();
-      if (trace)
-         log.trace("createSubscriber " + session + " topic=" + topic);
-      TopicSubscriber result = session.createSubscriber(topic);
-      result = new JmsTopicSubscriber(result, this);
-      if (trace)
-         log.trace("createdSubscriber " + session + " JmsTopicSubscriber=" + result);
-      addConsumer(result);
-      return result;
+      lock();
+      try
+      {
+         TopicSession session = getTopicSession();
+         if (trace)
+            log.trace("createSubscriber " + session + " topic=" + topic);
+         TopicSubscriber result = session.createSubscriber(topic);
+         result = new JmsTopicSubscriber(result, this);
+         if (trace)
+            log.trace("createdSubscriber " + session + " JmsTopicSubscriber=" + result);
+         addConsumer(result);
+         return result;
+      }
+      finally
+      {
+         unlock();
+      }
    }
 
    public TopicSubscriber createSubscriber(Topic topic, String messageSelector, boolean noLocal) throws JMSException
    {
-      TopicSession session = getTopicSession();
-      if (trace)
-         log.trace("createSubscriber " + session + " topic=" + topic + " selector=" + messageSelector + " noLocal=" + noLocal);
-      TopicSubscriber result = session.createSubscriber(topic, messageSelector, noLocal);
-      result = new JmsTopicSubscriber(result, this);
-      if (trace)
-         log.trace("createdSubscriber " + session + " JmsTopicSubscriber=" + result);
-      addConsumer(result);
-      return result;
+      lock();
+      try
+      {
+         TopicSession session = getTopicSession();
+         if (trace)
+            log.trace("createSubscriber " + session + " topic=" + topic + " selector=" + messageSelector + " noLocal=" + noLocal);
+         TopicSubscriber result = session.createSubscriber(topic, messageSelector, noLocal);
+         result = new JmsTopicSubscriber(result, this);
+         if (trace)
+            log.trace("createdSubscriber " + session + " JmsTopicSubscriber=" + result);
+         addConsumer(result);
+         return result;
+      }
+      finally
+      {
+         unlock();
+      }
    }
 
    public TopicSubscriber createDurableSubscriber(Topic topic, String name) throws JMSException
@@ -319,41 +386,66 @@ public class JmsSession implements Session, QueueSession, TopicSession
          throw new IllegalStateException("Cannot create durable subscriber from javax.jms.QueueSession");         
       }
       
-      Session session = getSession();
-      if (trace)
-         log.trace("createDurableSubscriber " + session + " topic=" + topic + " name=" + name);
-      TopicSubscriber result = session.createDurableSubscriber(topic, name);
-      result = new JmsTopicSubscriber(result, this);
-      if (trace)
-         log.trace("createdDurableSubscriber " + session + " JmsTopicSubscriber=" + result);
-      addConsumer(result);
-      return result;
+      lock();
+      try
+      {
+         Session session = getSession();
+         if (trace)
+            log.trace("createDurableSubscriber " + session + " topic=" + topic + " name=" + name);
+         TopicSubscriber result = session.createDurableSubscriber(topic, name);
+         result = new JmsTopicSubscriber(result, this);
+         if (trace)
+            log.trace("createdDurableSubscriber " + session + " JmsTopicSubscriber=" + result);
+         addConsumer(result);
+         return result;
+      }
+      finally
+      {
+         unlock();
+      }
    }
 
    public TopicSubscriber createDurableSubscriber(Topic topic, String name, String messageSelector, boolean noLocal)
          throws JMSException
    {
-      Session session = getSession();
-      if (trace)
-         log.trace("createDurableSubscriber " + session + " topic=" + topic + " name=" + name + " selector=" + messageSelector + " noLocal=" + noLocal);
-      TopicSubscriber result = session.createDurableSubscriber(topic, name, messageSelector, noLocal);
-      result = new JmsTopicSubscriber(result, this);
-      if (trace)
-         log.trace("createdDurableSubscriber " + session + " JmsTopicSubscriber=" + result);
-      addConsumer(result);
-      return result;
+      lock();
+      try
+      {
+         Session session = getSession();
+         if (trace)
+            log.trace("createDurableSubscriber " + session + " topic=" + topic + " name=" + name + " selector=" + messageSelector + " noLocal=" + noLocal);
+         TopicSubscriber result = session.createDurableSubscriber(topic, name, messageSelector, noLocal);
+         result = new JmsTopicSubscriber(result, this);
+         if (trace)
+            log.trace("createdDurableSubscriber " + session + " JmsTopicSubscriber=" + result);
+         addConsumer(result);
+         return result;
+      }
+      finally
+      {
+         unlock();
+      }
    }
 
    public TopicPublisher createPublisher(Topic topic) throws JMSException
    {
-      TopicSession session = getTopicSession();
-      if (trace)
-         log.trace("createPublisher " + session + " topic=" + topic);
-      TopicPublisher result = session.createPublisher(topic);
-      if (trace)
-         log.trace("createdPublisher " + session + " publisher=" + result);
-      addProducer(result);
-      return result;
+      lock();
+      try
+      {
+         TopicSession session = getTopicSession();
+         if (trace)
+            log.trace("createPublisher " + session + " topic=" + topic);
+         TopicPublisher result = session.createPublisher(topic);
+         result = new JmsTopicPublisher(result, this);
+         if (trace)
+            log.trace("createdPublisher " + session + " publisher=" + result);
+         addProducer(result);
+         return result;
+      }
+      finally
+      {
+         unlock();
+      }
    }
 
    public TemporaryTopic createTemporaryTopic() throws JMSException
@@ -363,14 +455,22 @@ public class JmsSession implements Session, QueueSession, TopicSession
          throw new IllegalStateException("Cannot create temporary topic for javax.jms.QueueSession");         
       }
       
-      Session session = getSession();
-      if (trace)
-         log.trace("createTemporaryTopic " + session);
-      TemporaryTopic temp = session.createTemporaryTopic();
-      if (trace)
-         log.trace("createdTemporaryTopic " + session + " temp=" + temp);
-      sf.addTemporaryTopic(temp);
-      return temp;
+      lock();
+      try
+      {
+         Session session = getSession();
+         if (trace)
+            log.trace("createTemporaryTopic " + session);
+         TemporaryTopic temp = session.createTemporaryTopic();
+         if (trace)
+            log.trace("createdTemporaryTopic " + session + " temp=" + temp);
+         sf.addTemporaryTopic(temp);
+         return temp;
+      }
+      finally
+      {
+         unlock();
+      }
    }
 
    public void unsubscribe(String name) throws JMSException
@@ -380,10 +480,18 @@ public class JmsSession implements Session, QueueSession, TopicSession
          throw new IllegalStateException("Cannot unsubscribe for javax.jms.QueueSession");         
       }
 
-      Session session = getSession();
-      if (trace)
-         log.trace("unsubscribe " + session + " name=" + name);
-      session.unsubscribe(name);
+      lock();
+      try
+      {
+         Session session = getSession();
+         if (trace)
+            log.trace("unsubscribe " + session + " name=" + name);
+         session.unsubscribe(name);
+      }
+      finally
+      {
+         unlock();
+      }
    }
 
    //--- QueueSession API
@@ -436,40 +544,65 @@ public class JmsSession implements Session, QueueSession, TopicSession
 
    public QueueReceiver createReceiver(Queue queue) throws JMSException
    {
-      QueueSession session = getQueueSession();
-      if (trace)
-         log.trace("createReceiver " + session + " queue=" + queue);
-      QueueReceiver result = session.createReceiver(queue);
-      result = new JmsQueueReceiver(result, this);
-      if (trace)
-         log.trace("createdReceiver " + session + " receiver=" + result);
-      addConsumer(result);
-      return result;
+      lock();
+      try
+      {
+         QueueSession session = getQueueSession();
+         if (trace)
+            log.trace("createReceiver " + session + " queue=" + queue);
+         QueueReceiver result = session.createReceiver(queue);
+         result = new JmsQueueReceiver(result, this);
+         if (trace)
+            log.trace("createdReceiver " + session + " receiver=" + result);
+         addConsumer(result);
+         return result;
+      }
+      finally
+      {
+         unlock();
+      }
    }
 
    public QueueReceiver createReceiver(Queue queue, String messageSelector) throws JMSException
    {
-      QueueSession session = getQueueSession();
-      if (trace)
-         log.trace("createReceiver " + session + " queue=" + queue + " selector=" + messageSelector);
-      QueueReceiver result = session.createReceiver(queue, messageSelector);
-      result = new JmsQueueReceiver(result, this);
-      if (trace)
-         log.trace("createdReceiver " + session + " receiver=" + result);
-      addConsumer(result);
-      return result;
+      lock();
+      try
+      {
+         QueueSession session = getQueueSession();
+         if (trace)
+            log.trace("createReceiver " + session + " queue=" + queue + " selector=" + messageSelector);
+         QueueReceiver result = session.createReceiver(queue, messageSelector);
+         result = new JmsQueueReceiver(result, this);
+         if (trace)
+            log.trace("createdReceiver " + session + " receiver=" + result);
+         addConsumer(result);
+         return result;
+      }
+      finally
+      {
+         unlock();
+      }
    }
 
    public QueueSender createSender(Queue queue) throws JMSException
    {
-      QueueSession session = getQueueSession();
-      if (trace)
-         log.trace("createSender " + session + " queue=" + queue);
-      QueueSender result = session.createSender(queue);
-      if (trace)
-         log.trace("createdSender " + session + " sender=" + result);
-      addProducer(result);
-      return result;
+      lock();
+      try
+      {
+         QueueSession session = getQueueSession();
+         if (trace)
+            log.trace("createSender " + session + " queue=" + queue);
+         QueueSender result = session.createSender(queue);
+         result = new JmsQueueSender(result, this);
+         if (trace)
+            log.trace("createdSender " + session + " sender=" + result);
+         addProducer(result);
+         return result;
+      }
+      finally
+      {
+         unlock();
+      }
    }
 
    public TemporaryQueue createTemporaryQueue() throws JMSException
@@ -479,68 +612,110 @@ public class JmsSession implements Session, QueueSession, TopicSession
          throw new IllegalStateException("Cannot create temporary queue for javax.jms.TopicSession");
          
       }
-      Session session = getSession();
-      if (trace)
-         log.trace("createTemporaryQueue " + session);
-      TemporaryQueue temp = session.createTemporaryQueue();
-      if (trace)
-         log.trace("createdTemporaryQueue " + session + " temp=" + temp);
-      sf.addTemporaryQueue(temp);
-      return temp;
+
+      lock();
+      try
+      {
+         Session session = getSession();
+         if (trace)
+            log.trace("createTemporaryQueue " + session);
+         TemporaryQueue temp = session.createTemporaryQueue();
+         if (trace)
+            log.trace("createdTemporaryQueue " + session + " temp=" + temp);
+         sf.addTemporaryQueue(temp);
+         return temp;
+      }
+      finally
+      {
+         unlock();
+      }
    }
 
    // -- JMS 1.1
 
    public MessageConsumer createConsumer(Destination destination) throws JMSException
    {
-      Session session = getSession();
-      if (trace)
-         log.trace("createConsumer " + session + " dest=" + destination);
-      MessageConsumer result = session.createConsumer(destination);
-      result = new JmsMessageConsumer(result, this);
-      if (trace)
-         log.trace("createdConsumer " + session + " consumer=" + result);
-      addConsumer(result);
-      return result;
+      lock();
+      try
+      {
+         Session session = getSession();
+         if (trace)
+            log.trace("createConsumer " + session + " dest=" + destination);
+         MessageConsumer result = session.createConsumer(destination);
+         result = new JmsMessageConsumer(result, this);
+         if (trace)
+            log.trace("createdConsumer " + session + " consumer=" + result);
+         addConsumer(result);
+         return result;
+      }
+      finally
+      {
+         unlock();
+      }
    }
 
    public MessageConsumer createConsumer(Destination destination, String messageSelector) throws JMSException
    {
-      Session session = getSession();
-      if (trace)
-         log.trace("createConsumer " + session + " dest=" + destination + " messageSelector=" + messageSelector);
-      MessageConsumer result = session.createConsumer(destination, messageSelector);
-      result = new JmsMessageConsumer(result, this);
-      if (trace)
-         log.trace("createdConsumer " + session + " consumer=" + result);
-      addConsumer(result);
-      return result;
+      lock();
+      try
+      {
+         Session session = getSession();
+         if (trace)
+            log.trace("createConsumer " + session + " dest=" + destination + " messageSelector=" + messageSelector);
+         MessageConsumer result = session.createConsumer(destination, messageSelector);
+         result = new JmsMessageConsumer(result, this);
+         if (trace)
+            log.trace("createdConsumer " + session + " consumer=" + result);
+         addConsumer(result);
+         return result;
+      }
+      finally
+      {
+         unlock();
+      }
    }
 
    public MessageConsumer createConsumer(Destination destination, String messageSelector, boolean noLocal)
          throws JMSException
    {
-      Session session = getSession();
-      if (trace)
-         log.trace("createConsumer " + session + " dest=" + destination + " messageSelector=" + messageSelector + " noLocal=" + noLocal);
-      MessageConsumer result = session.createConsumer(destination, messageSelector, noLocal);
-      result = new JmsMessageConsumer(result, this);
-      if (trace)
-         log.trace("createdConsumer " + session + " consumer=" + result);
-      addConsumer(result);
-      return result;
+      lock();
+      try
+      {
+         Session session = getSession();
+         if (trace)
+            log.trace("createConsumer " + session + " dest=" + destination + " messageSelector=" + messageSelector + " noLocal=" + noLocal);
+         MessageConsumer result = session.createConsumer(destination, messageSelector, noLocal);
+         result = new JmsMessageConsumer(result, this);
+         if (trace)
+            log.trace("createdConsumer " + session + " consumer=" + result);
+         addConsumer(result);
+         return result;
+      }
+      finally
+      {
+         unlock();
+      }
    }
 
    public MessageProducer createProducer(Destination destination) throws JMSException
    {
-      Session session = getSession();
-      if (trace)
-         log.trace("createProducer " + session + " dest=" + destination);
-      MessageProducer result = getSession().createProducer(destination);
-      if (trace)
-         log.trace("createdProducer " + session + " producer=" + result);
-      addProducer(result);
-      return result;
+      lock();
+      try
+      {
+         Session session = getSession();
+         if (trace)
+            log.trace("createProducer " + session + " dest=" + destination);
+         MessageProducer result = getSession().createProducer(destination);
+         result = new JmsMessageProducer(result, this);
+         if (trace)
+            log.trace("createdProducer " + session + " producer=" + result);
+         addProducer(result);
+         return result;
+      }
+      finally
+      {
+         unlock();
+      }
    }
 
    public int getAcknowledgeMode() throws JMSException
@@ -617,10 +792,10 @@ public class JmsSession implements Session, QueueSession, TopicSession
          {
             for (Iterator i = producers.iterator(); i.hasNext();)
             {
-               MessageProducer producer = (MessageProducer) i.next();
+               JmsMessageProducer producer = (JmsMessageProducer) i.next();
                try
                {
-                  producer.close();
+                  producer.closeProducer();
                }
                catch (Throwable t)
                {
