@@ -57,7 +57,7 @@ import EDU.oswego.cs.dl.util.concurrent.SynchronizedBoolean;
  * A generic jms Activation.
  * 
  * @author <a href="adrian@jboss.com">Adrian Brock</a>
- * @version $Revision: 64904 $
+ * @version $Revision: 67902 $
  */
 public class JmsActivation implements ExceptionListener
 {
@@ -77,7 +77,7 @@ public class JmsActivation implements ExceptionListener
    protected MessageEndpointFactory endpointFactory;
    
    /** Whether delivery is active */
-   protected SynchronizedBoolean deliveryActive;
+   protected SynchronizedBoolean deliveryActive = new SynchronizedBoolean(false);
 
    // Whether we are in the failure recovery loop
    private SynchronizedBoolean inFailure = new SynchronizedBoolean(false);
@@ -213,7 +213,7 @@ public class JmsActivation implements ExceptionListener
     */
    public void start() throws ResourceException
    {
-      deliveryActive = new SynchronizedBoolean(true);
+      deliveryActive.set(true);
       ra.getWorkManager().scheduleWork(new SetupActivation());
    }
 
