@@ -81,14 +81,15 @@ import org.jboss.util.NotImplementedException;
  * @author <a href="mailto:adrian@jboss.org">Adrian Brock</a>
  * @author <a href="weston.price@jboss.com">Weston Price</a>
  * 
- * @version $Revision: 57189 $
+ * @version $Revision: 68295 $
  */
 public abstract class BaseConnectionManager2 extends ServiceMBeanSupport
       implements
          BaseConnectionManager2MBean,
          ConnectionCacheListener,
          ConnectionListenerFactory,
-         TransactionTimeoutConfiguration
+         TransactionTimeoutConfiguration,
+         JTATransactionChecker
 {
    /**
     * Note that this copy has a trailing / unlike the original in
@@ -232,6 +233,11 @@ public abstract class BaseConnectionManager2 extends ServiceMBeanSupport
       throw new NotImplementedException("NYI: getTransactionTimeout()");
    }
 
+   public void checkTransactionActive() throws RollbackException, SystemException
+   {
+      // Nothing
+   }
+
    //ServiceMBeanSupport
 
    protected void startService() throws Exception
@@ -820,7 +826,8 @@ public abstract class BaseConnectionManager2 extends ServiceMBeanSupport
          implements
             ConnectionManager,
             Serializable,
-            TransactionTimeoutConfiguration
+            TransactionTimeoutConfiguration,
+            JTATransactionChecker
    {
       static final long serialVersionUID = -528322728929261214L;
 
@@ -866,6 +873,18 @@ public abstract class BaseConnectionManager2 extends ServiceMBeanSupport
          }
       }
 
+      public void checkTransactionActive() throws RollbackException, SystemException
+      {
+         try
+         {
+            getCM().checkTransactionActive();
+         }
+         catch (ResourceException e)
+         {
+            throw new NestedRuntimeException("Unable to retrieve connection manager", e);
+         }
+      }
+
       private BaseConnectionManager2 getCM() throws ResourceException
       {
          if (realCm == null)
