@@ -48,7 +48,7 @@ import EDU.oswego.cs.dl.util.concurrent.SynchronizedBoolean;
  * @author <a href="mailto:adrian@jboss.org">Adrian Brock</a>
  * @author <a href="weston.price@jboss.com">Weston Price</a>
  * 
- * @version $Revision: 61055 $
+ * @version $Revision: 71781 $
  */
 public class InternalManagedConnectionPool
 {
@@ -74,8 +74,8 @@ public class InternalManagedConnectionPool
    private int maxSize;
 
    /** The available connection event listeners */
-   private ArrayList cls;
-
+   private ArrayList cls;     
+   
    /** The permits used to control who can checkout a connection */
    private final FIFOSemaphore permits;
 
@@ -179,7 +179,9 @@ public class InternalManagedConnectionPool
       try
       {
          if (permits.attempt(poolParams.blockingTimeout))
-         {
+         {        	
+            long poolBlockTime =  System.currentTimeMillis() - startWait ;
+            connectionCounter.updateBlockTime(poolBlockTime);
             //We have a permit to get a connection. Is there one in the pool already?
             ConnectionListener cl = null;
             do
@@ -305,24 +307,29 @@ public class InternalManagedConnectionPool
       }
       catch (InterruptedException ie)
       {
-         long end = System.currentTimeMillis() - startWait;
+         long end = System.currentTimeMillis() - startWait;     	
+    	 connectionCounter.updateBlockTime(end);    	 
          throw new ResourceException("Interrupted while requesting permit! Waited " + end + " ms");
       }
    }
 
    public void returnConnection(ConnectionListener cl, boolean kill)
    {
-      if (cl.getState() == ConnectionListener.DESTROYED)
+      synchronized (cls)
       {
-         log.trace("ManagedConnection is being returned after it was destroyed" + cl);
-         if (cl.hasPermit())
+         if (cl.getState() == ConnectionListener.DESTROYED)
          {
-            // release semaphore
-            cl.grantPermit(false);
-            permits.release();
-         }
+            if (trace)
+               log.trace("ManagedConnection is being returned after it was destroyed" + cl);
+            if (cl.hasPermit())
+            {
+               // release semaphore
+               cl.grantPermit(false);
+               permits.release();
+            }
 
-         return;
+            return;
+         }
       }
 
       if (trace)
@@ -337,12 +344,12 @@ public class InternalManagedConnectionPool
          kill = true;
       }
 
-      // We need to destroy this one
-      if (cl.getState() == ConnectionListener.DESTROY)
-         kill = true;
-
       synchronized (cls)
       {
+         // We need to destroy this one
+         if (cl.getState() == ConnectionListener.DESTROY || cl.getState() == ConnectionListener.DESTROYED)
+            kill = true;
+
          checkedOut.remove(cl);
 
          // This is really an error
@@ -359,14 +366,17 @@ public class InternalManagedConnectionPool
             // a connection error occurred.
             // This could happen while the connection is not checked out.
             // e.g. JMS can do this via an ExceptionListener on the connection.
-            // I have twice had to reinstate this line of code, PLEASE DO NOT REMOTE IT!
+            // I have twice had to reinstate this line of code, PLEASE DO NOT REMOVE IT!
             cls.remove(cl);
          }
          // return to the pool
          else
          {
             cl.used();
-            cls.add(cl);
+            if (cls.contains(cl) == false)
+               cls.add(cl);
+            else
+               log.warn("Attempt to return connection twice (ignored): " + cl, new Throwable("STACKTRACE"));
          }
 
          if (cl.hasPermit())
@@ -383,7 +393,6 @@ public class InternalManagedConnectionPool
             log.trace("Destroying returned connection " + cl);
          doDestroy(cl);
       }
-
    }
 
    public void flush()
@@ -446,6 +455,7 @@ public class InternalManagedConnectionPool
             ConnectionListener cl = (ConnectionListener) cls.get(0);
             if (cl.isTimedOut(timeout))
             {
+               connectionCounter.incTimedOut();
                // We need to destroy this one
                cls.remove(0);
                if (destroy == null)
@@ -553,6 +563,26 @@ public class InternalManagedConnectionPool
       return connectionCounter.getCount();
    }
 
+   public long getTotalBlockTime()
+   {	     
+      return connectionCounter.getTotalBlockTime();	     
+   }
+	   
+   public int getTimedOut()
+   {
+	  return connectionCounter.getTimedOut();
+   }
+	   
+   public long getAverageBlockTime()
+   {
+      return connectionCounter.getTotalBlockTime() / getConnectionCreatedCount();	            
+   }
+   
+   public long getMaxWaitTime()
+   {
+	   return connectionCounter.getMaxWaitTime();
+   }
+   
    public int getConnectionCreatedCount()
    {
       return connectionCounter.getCreatedCount();
@@ -563,6 +593,18 @@ public class InternalManagedConnectionPool
       return connectionCounter.getDestroyedCount();
    }
 
+   
+   Set getConnectionListeners()
+   {
+      synchronized (cls)
+      {         
+         Set result = new HashSet();
+         result.addAll(cls); 
+         result.addAll(checkedOut); 
+         return result;
+      }
+   }
+   
    /**
     * Create a connection event listener
     *
@@ -784,6 +826,15 @@ public class InternalManagedConnectionPool
       private int created = 0;
 
       private int destroyed = 0;
+      
+      // Total wait time to get Connection from Pool.
+      private long totalBlockTime;
+      
+      // Idle timed out Connection Count.
+      private int timedOut;
+
+      // The maximum wait time */      
+      private long maxWaitTime;
 
       synchronized int getGuaranteedCount()
       {
@@ -814,5 +865,32 @@ public class InternalManagedConnectionPool
       {
          ++destroyed;
       }
+      
+      synchronized void updateBlockTime(long latest)
+      {    
+         totalBlockTime += latest;         
+         if (maxWaitTime < latest)
+            maxWaitTime = latest;
+      }
+
+      long getTotalBlockTime()
+      {
+         return totalBlockTime;
+      }
+  
+      int getTimedOut()
+      {
+         return timedOut;
+      }
+             
+      synchronized void incTimedOut()
+      {
+         ++timedOut;
+      }
+
+      long getMaxWaitTime()
+      {
+          return maxWaitTime;
+      }
    }
-}
\ No newline at end of file
+}
