@@ -25,6 +25,7 @@ import java.security.AccessController;
 import java.security.PrivilegedAction;
 import java.util.Iterator;
 import java.util.Map;
+import java.util.Set;
 
 import javax.management.Notification;
 import javax.management.NotificationFilter;
@@ -32,6 +33,7 @@ import javax.management.NotificationListener;
 import javax.management.ObjectName;
 import javax.resource.ResourceException;
 import javax.resource.spi.ConnectionRequestInfo;
+import javax.resource.spi.ManagedConnection;
 import javax.resource.spi.ManagedConnectionFactory;
 import javax.security.auth.Subject;
 import javax.transaction.Transaction;
@@ -42,6 +44,14 @@ import org.jboss.logging.Logger;
 import org.jboss.mx.util.JMXExceptionDecoder;
 import org.jboss.resource.JBossResourceException;
 import org.jboss.resource.connectionmanager.InternalManagedConnectionPool.PoolParams;
+import org.jboss.resource.statistic.JBossStatistics;
+import org.jboss.resource.statistic.StatisticsReporter;
+import org.jboss.resource.statistic.formatter.StatisticsFormatter;
+import org.jboss.resource.statistic.pool.JBossDefaultSubPoolStatisticFormatter;
+import org.jboss.resource.statistic.pool.JBossManagedConnectionPoolStatistics;
+import org.jboss.resource.statistic.pool.JBossSubPoolStatistics;
+import org.jboss.resource.statistic.pool.ManagedConnectionPoolStatistics;
+
 import org.jboss.system.ServiceMBeanSupport;
 import org.jboss.tm.TransactionLocal;
 
@@ -59,7 +69,7 @@ import EDU.oswego.cs.dl.util.concurrent.ConcurrentReaderHashMap;
  * @author <a href="mailto:adrian@jboss.org">Adrian Brock</a>
  * @author <a href="weston.price@jboss.com">Weston Price</a>
  * 
- * @version $Revision: 59880 $
+ * @version $Revision: 70167 $
  */
 public class JBossManagedConnectionPool extends ServiceMBeanSupport
       implements
@@ -85,12 +95,67 @@ public class JBossManagedConnectionPool extends ServiceMBeanSupport
    /** Whether to use separate pools for transactional and non-transaction use */
    private boolean noTxSeparatePools;
 
+   /** The statisticsFormatter 
+   *
+   * */
+   private String statisticsFormatter;
+   
    private String poolJndiName;
 
    public JBossManagedConnectionPool()
    {
    }
 
+   public Object listFormattedSubPoolStatistics(String formatClassName)
+   {
+      final JBossStatistics stats = (JBossStatistics)listStatistics();
+      final ClassLoader cl = Thread.currentThread().getContextClassLoader();
+      Class clazz;
+      StatisticsFormatter formatter = null;
+      
+      try
+      {
+         clazz = cl.loadClass(formatClassName);
+         formatter = (StatisticsFormatter)clazz.newInstance();
+      }
+      catch (Exception e)
+      {
+         log.warn("warn: statistics formatter not found, setting to " + statisticsFormatter);
+         formatter = new JBossDefaultSubPoolStatisticFormatter();
+         
+      }
+      ManagedConnectionPoolStatistics poolStats = (ManagedConnectionPoolStatistics)stats;
+      return formatter.formatStatistics(stats);      
+   }
+
+   public Object listFormattedSubPoolStatistics()
+   {
+      
+      Object formatted = listFormattedSubPoolStatistics(statisticsFormatter);
+      return formatted;  
+   }
+   
+   public Object listStatistics()
+   {
+      ManagedConnectionPoolStatistics stats = null;
+      
+      if (poolingStrategy instanceof StatisticsReporter)
+      {
+         
+         StatisticsReporter reporter = (StatisticsReporter) poolingStrategy;
+         stats = (ManagedConnectionPoolStatistics)reporter.listStatistics();
+         stats.setCriteria(getCriteria());
+         stats.setName(getManagedConnectionFactoryName().toString());
+         
+      }      
+
+      return stats;      
+   }
+   
+   public Object listUnderlyingNativeConnectionStatistics()
+   {
+	   return poolingStrategy.listUnderlyingNativeConnectionStatistics();
+   }
    public ManagedConnectionPool getManagedConnectionPool()
    {
       return poolingStrategy;
@@ -283,6 +348,16 @@ public class JBossManagedConnectionPool extends ServiceMBeanSupport
       return "JBossManagedConnectionPool";
    }
 
+   public String getStatisticsFormatter()
+   {
+      return statisticsFormatter;
+   }
+
+   public void setStatisticsFormatter(String statisticsFormatter)
+   {
+      this.statisticsFormatter = statisticsFormatter;
+   }
+   
    public String getPoolJndiName()
    {
       return this.poolJndiName;
@@ -406,7 +481,7 @@ public class JBossManagedConnectionPool extends ServiceMBeanSupport
    /**
     * The base pool implementation
     */
-   public abstract static class BasePool implements ManagedConnectionPool, PreFillPoolSupport
+   public abstract static class BasePool implements ManagedConnectionPool, StatisticsReporter, PreFillPoolSupport
    {
       /** The subpools */
       private final Map subPools = new ConcurrentReaderHashMap();
@@ -628,7 +703,7 @@ public class JBossManagedConnectionPool extends ServiceMBeanSupport
          InternalManagedConnectionPool mcp = (InternalManagedConnectionPool) cl.getContext();
          mcp.returnConnection(cl, kill);
          if (traceEnabled)
-            dump("Returning connection to pool " + cl);
+            dump("Returned connection to pool " + cl);
       }
 
       /**
@@ -843,7 +918,70 @@ public class JBossManagedConnectionPool extends ServiceMBeanSupport
             log.trace(toLog);
          }
       }
+      
+      public JBossStatistics listStatistics()
+      {   
+         final ManagedConnectionPoolStatistics subPoolStats = new JBossManagedConnectionPoolStatistics(subPools.size());
+         
+         subPoolStats.setBlockingTimeout(poolParams.blockingTimeout);
+         subPoolStats.setIdleTimeout(poolParams.idleTimeout);
+         subPoolStats.setMax(poolParams.maxSize);
+         subPoolStats.setMin(poolParams.minSize);
+         subPoolStats.setPrefill(poolParams.prefill);
+         subPoolStats.setNoTxnSeperatePool(noTxSeparatePools);
+         
+         for(Iterator iter = subPools.values().iterator(); iter.hasNext();)
+         {           	
+            JBossSubPoolStatistics stat = new JBossSubPoolStatistics();
+            SubPoolContext subContext = (SubPoolContext)iter.next();
+            Boolean trackByTxn = (subContext.getTrackByTx() != null) ? Boolean.TRUE : Boolean.FALSE;
+            stat.setTrackByTxn(trackByTxn);
+            final InternalManagedConnectionPool internalPool = subContext.getSubPool();
+            stat.setAvailableConnections(internalPool.getAvailableConnections());
+            stat.setConnectionsDestroyed(internalPool.getConnectionDestroyedCount());
+            stat.setConnectionsInUse(internalPool.getConnectionInUseCount());
+            stat.setMaxConnectionsInUse(internalPool.getMaxConnectionsInUseCount());
+            stat.setTotalBlockTime(internalPool.getTotalBlockTime());
+            stat.setAverageBlockTime(internalPool.getAverageBlockTime());
+            stat.setMaxWaitTime(internalPool.getMaxWaitTime());
+            stat.setTotalTimedOut(internalPool.getTimedOut());
+            subPoolStats.addSubPool(stat);            
+         }
+         return (JBossStatistics)subPoolStats;
+      }
+      
+      public Object listUnderlyingNativeConnectionStatistics()
+      {
+    	  String statistics = "";
+    	  for(Iterator iter = subPools.values().iterator(); iter.hasNext();)
+          {                      
+    		  SubPoolContext subContext = (SubPoolContext)iter.next();
+    		  InternalManagedConnectionPool internalPool = subContext.getSubPool();
+    		  Set cels = internalPool.getConnectionListeners();
+    		  for(Iterator celsIter = cels.iterator(); celsIter.hasNext();)
+              {
+    			 ConnectionListener cl = (ConnectionListener) celsIter.next();
+              	 ManagedConnection mc = cl.getManagedConnection();
+              	 if (mc instanceof org.jboss.resource.statistic.JBossConnectionStatistics)
+              	 {
+              		org.jboss.resource.statistic.JBossConnectionStatistics stats = (org.jboss.resource.statistic.JBossConnectionStatistics)mc;
+              		statistics += stats.listConnectionStats();
+              		if(statistics.startsWith("-1")){
+              		  statistics = " ManagedConnetion in a Pool does not expose NativeConnectionStatistics !!!... ";
+              		  break;
+              		}
+              	 }
+              	 else
+              	 {
+              		 statistics = mc + " does not implement org.jboss.resource.statistic.JBossConnectionStatistics , <br><font color='red'>So this Operation is Not available!!!</font> ";              		 
+              		 break;              		 
+              	 }
+              }
+          }   
+    	  return statistics;
+      }
    }
+      
 
    /**
     * Pooling by subject and connection request information
