@@ -125,7 +125,7 @@ import EDU.oswego.cs.dl.util.concurrent.SynchronizedBoolean;
  *
  * @author <a href="mailto:d_jencks@users.sourceforge.net">David Jencks</a>
  * @author <a href="mailto:adrian@jboss.org">Adrian Brock</a>
- * @version $Revision: 59870 $
+ * @version $Revision: 68295 $
  */
 public class TxConnectionManager extends BaseConnectionManager2 implements TxConnectionManagerMBean
 {
@@ -268,6 +268,21 @@ public class TxConnectionManager extends BaseConnectionManager2 implements TxCon
       return -1;
    }
 
+   @Override
+   public void checkTransactionActive() throws RollbackException, SystemException
+   {
+      if (tm == null)
+         throw new IllegalStateException("No transaction manager: " + ccmName);
+      Transaction tx = tm.getTransaction();
+      if (tx != null)
+      {
+         int status = tx.getStatus();
+         // Only allow states that will actually succeed
+         if (status != Status.STATUS_ACTIVE && status != Status.STATUS_PREPARING && status != Status.STATUS_PREPARED && status != Status.STATUS_COMMITTING)
+            throw new RollbackException("Transaction " + tx + " cannot proceed " + TxUtils.getStatusAsString(status));
+      }
+   }
+
    protected void startService() throws Exception
    {
       if (transactionManagerService != null)
