@@ -136,28 +136,7 @@ import org.jboss.ejb3.mdb.ConsumerContainer;
 import org.jboss.ejb3.mdb.MDB;
 import org.jboss.ejb3.mdb.ProducerImpl;
 import org.jboss.ejb3.mdb.ProducersImpl;
-import org.jboss.ejb3.metamodel.ActivationConfig;
-import org.jboss.ejb3.metamodel.AssemblyDescriptor;
-import org.jboss.ejb3.metamodel.CacheConfig;
-import org.jboss.ejb3.metamodel.ClusterConfig;
-import org.jboss.ejb3.metamodel.ContainerTransaction;
-import org.jboss.ejb3.metamodel.EjbJarDD;
-import org.jboss.ejb3.metamodel.EnterpriseBean;
-import org.jboss.ejb3.metamodel.EnterpriseBeans;
-import org.jboss.ejb3.metamodel.ExcludeList;
-import org.jboss.ejb3.metamodel.InitMethod;
-import org.jboss.ejb3.metamodel.InterceptorBinding;
-import org.jboss.ejb3.metamodel.MessageDestination;
-import org.jboss.ejb3.metamodel.MessageDrivenBean;
-import org.jboss.ejb3.metamodel.MessageDrivenDestination;
-import org.jboss.ejb3.metamodel.Method;
-import org.jboss.ejb3.metamodel.MethodAttributes;
-import org.jboss.ejb3.metamodel.MethodPermission;
-import org.jboss.ejb3.metamodel.PoolConfig;
-import org.jboss.ejb3.metamodel.RemoveMethod;
-import org.jboss.ejb3.metamodel.SecurityIdentity;
-import org.jboss.ejb3.metamodel.SessionEnterpriseBean;
-import org.jboss.ejb3.metamodel.XmlAnnotation;
+import org.jboss.ejb3.metamodel.*;
 import org.jboss.ejb3.service.ServiceContainer;
 import org.jboss.ejb3.stateful.StatefulContainer;
 import org.jboss.logging.Logger;
@@ -167,9 +146,11 @@ import org.jboss.metamodel.descriptor.NameValuePair;
 import org.jboss.metamodel.descriptor.ResourceRef;
 import org.jboss.metamodel.descriptor.RunAs;
 import org.jboss.metamodel.descriptor.SecurityRole;
+import org.jboss.wsf.spi.annotation.WebContextImpl;
+import org.jboss.wsf.spi.annotation.WebContext;
 
 /**
- * @version <tt>$Revision: 63299 $</tt>
+ * @version <tt>$Revision: 69088 $</tt>
  * @author <a href="mailto:bdecoste@jboss.com">William DeCoste</a>
  * @author <a href="mailto:bill@jboss.com">Bill Burke</a>
  */
@@ -722,6 +703,23 @@ public class Ejb3DescriptorHandler extends Ejb3AnnotationHandler
       addEjbAnnotations(container, enterpriseBean);
 
       addEjb21Annotations(container, isStateful);
+
+      addWebServiceAnnotations(container, enterpriseBean, ejbName);
+   }
+
+   private void addWebServiceAnnotations(EJBContainer container, EnterpriseBean enterpriseBean, String ejbName)
+   {
+      if(dd.getWebservices()!=null)
+      {
+         Webservices webservices = dd.getWebservices();
+         if(webservices.getContextRoot()!=null)
+         {
+            WebContextImpl webContextImpl = new WebContextImpl();
+            webContextImpl.setContextRoot(webservices.getContextRoot());
+
+            addClassAnnotation(container, WebContext.class, webContextImpl);
+         }
+      }
    }
 
    /**
@@ -1286,18 +1284,22 @@ public class Ejb3DescriptorHandler extends Ejb3AnnotationHandler
       }
    }
    
+   // FIXME: Wolf: Why do we have this method here? It's a bean property setter, so should
+   // be part of bean spi.
    protected void setAnnotationPropertyField(Field field, Object annotation, String value) throws Exception
    {
       if (field.getType() == String.class)
          field.set(annotation, value);
-      else if (field.getType() == Long.class)
+      else if (field.getType() == Long.class || field.getType() == Long.TYPE)
          field.setLong(annotation, Long.parseLong(value));
-      else if (field.getType() == Integer.class)
+      else if (field.getType() == Integer.class || field.getType() == Integer.TYPE)
          field.setInt(annotation, Integer.parseInt(value));
       else if (field.getType() == Class.class)
          field.set(annotation, di.getClassLoader().loadClass(value));
-      else if (field.getType() == Boolean.class)
+      else if (field.getType() == Boolean.class || field.getType() == Boolean.TYPE)
          field.setBoolean(annotation, Boolean.parseBoolean(value));
+      else
+         throw new IllegalArgumentException("unsupported field type " + field.getType() + " on field " + field);
    }
 
    private void addCacheAnnotations(EJBContainer container,
