@@ -67,7 +67,7 @@ import org.jboss.cache.config.Option;
  * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
  * @author Brian Stansberry
  * 
- * @version $Revision: 65637 $
+ * @version $Revision: 73125 $
  */
 public class StatefulTreeCache implements ClusteredStatefulCache
 {
@@ -319,6 +319,9 @@ public class StatefulTreeCache implements ClusteredStatefulCache
               config.maxSize());
       Region region = evictRegionManager.createRegion(cacheNode, element);
       
+      // JBCACHE-1136.  There's no reason to have state in an inactive region
+      cleanBeanRegion();
+      
       cache.registerClassLoader(cacheNode.toString(), cl);
       try
       {
