@@ -54,7 +54,7 @@ import java.security.Principal;
 import java.util.*;
 
 /**
- * @version <tt>$Revision: 65665 $</tt>
+ * @version <tt>$Revision: 66614 $</tt>
  * @author <a href="mailto:bdecoste@jboss.com">William DeCoste</a>
  */
 public abstract class MessagingContainer extends EJBContainer implements TimedObjectInvoker
@@ -581,19 +581,12 @@ public abstract class MessagingContainer extends EJBContainer implements TimedOb
    protected JMSProviderAdapter getJMSProviderAdapter()
            throws NamingException
    {
-      Context context = getInitialContext();
       //todo make this pluggable
       String providerAdapterJNDI = getProviderAdapterJNDI();
-      try
-      {
-         log.debug("Looking up provider adapter: " + providerAdapterJNDI);
+      
+      log.debug("Looking up provider adapter: " + providerAdapterJNDI);
      
-         return (JMSProviderAdapter) context.lookup(providerAdapterJNDI);
-      }
-      finally
-      {
-         context.close();
-      }
+      return (JMSProviderAdapter) getInitialContext().lookup(providerAdapterJNDI);
    }
 
    /**
