@@ -50,9 +50,9 @@ import org.xml.sax.Attributes;
  *
  * @author <a href="mailto:bdecoste@jboss.com">William DeCoste</a>
  * @author Anil.Saldhana@jboss.com
- * @version <tt>$Revision: 65717 $</tt>
+ * @version <tt>$Revision: 69023 $</tt>
  */ 
-public class JBossDDObjectFactory extends DDObjectFactory
+   public class JBossDDObjectFactory extends DDObjectFactory
 {
    private static final Logger log = Logger.getLogger(JBossDDObjectFactory.class);
 
@@ -135,6 +135,12 @@ public class JBossDDObjectFactory extends DDObjectFactory
       {
          child = new ResourceManager();
       }
+      else if (localName.equals("webservices"))
+      {
+         dd.setWebservices(new Webservices());
+         child = dd.getWebservices();
+      }
+
 
       return child;
    }
@@ -843,6 +849,30 @@ public class JBossDDObjectFactory extends DDObjectFactory
       }
    }
 
+   public void setValue(Webservices webservices, UnmarshallingContext navigator, String namespaceURI, String localName, String value)
+   {
+      if (localName.equals("context-root"))
+      {
+         webservices.setContextRoot(value);
+      }
+      else if (localName.equals("webservice-description-name"))
+      {
+         webservices.setDescriptionName(value);
+      }
+      else if (localName.equals("config-name"))
+      {
+         webservices.setConfigName(value);
+      }
+      else if (localName.equals("config-file"))
+      {
+         webservices.setConfigFile(value);
+      }
+      else if(localName.equals("wsdl-publish-location"))
+      {
+         webservices.setWsdlPublishLocation(value);
+      }
+
+   }
    /**
     * Called when a child element with simple content is read for DD.
     */
