@@ -28,7 +28,13 @@ public interface StatefulRemote
 {
    void localCall() throws Exception;
    
+   void localHomeCall() throws Exception;
+   
    int remoteCall() throws Exception;
    
+   int remoteHomeCall() throws Exception;
+   
    int method() throws Exception;
+   
+   int homeMethod() throws Exception;
 }
