@@ -47,7 +47,7 @@ import org.jboss.util.id.SerialVersion;
  * 
  * @author Scott.Stark@jboss.org
  * @author Ron Monzillo, Gary Ellison (javadoc)
- * @version $Revision: 57196 $
+ * @version $Revision: 68067 $
  */
 public final class EJBMethodPermission
    extends Permission
@@ -368,6 +368,8 @@ public final class EJBMethodPermission
       {
          implies = methodName.equals(perm.methodName);
       }
+      else
+         implies = true;
 
       // Check the method interface
       if( implies == true && methodInterface != null )
@@ -375,7 +377,8 @@ public final class EJBMethodPermission
          implies = methodInterface.equals(perm.methodInterface);
       }
       // Check the method signature
-      if( implies == true && methodSig != null )
+      if( implies == true && methodSig != null)
+    	//  if( implies == true && methodSig != null && !(methodSig.equals("")&&perm.methodSig == null))
       {
          implies = methodSig.equals(perm.methodSig);
       }      
@@ -416,16 +419,16 @@ public final class EJBMethodPermission
       return tmp.toString();
    }
 
-   private static String[] convertParameters(Class[] params)
+   private static String[] convertParameters(Class<?>[] params)
    {
-      ArrayList tmp = new ArrayList();
+      ArrayList<String> tmp = new ArrayList<String>();
       for(int p = 0; p < params.length; p++)
       {
-         Class c = params[p];
+         Class<?> c = params[p];
          if( c.isArray() )
          {
             StringBuffer sb = new StringBuffer();
-            Class subType = c.getComponentType();
+            Class<?> subType = c.getComponentType();
             sb.append(subType.getName());
             // Convert to type[][]...[]
             while( subType != null )
