@@ -21,11 +21,13 @@
  */
 package org.jboss.ws.core.jaxws.client;
 
-// $Id: ClientImpl.java 3571 2007-06-13 14:52:22Z heiko.braun@jboss.com $
+// $Id: ClientImpl.java 5906 2008-03-10 13:25:02Z richard.opalka@jboss.com $
 
+import java.net.URI;
 import java.rmi.RemoteException;
 import java.util.HashMap;
 import java.util.HashSet;
+import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
 import java.util.Observable;
@@ -37,6 +39,9 @@ import javax.xml.ws.Binding;
 import javax.xml.ws.BindingProvider;
 import javax.xml.ws.EndpointReference;
 import javax.xml.ws.WebServiceException;
+import javax.xml.ws.addressing.AddressingBuilder;
+import javax.xml.ws.addressing.AddressingProperties;
+import javax.xml.ws.addressing.JAXWSAConstants;
 import javax.xml.ws.handler.Handler;
 import javax.xml.ws.handler.HandlerResolver;
 import javax.xml.ws.handler.MessageContext;
@@ -46,6 +51,7 @@ import javax.xml.ws.http.HTTPBinding;
 import javax.xml.ws.http.HTTPException;
 import javax.xml.ws.soap.SOAPBinding;
 import javax.xml.ws.soap.SOAPFaultException;
+import javax.xml.ws.wsaddressing.BindingProvider21;
 
 import org.jboss.logging.Logger;
 import org.jboss.remoting.transport.http.HTTPMetadataConstants;
@@ -58,11 +64,24 @@ import org.jboss.ws.core.jaxws.binding.BindingProviderImpl;
 import org.jboss.ws.core.jaxws.handler.HandlerChainExecutor;
 import org.jboss.ws.core.jaxws.handler.HandlerResolverImpl;
 import org.jboss.ws.core.jaxws.handler.MessageContextJAXWS;
-import org.jboss.ws.core.jaxws.handler.PortInfoImpl;
 import org.jboss.ws.core.jaxws.handler.SOAPMessageContextJAXWS;
 import org.jboss.ws.core.soap.MessageContextAssociation;
+import org.jboss.ws.extensions.addressing.AddressingClientUtil;
+import org.jboss.ws.extensions.wsrm.RMAddressingConstants;
+import org.jboss.ws.extensions.wsrm.RMConstant;
+import org.jboss.ws.extensions.wsrm.RMClientSequence;
+import org.jboss.ws.extensions.wsrm.api.RMException;
+import org.jboss.ws.extensions.wsrm.common.RMHelper;
+import org.jboss.ws.extensions.wsrm.protocol.RMConstants;
+import org.jboss.ws.extensions.wsrm.protocol.RMProvider;
+import org.jboss.ws.extensions.wsrm.protocol.spi.RMAckRequested;
+import org.jboss.ws.extensions.wsrm.protocol.spi.RMCreateSequenceResponse;
+import org.jboss.ws.extensions.wsrm.protocol.spi.RMSequence;
+import org.jboss.ws.extensions.wsrm.protocol.spi.RMSequenceAcknowledgement;
+import org.jboss.ws.extensions.wsrm.protocol.spi.RMSerializable;
 import org.jboss.ws.metadata.config.Configurable;
 import org.jboss.ws.metadata.config.ConfigurationProvider;
+import org.jboss.ws.metadata.umdm.ClientEndpointMetaData;
 import org.jboss.ws.metadata.umdm.EndpointMetaData;
 import org.jboss.ws.metadata.umdm.OperationMetaData;
 import org.jboss.wsf.spi.metadata.j2ee.serviceref.UnifiedHandlerMetaData.HandlerType;
@@ -73,27 +92,39 @@ import org.jboss.wsf.spi.metadata.j2ee.serviceref.UnifiedHandlerMetaData.Handler
  * @author Thomas.Diesler@jboss.org
  * @since 04-Jul-2006
  */
-public class ClientImpl extends CommonClient implements BindingProvider, Configurable
+public class ClientImpl extends CommonClient implements org.jboss.ws.extensions.wsrm.api.RMProvider, BindingProvider21, Configurable
 {
    // provide logging
    private static Logger log = Logger.getLogger(ClientImpl.class);
 
-	// the associated endpoint meta data
-	private final EndpointMetaData epMetaData;
+   // the associated endpoint meta data
+   private final ClientEndpointMetaData epMetaData;
 
-	// Keep a handle on the resolver so that updateConfig calls may revisit the associated chains
-	private final HandlerResolver handlerResolver;
-	
-	private Map<HandlerType, HandlerChainExecutor> executorMap = new HashMap<HandlerType, HandlerChainExecutor>();
+   // Keep a handle on the resolver so that updateConfig calls may revisit the associated chains
+   private final HandlerResolver handlerResolver;
 
-	private static HandlerType[] HANDLER_TYPES = new HandlerType[] {HandlerType.PRE, HandlerType.ENDPOINT, HandlerType.POST};
-	
-	public ClientImpl(EndpointMetaData epMetaData, HandlerResolver handlerResolver)
+   private Map<HandlerType, HandlerChainExecutor> executorMap = new HashMap<HandlerType, HandlerChainExecutor>();
+   private static HandlerType[] HANDLER_TYPES = new HandlerType[] { HandlerType.PRE, HandlerType.ENDPOINT, HandlerType.POST };
+
+   // WS-RM sequence associated with the proxy
+   private RMClientSequence wsrmSequence;
+
+   public final void setWSRMSequence(RMClientSequence wsrmSequence)
+   {
+      this.wsrmSequence = wsrmSequence;
+   }
+
+   public final RMClientSequence getWSRMSequence()
+   {
+      return this.wsrmSequence;
+   }
+
+   public ClientImpl(EndpointMetaData epMetaData, HandlerResolver handlerResolver)
    {
       super(epMetaData);
       setTargetEndpointAddress(epMetaData.getEndpointAddress());
 
-      this.epMetaData = epMetaData;
+      this.epMetaData = (ClientEndpointMetaData)epMetaData;
       this.handlerResolver = handlerResolver;
 
       initBindingHandlerChain(false);
@@ -111,31 +142,31 @@ public class ClientImpl extends CommonClient implements BindingProvider, Configu
    {
       BindingExt binding = (BindingExt)getBindingProvider().getBinding();
 
-		PortInfo portInfo = getPortInfo(epMetaData);
+      PortInfo portInfo = epMetaData.getPortInfo();
 
-		if (handlerResolver != null)
-		{
+      if (handlerResolver != null)
+      {
 
-			boolean jbossHandlerResolver = handlerResolver instanceof HandlerResolverImpl;
-			
-			if (jbossHandlerResolver) // knows about PRE and POST handlers
-			{
-				HandlerResolverImpl impl = (HandlerResolverImpl)handlerResolver;
-				impl.initHandlerChain(epMetaData, HandlerType.PRE, clearExistingHandlers);
-				impl.initHandlerChain(epMetaData, HandlerType.ENDPOINT, clearExistingHandlers);
-				impl.initHandlerChain(epMetaData, HandlerType.POST, clearExistingHandlers);
+         boolean jbossHandlerResolver = handlerResolver instanceof HandlerResolverImpl;
 
-				List<Handler> preChain = impl.getHandlerChain(portInfo, HandlerType.PRE);
-				List<Handler> postChain = impl.getHandlerChain(portInfo, HandlerType.POST);
-				
-				binding.setHandlerChain(postChain, HandlerType.POST);
-				binding.setHandlerChain(preChain, HandlerType.PRE);
-			}
+         if (jbossHandlerResolver) // knows about PRE and POST handlers
+         {
+            HandlerResolverImpl impl = (HandlerResolverImpl)handlerResolver;
+            impl.initHandlerChain(epMetaData, HandlerType.PRE, clearExistingHandlers);
+            impl.initHandlerChain(epMetaData, HandlerType.ENDPOINT, clearExistingHandlers);
+            impl.initHandlerChain(epMetaData, HandlerType.POST, clearExistingHandlers);
 
-			// The regular handler chain
-			List<Handler> endpointChain = handlerResolver.getHandlerChain(portInfo);
-			binding.setHandlerChain(endpointChain);
-		}
+            List<Handler> preChain = impl.getHandlerChain(portInfo, HandlerType.PRE);
+            List<Handler> postChain = impl.getHandlerChain(portInfo, HandlerType.POST);
+
+            binding.setHandlerChain(postChain, HandlerType.POST);
+            binding.setHandlerChain(preChain, HandlerType.PRE);
+         }
+
+         // The regular handler chain
+         List<Handler> endpointChain = handlerResolver.getHandlerChain(portInfo);
+         binding.setHandlerChain(endpointChain);
+      }
    }
 
    /**
@@ -236,6 +267,34 @@ public class ClientImpl extends CommonClient implements BindingProvider, Configu
       // prior to invoking any handlers (see chapter 9) for the outbound message. Each property within the
       // request context is copied to the message context with a scope of HANDLER.
       Map<String, Object> reqContext = getBindingProvider().getRequestContext();
+
+      if (this.wsrmSequence != null)
+      {
+         if (RMConstant.PROTOCOL_OPERATION_QNAMES.contains(opName) == false)
+         {
+            if (this.wsrmSequence.getBackPort() != null)
+            {
+               // rewrite ReplyTo to use client addressable back port
+               Map<String, Object> requestContext = getBindingProvider().getRequestContext();
+               AddressingProperties addressingProps = (AddressingProperties)requestContext.get(JAXWSAConstants.CLIENT_ADDRESSING_PROPERTIES_OUTBOUND);
+               addressingProps.setReplyTo(AddressingBuilder.getAddressingBuilder().newEndpointReference(this.wsrmSequence.getBackPort()));
+            }
+            Map<String, Object> rmRequestContext = new HashMap<String, Object>();
+            List<QName> outMsgs = new LinkedList<QName>();
+            wsrmSequence.newMessageNumber();
+            outMsgs.add(RMProvider.get().getConstants().getSequenceQName());
+            outMsgs.add(RMProvider.get().getConstants().getAckRequestedQName());
+            if (wsrmSequence.isAckRequested())
+            {
+               // piggy backing
+               outMsgs.add(RMProvider.get().getConstants().getSequenceAcknowledgementQName());
+            }
+            rmRequestContext.put(RMConstant.PROTOCOL_MESSAGES, outMsgs);
+            rmRequestContext.put(RMConstant.SEQUENCE_REFERENCE, wsrmSequence);
+            reqContext.put(RMConstant.REQUEST_CONTEXT, rmRequestContext);
+         }
+      }
+
       msgContext.putAll(reqContext);
 
       try
@@ -258,8 +317,36 @@ public class ClientImpl extends CommonClient implements BindingProvider, Configu
       }
       finally
       {
-         // Copy the inbound msg properties to the binding's response context
          msgContext = MessageContextAssociation.peekMessageContext();
+         if (this.wsrmSequence != null)
+         {
+            if (RMConstant.PROTOCOL_OPERATION_QNAMES.contains(opName) == false)
+            {
+               Map<String, Object> wsrmResCtx = (Map<String, Object>) msgContext.get(RMConstant.RESPONSE_CONTEXT);
+               if (wsrmResCtx != null)
+               {
+                  RMConstants wsrmConstants = RMProvider.get().getConstants();
+                  Map<QName, RMSerializable> mapping = (Map<QName, RMSerializable>)wsrmResCtx.get(RMConstant.PROTOCOL_MESSAGES_MAPPING);
+                  QName seq = wsrmConstants.getSequenceQName();
+                  if (mapping.keySet().contains(seq))
+                  {
+                     RMHelper.handleSequenceHeader((RMSequence)mapping.get(seq), this.wsrmSequence);
+                  }
+                  QName seqAck = wsrmConstants.getSequenceAcknowledgementQName();
+                  if (mapping.keySet().contains(seqAck))
+                  {
+                     RMHelper.handleSequenceAcknowledgementHeader((RMSequenceAcknowledgement)mapping.get(seqAck), this.wsrmSequence);
+                  }
+                  QName ackReq = wsrmConstants.getAckRequestedQName();
+                  if (mapping.keySet().contains(ackReq))
+                  {
+                     RMHelper.handleAckRequestedHeader((RMAckRequested)mapping.get(ackReq), this.wsrmSequence);
+                  }
+               }
+            }
+         }
+
+         // Copy the inbound msg properties to the binding's response context
          for (String key : msgContext.keySet())
          {
             Object value = msgContext.get(key);
@@ -388,26 +475,26 @@ public class ClientImpl extends CommonClient implements BindingProvider, Configu
       configProvider.setConfigName(configName, configFile);
    }
 
-	/**
-	 * Retrieve header names that can be processed by this binding
-	 * @return
-	 */
-	public Set<QName> getHeaders()
+   /**
+    * Retrieve header names that can be processed by this binding
+    * @return
+    */
+   public Set<QName> getHeaders()
    {
-		Set<QName> headers = new HashSet<QName>();
+      Set<QName> headers = new HashSet<QName>();
 
-		BindingExt binding = (BindingExt)getBinding();
+      BindingExt binding = (BindingExt)getBinding();
 
-		for(HandlerType type : HANDLER_TYPES)
-		{
-			for(Handler bindingHandler : binding.getHandlerChain(type))
-			{
-				if(bindingHandler instanceof SOAPHandler)
-					headers.addAll( ((SOAPHandler)bindingHandler).getHeaders());
-			}
-		}
-		
-		return headers;
+      for (HandlerType type : HANDLER_TYPES)
+      {
+         for (Handler bindingHandler : binding.getHandlerChain(type))
+         {
+            if (bindingHandler instanceof SOAPHandler)
+               headers.addAll(((SOAPHandler)bindingHandler).getHeaders());
+         }
+      }
+
+      return headers;
    }
 
    @Override
@@ -416,13 +503,70 @@ public class ClientImpl extends CommonClient implements BindingProvider, Configu
       Object bool = getRequestContext().get(BindingProvider.SESSION_MAINTAIN_PROPERTY);
       return Boolean.TRUE.equals(bool);
    }
-   
-   private PortInfo getPortInfo(EndpointMetaData epMetaData)
+
+   ///////////////////
+   // WS-RM support //
+   ///////////////////
+   @SuppressWarnings("unchecked")
+   public void createSequence() throws RMException
    {
-      QName serviceName = epMetaData.getServiceMetaData().getServiceName();
-      QName portName = epMetaData.getPortName();
-      String bindingID = epMetaData.getBindingId();
-      PortInfo portInfo = new PortInfoImpl(serviceName, portName, bindingID);
-      return portInfo;
+      if (this.wsrmSequence != null)
+         throw new IllegalStateException("Sequence already registered with proxy instance");
+
+      try
+      {
+         // set up addressing data
+         RMClientSequence candidateSequence = new RMClientSequence(getEndpointMetaData().getConfig().getRMMetaData());
+         String address = getEndpointMetaData().getEndpointAddress();
+         String action = RMAddressingConstants.CREATE_SEQUENCE_WSA_ACTION;
+         AddressingProperties addressingProps = null;
+         URI backPort = candidateSequence.getBackPort();
+         if (backPort != null)
+         {
+            addressingProps = AddressingClientUtil.createDefaultProps(action, address);
+            addressingProps.setReplyTo(AddressingBuilder.getAddressingBuilder().newEndpointReference(backPort));
+         }
+         else
+         {
+            addressingProps = AddressingClientUtil.createAnonymousProps(action, address);
+         }
+         Map requestContext = getBindingProvider().getRequestContext();
+         requestContext.put(JAXWSAConstants.CLIENT_ADDRESSING_PROPERTIES_OUTBOUND, addressingProps);
+         // set up wsrm request context
+         QName createSequenceQN = RMProvider.get().getConstants().getCreateSequenceQName();
+         Map rmRequestContext = new HashMap();
+         List outMsgs = new LinkedList();
+         outMsgs.add(createSequenceQN);
+         rmRequestContext.put(RMConstant.PROTOCOL_MESSAGES, outMsgs);
+         rmRequestContext.put(RMConstant.SEQUENCE_REFERENCE, candidateSequence);
+         requestContext.put(RMConstant.REQUEST_CONTEXT, rmRequestContext);
+         // invoke stub method
+         invoke(createSequenceQN, new Object[] {}, getBindingProvider().getResponseContext());
+         // read WSRM sequence id from response context
+         Map rmResponseContext = (Map)getBindingProvider().getResponseContext().get(RMConstant.RESPONSE_CONTEXT);
+         RMCreateSequenceResponse createSequenceResponse = ((RMCreateSequenceResponse)((Map)rmResponseContext.get(RMConstant.PROTOCOL_MESSAGES_MAPPING)).get(RMProvider.get().getConstants().getCreateSequenceResponseQName())); 
+         String outboundId = createSequenceResponse.getIdentifier();
+         candidateSequence.setClient(this);
+         candidateSequence.setOutboundId(outboundId);
+         candidateSequence.setBehavior(createSequenceResponse.getIncompleteSequenceBehavior());
+         candidateSequence.setDuration(RMHelper.durationToLong(createSequenceResponse.getExpires()));
+         this.wsrmSequence = candidateSequence;
+      }
+      catch (Exception e)
+      {
+         throw new RMException("Unable to create WSRM sequence", e);
+      }
    }
-}
\ No newline at end of file
+   
+   public void closeSequence()
+   {
+      try
+      {
+         this.wsrmSequence.close();
+      }
+      finally
+      {
+         this.wsrmSequence = null;
+      }
+   }
+}
