@@ -21,13 +21,13 @@
 */
 package org.jboss.ws.extensions.security.element;
 
-import org.jboss.ws.extensions.security.WSSecurityException;
+import org.jboss.ws.extensions.security.exception.WSSecurityException;
 import org.w3c.dom.Element;
 
 
 /**
  * @author <a href="mailto:jason.greene@jboss.com">Jason T. Greene</a>
- * @version $Revision: 1757 $
+ * @version $Revision: 5034 $
  */
 public interface SecurityElement
 {
