@@ -36,6 +36,7 @@ import javax.management.ObjectName;
 import javax.wsdl.Definition;
 import javax.wsdl.Import;
 import javax.wsdl.Port;
+import javax.wsdl.extensions.http.HTTPAddress;
 import javax.wsdl.extensions.soap.SOAPAddress;
 import javax.wsdl.extensions.soap12.SOAP12Address;
 import javax.xml.namespace.QName;
@@ -316,7 +317,7 @@ public abstract class MetaDataBuilder
 
                   // modify the wsdl-1.1 definition
                   if (wsdlDefinitions.getWsdlOneOneDefinition() != null)
-                     replaceWSDL11SOAPAddress(wsdlDefinitions, portName, serviceEndpointURL);
+                     replaceWSDL11PortAddress(wsdlDefinitions, portName, serviceEndpointURL);
                }
                else
                {
@@ -338,13 +339,13 @@ public abstract class MetaDataBuilder
          throw new WSException("Cannot find port in wsdl: " + portName);
    }
 
-   private static void replaceWSDL11SOAPAddress(WSDLDefinitions wsdlDefinitions, QName portQName, String serviceEndpointURL)
+   private static void replaceWSDL11PortAddress(WSDLDefinitions wsdlDefinitions, QName portQName, String serviceEndpointURL)
    {
       Definition wsdlOneOneDefinition = wsdlDefinitions.getWsdlOneOneDefinition();
       String tnsURI = wsdlOneOneDefinition.getTargetNamespace();
 
       // search for matching portElement and replace the address URI
-      Port wsdlOneOnePort = modifySOAPAddress(tnsURI, portQName, serviceEndpointURL, wsdlOneOneDefinition.getServices());
+      Port wsdlOneOnePort = modifyPortAddress(tnsURI, portQName, serviceEndpointURL, wsdlOneOneDefinition.getServices());
 
       // recursivly process imports if none can be found
       if (wsdlOneOnePort == null && !wsdlOneOneDefinition.getImports().isEmpty())
@@ -358,7 +359,7 @@ public abstract class MetaDataBuilder
             while (importsByNS.hasNext())
             {
                Import anImport = (Import)importsByNS.next();
-               wsdlOneOnePort = modifySOAPAddress(anImport.getNamespaceURI(), portQName, serviceEndpointURL, anImport.getDefinition().getServices());
+               wsdlOneOnePort = modifyPortAddress(anImport.getNamespaceURI(), portQName, serviceEndpointURL, anImport.getDefinition().getServices());
             }
          }
       }
@@ -368,7 +369,7 @@ public abstract class MetaDataBuilder
          throw new IllegalArgumentException("Cannot find port with name '" + portQName + "' in wsdl document");
    }
 
-   private static Port modifySOAPAddress(String tnsURI, QName portQName, String serviceEndpointURL, Map services)
+   private static Port modifyPortAddress(String tnsURI, QName portQName, String serviceEndpointURL, Map services)
    {
       Port wsdlOneOnePort = null;
       Iterator itServices = services.values().iterator();
@@ -391,11 +392,16 @@ public abstract class MetaDataBuilder
                      SOAPAddress address = (SOAPAddress)extElement;
                      address.setLocationURI(serviceEndpointURL);
                   }
-                  if (extElement instanceof SOAP12Address)
+                  else if (extElement instanceof SOAP12Address)
                   {
                      SOAP12Address address = (SOAP12Address)extElement;
                      address.setLocationURI(serviceEndpointURL);
                   }
+                  else if (extElement instanceof HTTPAddress)
+                  {
+                     HTTPAddress address = (HTTPAddress)extElement;
+                     address.setLocationURI(serviceEndpointURL);
+                  }
                }
             }
          }
@@ -475,7 +481,14 @@ public abstract class MetaDataBuilder
       else
       {
          WSDLProperty messageName = wsdlOperation.getProperty(Constants.WSDL_PROPERTY_MESSAGE_NAME_IN);
-         addrExt.setInboundAction(tns + "/" + portTypeName + "/" + messageName);
+         if (messageName != null)
+         {
+            addrExt.setInboundAction(tns + "/" + portTypeName + "/" + messageName.getValue());
+         }
+         else
+         {
+            addrExt.setInboundAction(tns + "/" + portTypeName + "/IN");
+         }
       }
 
       // outbound action
@@ -487,7 +500,14 @@ public abstract class MetaDataBuilder
       else
       {
          WSDLProperty messageName = wsdlOperation.getProperty(Constants.WSDL_PROPERTY_MESSAGE_NAME_OUT);
-         addrExt.setOutboundAction(tns + "/" + portTypeName + "/" + messageName);
+         if (messageName != null)
+         {
+            addrExt.setOutboundAction(tns + "/" + portTypeName + "/" + messageName.getValue());
+         }
+         else
+         {
+            addrExt.setOutboundAction(tns + "/" + portTypeName + "/OUT");
+         }
       }
 
       opMetaData.addExtension(addrExt);
