@@ -204,7 +204,7 @@ public class WSDLToJava implements WSDLToJavaIntf
          if (namespacePackageMap == null)
             namespacePackageMap = new HashMap<String, String>();
          String pkg = iter.next();
-         namespacePackageMap.put(map.get(pkg), pkg);
+         namespacePackageMap.put(pkg, map.get(pkg));
       }
    }
 
@@ -315,7 +315,6 @@ public class WSDLToJava implements WSDLToJavaIntf
             JBossXSModel xsmodel = WSDLUtils.getSchemaModel(wsdl.getWsdlTypes());
             QName faultXMLName = intfFault.getElement();
             QName faultXMLType = intfFault.getXmlType();
-
             XSElementDeclaration xe = xsmodel.getElementDeclaration(faultXMLName.getLocalPart(), faultXMLName.getNamespaceURI());
             XSTypeDefinition xt = xe.getTypeDefinition();
             if (!xt.getAnonymous())
@@ -327,7 +326,8 @@ public class WSDLToJava implements WSDLToJavaIntf
             if (cl == null)
             {
                String faultTypeName = (!xt.getAnonymous()) ? faultXMLType.getLocalPart() : faultXMLName.getLocalPart();
-               buf.append(seiPkgName + "." + JavaUtils.capitalize(faultTypeName));
+               String packageName = getPackageName(xt.getNamespace());
+               buf.append(packageName + "." + JavaUtils.capitalize(faultTypeName));
             }
             else buf.append(cl.getName());
             buf.append(",");
@@ -523,7 +523,6 @@ public class WSDLToJava implements WSDLToJavaIntf
       if (unwrappedElement)
       {
          buf.append(tempBuf);
-
          // We need a wrapper class generated
          generateJavaSource(wrapper, WSDLUtils.getSchemaModel(wsdl.getWsdlTypes()), containingElement);
 
@@ -555,7 +554,7 @@ public class WSDLToJava implements WSDLToJavaIntf
 
             XSElementDeclaration element = (XSElementDeclaration)term;
             XSTypeDefinition type = element.getTypeDefinition();
-            String tempContainingElement = containingElement + element.getName();
+            String tempContainingElement = containingElement + ToolsUtils.firstLetterUpperCase(element.getName());
 
             QName xmlType = null;
             if (type.getAnonymous() == false)
@@ -628,11 +627,12 @@ public class WSDLToJava implements WSDLToJavaIntf
             className = className.substring(1);
          className = ToolsUtils.convertInvalidCharacters(className);
          className = utils.firstLetterUpperCase(className);
-         className = seiPkgName + "." + className + arraySuffix;
 
+         String packageName = getPackageName(xt.getNamespace());
+         className = packageName + "." + className + arraySuffix;
          if (holder)
          {
-            className = holderWriter.getOrCreateHolder(className, getLocationForJavaGeneration());
+            className = holderWriter.getOrCreateHolder(className, getLocationForJavaGeneration(packageName));
          }
 
          buf.append(className);
@@ -711,7 +711,7 @@ public class WSDLToJava implements WSDLToJavaIntf
             if (unwrapper.xmlType != null)
                xmlType = unwrapper.xmlType;
 
-            containingElement = containingElement + unwrapper.unwrappedElement.getName();
+            containingElement = containingElement + ToolsUtils.firstLetterUpperCase(unwrapper.unwrappedElement.getName());
 
             if (unwrapper.array)
                arraySuffix = "[]";
@@ -751,7 +751,8 @@ public class WSDLToJava implements WSDLToJavaIntf
             className = className.substring(1);
          className = ToolsUtils.convertInvalidCharacters(className);
          className = utils.firstLetterUpperCase(className);
-         return seiPkgName + "." + className + arraySuffix;
+         String packageName = getPackageName(xt.getNamespace());
+         return packageName + "." + className + arraySuffix;
       }
 
       if (cls.isArray())
@@ -795,9 +796,9 @@ public class WSDLToJava implements WSDLToJavaIntf
       return paramName;
    }
 
-   private File getLocationForJavaGeneration()
+   private File getLocationForJavaGeneration(String packageName)
    {
-      return new File(this.directoryToGenerate + "/" + seiPkgName.replace(".", "/"));
+      return new File(this.directoryToGenerate + "/" + packageName.replace('.', '/'));
    }
 
    private void generateJavaSource(XSComplexTypeDefinition xt, JBossXSModel xsmodel, String containingElement) throws IOException
@@ -807,9 +808,16 @@ public class WSDLToJava implements WSDLToJavaIntf
 
    private void generateJavaSource(XSComplexTypeDefinition xt, JBossXSModel xsmodel, String containingElement, boolean exception) throws IOException
    {
-      XSDTypeToJava xtj = new XSDTypeToJava();
+      XSDTypeToJava xtj = new XSDTypeToJava(namespacePackageMap);
       xtj.setTypeMapping(this.typeMapping);
-      xtj.createJavaFile((XSComplexTypeDefinition)xt, containingElement, getLocationForJavaGeneration(), seiPkgName, xsmodel, exception);
+      String targetNS = wsdl.getTargetNamespace();
+      String tgtNS = xt.getNamespace();
+      String packName = getPackageName(tgtNS);
+      if(!tgtNS.equals(targetNS))
+      {
+          File dir = utils.createPackage(this.directoryToGenerate, packName);
+      }
+      xtj.createJavaFile((XSComplexTypeDefinition)xt, containingElement, this.directoryToGenerate, packName, xsmodel, exception);
    }
 
    public void setParameterStyle(String paramStyle)
@@ -817,4 +825,19 @@ public class WSDLToJava implements WSDLToJavaIntf
       this.parameterStyle = paramStyle;
    }
 
+   private String getPackageName(String targetNamespace)
+   {
+      //Get it from global config
+      if (namespacePackageMap != null)
+      {
+         String pkg = namespacePackageMap.get(targetNamespace);
+         if (pkg != null)
+         {
+            return pkg;
+         }
+      }
+     //return NamespacePackageMapping.getJavaPackageName(targetNamespace);
+     //Default behaviour will always generate all classes in the SEI package only
+     return seiPkgName;
+   }
 }
