@@ -23,8 +23,10 @@ package org.jboss.ws.tools.helpers;
 
 import java.beans.Introspector;
 import java.util.ArrayList;
+import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
+import java.util.Map;
 import java.util.Set;
 
 import javax.xml.namespace.QName;
@@ -76,6 +78,7 @@ import org.jboss.ws.metadata.wsdl.WSDLUtils;
 import org.jboss.ws.metadata.wsdl.xmlschema.JBossXSModel;
 import org.jboss.ws.metadata.wsdl.xsd.SchemaUtils;
 import org.jboss.ws.tools.HeaderUtil;
+import org.jboss.ws.tools.NamespacePackageMapping;
 import org.jboss.ws.tools.RPCSignature;
 import org.jboss.ws.tools.ToolsUtils;
 import org.jboss.ws.tools.WSToolsConstants;
@@ -97,6 +100,7 @@ public class MappingFileGeneratorHelper
    private String typeNamespace;
    private String serviceName = null;
    private String packageName = null;
+   private Map<String, String> namespacePackageMap = new HashMap<String,String>();
    private Set<String> registeredTypes = new HashSet<String>();
    private Set<String> registeredExceptions = new HashSet<String>();
 
@@ -107,11 +111,13 @@ public class MappingFileGeneratorHelper
 
    private String parameterStyle;
 
-   public MappingFileGeneratorHelper(WSDLDefinitions wsdl, String sname, String pname, Class seiClass, LiteralTypeMapping ltm, String paramStyle)
+   public MappingFileGeneratorHelper(WSDLDefinitions wsdl, String sname, Map<String, String> map, Class seiClass, LiteralTypeMapping ltm, String paramStyle)
    {
       this.wsdlDefinitions = wsdl;
       this.serviceName = sname;
-      this.packageName = pname;
+      String targetNS = wsdl.getTargetNamespace();
+      packageName = NamespacePackageMapping.getJavaPackageName(targetNS);
+      this.namespacePackageMap = map;
       this.typeMapping = ltm;
 
       this.wsdlStyle = utils.getWSDLStyle(wsdl);
@@ -149,7 +155,8 @@ public class MappingFileGeneratorHelper
       String targetNS = wsdlDefinitions.getTargetNamespace();
       String prefix = WSToolsConstants.WSTOOLS_CONSTANT_MAPPING_SERVICE_PREFIX;
       ServiceInterfaceMapping sim = new ServiceInterfaceMapping(jwm);
-      sim.setServiceInterface(packageName + "." + javaServiceName);
+      String className = ToolsUtils.firstLetterUpperCase(javaServiceName);
+      sim.setServiceInterface(getPackageName(targetNS) + "." + className);
       sim.setWsdlServiceName(new QName(targetNS, serviceName, prefix));
 
       WSDLEndpoint[] endpoints = ser.getEndpoints();
@@ -195,7 +202,8 @@ public class MappingFileGeneratorHelper
             javaPortName += "_PortType";
 
          ServiceEndpointInterfaceMapping seim = new ServiceEndpointInterfaceMapping(jwm);
-         seim.setServiceEndpointInterface(packageName + "." + javaPortName);
+         String className = ToolsUtils.firstLetterUpperCase(javaPortName);
+         seim.setServiceEndpointInterface(getPackageName(targetNS) + "." + className);
          seim.setWsdlPortType(new QName(targetNS, portName, "portTypeNS"));
          seim.setWsdlBinding(new QName(targetNS, bindName, "bindingNS"));
          constructServiceEndpointMethodMapping(seim, wsdlintf);
@@ -298,7 +306,7 @@ public class MappingFileGeneratorHelper
                xt = element.getTypeDefinition();
                primitive = unwrapper.primitive;
                partName = element.getName();
-               containingElement = containingElement + unwrapper.unwrappedElement.getName();
+               containingElement = containingElement + ToolsUtils.firstLetterUpperCase(unwrapper.unwrappedElement.getName());
                array = unwrapper.array;
                if (xt.getAnonymous())
                {
@@ -314,11 +322,15 @@ public class MappingFileGeneratorHelper
          //Check it is a holder.
          if (wiop.getInputByPartName(xmlName.getLocalPart()) == null)
          {
-
+            String nameSpace = null;
+            if (xt != null)
+            {
+               nameSpace = xt.getNamespace();
+            }
             if (xt instanceof XSSimpleTypeDefinition)
                xmlType = SchemaUtils.handleSimpleType((XSSimpleTypeDefinition)xt);
 
-            String javaType = getJavaTypeAsString(xmlName, xmlType, array, primitive);
+            String javaType = getJavaTypeAsString(xmlName, xmlType, nameSpace, array, primitive);
 
             if ((isDocStyle() == false && "void".equals(javaType)) == false)
             {
@@ -455,12 +467,20 @@ public class MappingFileGeneratorHelper
          QName xmlType = returnParameter.getType();
 
          XSTypeDefinition xt = schemaModel.getTypeDefinition(xmlType.getLocalPart(), xmlType.getNamespaceURI());
+         String nameSpace = null;
+         if (xt != null)
+         {
+            nameSpace = xt.getNamespace();
+         }
          if (xt instanceof XSSimpleTypeDefinition)
+         {
             xmlType = SchemaUtils.handleSimpleType((XSSimpleTypeDefinition)xt);
+         }
 
          WsdlReturnValueMapping wrvm = new WsdlReturnValueMapping(semm);
-         wrvm.setMethodReturnValue(getJavaTypeAsString(xmlName, xmlType, false, true));
+         wrvm.setMethodReturnValue(getJavaTypeAsString(xmlName, xmlType, nameSpace, false, true));
          QName messageName = WSDLUtils.getWsdl11Output(wiop).getMessageName();
+
          wrvm.setWsdlMessage(new QName(messageName.getNamespaceURI(), messageName.getLocalPart(), WSToolsConstants.WSTOOLS_CONSTANT_MAPPING_WSDL_MESSAGE_NS));
          wrvm.setWsdlMessagePartName(partName);
          semm.setWsdlReturnValueMapping(wrvm);
@@ -489,7 +509,8 @@ public class MappingFileGeneratorHelper
                if (isDocStyle())
                {
                   XSTypeDefinition xt = getXSType(input);
-                  addJavaXMLTypeMap(xt, input.getElement().getLocalPart(), "", "", jwm, !isWrapped());
+                  // Don't unwrap the actual parameter.
+                  addJavaXMLTypeMap(xt, input.getElement().getLocalPart(), "", "", jwm, false);
                }
                else
                {
@@ -503,7 +524,8 @@ public class MappingFileGeneratorHelper
                if (isDocStyle())
                {
                   XSTypeDefinition xt = getXSType(output);
-                  addJavaXMLTypeMap(xt, output.getElement().getLocalPart(), "", "", jwm, !isWrapped());
+                  // Don't unwrap the response type.
+                  addJavaXMLTypeMap(xt, output.getElement().getLocalPart(), "", "", jwm, false);
                }
                else
                {
@@ -522,7 +544,7 @@ public class MappingFileGeneratorHelper
                XSTypeDefinition xt = xsmodel.getTypeDefinition(xmlType.getLocalPart(), xmlType.getNamespaceURI());
                addJavaXMLTypeMap(xt, xmlName.getLocalPart(), "", "", jwm, true);
 
-               String exceptionType = getJavaTypeAsString(null, xmlType, false, true);
+               String exceptionType = getJavaTypeAsString(null, xmlType, xt.getNamespace(), false, true);
 
                if (registeredExceptions.contains(exceptionType) == false)
                {
@@ -630,7 +652,8 @@ public class MappingFileGeneratorHelper
             QName xmlType;
             if (type.getAnonymous())
             {
-               xmlType = new QName(type.getNamespace(), containingElement + element.getName());
+               String tempName = ToolsUtils.firstLetterUpperCase(containingElement) + ToolsUtils.firstLetterUpperCase(element.getName());
+               xmlType = new QName(type.getNamespace(), tempName);
             }
             else
             {
@@ -690,9 +713,17 @@ public class MappingFileGeneratorHelper
 
       if (xt instanceof XSComplexTypeDefinition)
       {
-
          XSModelGroup xm = null;
          XSComplexTypeDefinition xc = (XSComplexTypeDefinition)xt;
+         XSTypeDefinition baseType = xc.getBaseType();
+         short der = xc.getDerivationMethod();
+
+         if ((baseType != null) && !utils.isBaseTypeIgnorable(baseType, xc))
+         {
+               addJavaXMLTypeMap(baseType, baseType.getName(), "", "", jwm, skipWrapperArray); //Recurse for base types
+         }
+
+         // handleContentTypeElementsWithDerivationNone
          if (xc.getContentType() != XSComplexTypeDefinition.CONTENTTYPE_EMPTY)
          {
             XSParticle xp = xc.getParticle();
@@ -704,7 +735,7 @@ public class MappingFileGeneratorHelper
             }
          }
 
-         if ((skipWrapperArray && isRepresentsArray(xt)) == false)
+         if ((skipWrapperArray && SchemaUtils.isWrapperArrayType(xt)) == false)
          {
             jxtm = new JavaXmlTypeMapping(jwm);
             String javaType;
@@ -713,7 +744,8 @@ public class MappingFileGeneratorHelper
             // Anonymous
             if (localName == null)
             {
-               javaType = getJavaTypeAsString(null, new QName(containingElement + name), false, true);
+               String tempName = containingElement + ToolsUtils.firstLetterUpperCase(name);
+               javaType = getJavaTypeAsString(null, new QName(tempName), xt.getNamespace(), false, true);
                StringBuilder temp = new StringBuilder();
                if (containingType != null && containingType.length() > 0)
                   temp.append(">").append(containingType);
@@ -723,7 +755,7 @@ public class MappingFileGeneratorHelper
             }
             else
             {
-               javaType = getJavaTypeAsString(null, new QName(localName), false, true);
+               javaType = getJavaTypeAsString(null, new QName(localName), xt.getNamespace(), false, true);
                jxtm.setRootTypeQName(new QName(xt.getNamespace(), xt.getName(), "typeNS"));
             }
 
@@ -775,12 +807,15 @@ public class MappingFileGeneratorHelper
       {
          XSTerm xsterm = ((XSParticle)xo.item(i)).getTerm();
          if (xsterm instanceof XSModelGroup)
+         {
             addVariableMappingMap((XSModelGroup)xsterm, jxtm, javaType);
+         }
          else if (xsterm instanceof XSElementDeclaration)
          {
             XSElementDeclaration xe = (XSElementDeclaration)xsterm;
             VariableMapping vm = new VariableMapping(jxtm);
             String name = xe.getName();
+            
             // JBWS-1170 Convert characters which are illegal in Java identifiers
             vm.setJavaVariableName(ToolsUtils.convertInvalidCharacters(Introspector.decapitalize(name)));
             vm.setXmlElementName(name);
@@ -834,7 +869,7 @@ public class MappingFileGeneratorHelper
       }
    }
 
-   private String getJavaTypeAsString(QName xmlName, QName xmlType, boolean array, boolean primitive)
+   private String getJavaTypeAsString(QName xmlName, QName xmlType, String targetNS, boolean array, boolean primitive)
    {
       String jtype = null;
 
@@ -879,7 +914,7 @@ public class MappingFileGeneratorHelper
          if (className.charAt(0) == '>')
             className = className.substring(1);
          className = ToolsUtils.convertInvalidCharacters(className);
-         jtype = packageName + "." + utils.firstLetterUpperCase(className);
+         jtype = getPackageName(targetNS) + "." + utils.firstLetterUpperCase(className);
       }
       else
       {
@@ -899,40 +934,6 @@ public class MappingFileGeneratorHelper
       return Constants.DOCUMENT_LITERAL.equals(wsdlStyle);
    }
 
-   /**
-    * Checks whether the type represents an array type
-    *
-    * @param xst
-    * @return true: type represents an array
-    */
-   private boolean isRepresentsArray(XSTypeDefinition xst)
-   {
-      boolean bool = false;
-      if (xst instanceof XSComplexTypeDefinition)
-      {
-         XSComplexTypeDefinition xc = (XSComplexTypeDefinition)xst;
-         if (xc.getContentType() == XSComplexTypeDefinition.CONTENTTYPE_EMPTY)
-            return false;
-         XSParticle xsp = xc.getParticle();
-
-         if (xsp == null)
-            return false;
-
-         XSTerm xsterm = xsp.getTerm();
-         if (xsterm instanceof XSModelGroup)
-         {
-            XSModelGroup xm = (XSModelGroup)xsterm;
-            XSObjectList xo = xm.getParticles();
-            if (xo.getLength() == 1)
-            {
-               XSParticle xp = (XSParticle)xo.item(0);
-               bool = xp.getMaxOccursUnbounded() || xp.getMaxOccurs() > 1;
-            }
-         }
-      }
-      return bool;
-   }
-
    /**
     * Any custom decisions that need to be made will be done here
     *
@@ -961,7 +962,7 @@ public class MappingFileGeneratorHelper
       String targetNS = wsdlDefinitions.getTargetNamespace();
       MethodParamPartsMapping mppm = new MethodParamPartsMapping(semm);
       mppm.setParamPosition(paramPosition);
-      String javaType = getJavaTypeAsString(xmlName, xmlType, array, primitive);
+      String javaType = getJavaTypeAsString(xmlName, xmlType, xmlType.getNamespaceURI(), array, primitive);
       mppm.setParamType(javaType);
 
       //WSDL Message Mapping
@@ -973,4 +974,19 @@ public class MappingFileGeneratorHelper
       mppm.setWsdlMessageMapping(wmm);
       return mppm;
    }
+
+   private String getPackageName(String targetNamespace)
+   {
+      //Get it from global config
+      if (namespacePackageMap != null)
+      {
+         String pkg = namespacePackageMap.get(targetNamespace);
+         if (pkg != null)
+         {
+            return pkg;
+         }
+      }
+      //Default behaviour will always generate all classes in the SEI package only
+      return packageName;
+   }
 }
