@@ -21,7 +21,7 @@
  */
 package org.jboss.ws.tools.wsdl;
 
-// $Id: WSDL11Reader.java 3959 2007-07-20 14:44:19Z heiko.braun@jboss.com $
+// $Id: WSDL11Reader.java 5471 2008-01-15 18:14:04Z alessio.soldano@jboss.com $
 
 import java.io.File;
 import java.io.FileWriter;
@@ -31,10 +31,12 @@ import java.net.URL;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.Iterator;
 import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.Set;
 
 import javax.wsdl.Binding;
 import javax.wsdl.BindingInput;
@@ -57,6 +59,7 @@ import javax.wsdl.WSDLException;
 import javax.wsdl.extensions.ElementExtensible;
 import javax.wsdl.extensions.ExtensibilityElement;
 import javax.wsdl.extensions.UnknownExtensibilityElement;
+import javax.wsdl.extensions.http.HTTPBinding;
 import javax.wsdl.extensions.mime.MIMEContent;
 import javax.wsdl.extensions.mime.MIMEMultipartRelated;
 import javax.wsdl.extensions.mime.MIMEPart;
@@ -83,6 +86,7 @@ import org.jboss.ws.metadata.wsdl.WSDLBindingOperation;
 import org.jboss.ws.metadata.wsdl.WSDLBindingOperationInput;
 import org.jboss.ws.metadata.wsdl.WSDLBindingOperationOutput;
 import org.jboss.ws.metadata.wsdl.WSDLDefinitions;
+import org.jboss.ws.metadata.wsdl.WSDLDocumentation;
 import org.jboss.ws.metadata.wsdl.WSDLEndpoint;
 import org.jboss.ws.metadata.wsdl.WSDLExtensibilityElement;
 import org.jboss.ws.metadata.wsdl.WSDLInterface;
@@ -150,6 +154,8 @@ public class WSDL11Reader
    private Map<String, List<Binding>> bindingsByNamespace = new HashMap<String, List<Binding>>();
    private Map<String, List<PortType>> portTypesByNamespace = new HashMap<String, List<PortType>>();
    private Map<String, List<Message>> messagesByNamespace = new HashMap<String, List<Message>>();
+   
+   private Set<Definition> importedDefinitions = new HashSet<Definition>();
 
    /**
     * Takes a WSDL11 Definition element and converts into
@@ -184,6 +190,7 @@ public class WSDL11Reader
    private void processTopLevelElements(Definition srcWsdl)
    {
       String targetNS = srcWsdl.getTargetNamespace();
+      importedDefinitions.add(srcWsdl);
 
       // Messages
       Collection<Message> messages = srcWsdl.getMessages().values();
@@ -247,7 +254,10 @@ public class WSDL11Reader
          for (Import imp : imports)
          {
             Definition impWsdl = imp.getDefinition();
-            processTopLevelElements(impWsdl);
+            if (!importedDefinitions.contains(impWsdl))
+            {
+               processTopLevelElements(impWsdl);
+            }
          }
       }
    }
@@ -552,9 +562,9 @@ public class WSDL11Reader
       return tmpFile != null ? tmpFile.toURL() : null;
    }
 
-   private void handleSchemaImports(Element schemaEl, URL wsdlLoc) throws MalformedURLException, WSDLException
+   private void handleSchemaImports(Element schemaEl, URL parentURL) throws WSDLException, IOException
    {
-      if (wsdlLoc == null)
+      if (parentURL == null)
          throw new IllegalArgumentException("Cannot process import, parent location not set");
 
       Iterator it = DOMUtils.getChildElements(schemaEl, new QName(Constants.NS_SCHEMA_XSD, "import"));
@@ -569,8 +579,15 @@ public class WSDL11Reader
          // Skip, let the entity resolver resolve these
          if (namespace != null && schemaLocation != null)
          {
-            URL currLoc = getLocationURL(wsdlLoc, schemaLocation);
-            schemaLocationsMap.put(namespace, currLoc);
+            URL currLoc = getLocationURL(parentURL, schemaLocation);
+            if (schemaLocationsMap.get(namespace) == null)
+            {
+               schemaLocationsMap.put(namespace, currLoc);
+               
+               // Recursively handle schema imports
+               Element importedSchema = DOMUtils.parse(currLoc.openStream());
+               handleSchemaImports(importedSchema, currLoc);
+            }
          }
          else
          {
@@ -614,7 +631,7 @@ public class WSDL11Reader
       return locationURL;
    }
 
-   private void processPortType(Definition srcWsdl, PortType srcPortType) throws WSDLException
+   private void processPortType(Definition srcWsdl, PortType srcPortType, WSDLBinding destBinding) throws WSDLException
    {
       log.trace("processPortType: " + srcPortType.getQName());
 
@@ -636,14 +653,21 @@ public class WSDL11Reader
          {
             destInterface.addProperty(new WSDLProperty(Constants.WSDL_PROPERTY_EVENTSOURCE, eventSourceProp.getLocalPart()));
          }
+         
+         // documentation
+         Element documentationElement = srcPortType.getDocumentationElement();
+         if (documentationElement != null && documentationElement.getTextContent() != null)
+         {
+            destInterface.setDocumentationElement(new WSDLDocumentation(documentationElement.getTextContent()));
+         }
 
          destWsdl.addInterface(destInterface);
 
-         processPortTypeOperations(srcWsdl, destInterface, srcPortType);
+         processPortTypeOperations(srcWsdl, destInterface, srcPortType, destBinding);
       }
    }
 
-   private void processPortTypeOperations(Definition srcWsdl, WSDLInterface destInterface, PortType srcPortType) throws WSDLException
+   private void processPortTypeOperations(Definition srcWsdl, WSDLInterface destInterface, PortType srcPortType, WSDLBinding destBinding) throws WSDLException
    {
       Iterator itOperations = srcPortType.getOperations().iterator();
       while (itOperations.hasNext())
@@ -656,17 +680,25 @@ public class WSDL11Reader
 
          if (srcOperation.getStyle() != null && false == OperationType.NOTIFICATION.equals(srcOperation.getStyle()))
          {
-            processOperationInput(srcWsdl, srcOperation, destOperation, srcPortType);
+            processPortTypeOperationInput(srcWsdl, srcOperation, destOperation, srcPortType, destBinding);
          }
 
-         processOperationOutput(srcWsdl, srcOperation, destOperation, srcPortType);
-         processOperationFaults(srcOperation, destOperation, destInterface);
+         processPortTypeOperationOutput(srcWsdl, srcOperation, destOperation, srcPortType, destBinding);
+         processPortTypeOperationFaults(srcOperation, destOperation, destInterface, destBinding);
+         
+         // documentation
+         Element documentationElement = srcOperation.getDocumentationElement();
+         if (documentationElement != null && documentationElement.getTextContent() != null)
+         {
+            destOperation.setDocumentationElement(new WSDLDocumentation(documentationElement.getTextContent()));
+         }
 
          destInterface.addOperation(destOperation);
       }
    }
 
-   private void processOperationInput(Definition srcWsdl, Operation srcOperation, WSDLInterfaceOperation destOperation, PortType srcPortType) throws WSDLException
+   private void processPortTypeOperationInput(Definition srcWsdl, Operation srcOperation, WSDLInterfaceOperation destOperation, PortType srcPortType,
+         WSDLBinding destBinding) throws WSDLException
    {
       Input srcInput = srcOperation.getInput();
       if (srcInput != null)
@@ -702,7 +734,7 @@ public class WSDL11Reader
             if (Constants.URI_STYLE_DOCUMENT == destOperation.getStyle())
             {
                WSDLInterfaceOperationInput destInput = new WSDLInterfaceOperationInput(destOperation);
-               QName elementName = messagePartToElementName(srcMessage, srcPart, destOperation);
+               QName elementName = messagePartToElementName(srcMessage, srcPart, destOperation, destBinding);
                destInput.setElement(elementName);
 
                //Lets remember the Message name
@@ -721,8 +753,13 @@ public class WSDL11Reader
                // binding will pick it up
                QName xmlType = srcPart.getTypeName();
                if (xmlType != null)
+               {
                   rpcInput.addChildPart(new WSDLRPCPart(srcPart.getName(), destWsdl.registerQName(xmlType)));
-               else messagePartToElementName(srcMessage, srcPart, destOperation);
+               }
+               else
+               {
+                  messagePartToElementName(srcMessage, srcPart, destOperation, destBinding);
+               }
             }
          }
          if (Constants.URI_STYLE_RPC == destOperation.getStyle())
@@ -754,7 +791,8 @@ public class WSDL11Reader
       return canBeSkipped;
    }
 
-   private void processOperationOutput(Definition srcWsdl, Operation srcOperation, WSDLInterfaceOperation destOperation, PortType srcPortType) throws WSDLException
+   private void processPortTypeOperationOutput(Definition srcWsdl, Operation srcOperation, WSDLInterfaceOperation destOperation, PortType srcPortType,
+         WSDLBinding destBinding) throws WSDLException
    {
       Output srcOutput = srcOperation.getOutput();
       if (srcOutput == null)
@@ -801,7 +839,7 @@ public class WSDL11Reader
          {
             WSDLInterfaceOperationOutput destOutput = new WSDLInterfaceOperationOutput(destOperation);
 
-            QName elementName = messagePartToElementName(srcMessage, srcPart, destOperation);
+            QName elementName = messagePartToElementName(srcMessage, srcPart, destOperation, destBinding);
             destOutput.setElement(elementName);
 
             // Lets remember the Message name
@@ -821,7 +859,7 @@ public class WSDL11Reader
             QName xmlType = srcPart.getTypeName();
             if (xmlType != null)
                rpcOutput.addChildPart(new WSDLRPCPart(srcPart.getName(), destWsdl.registerQName(xmlType)));
-            else messagePartToElementName(srcMessage, srcPart, destOperation);
+            else messagePartToElementName(srcMessage, srcPart, destOperation, destBinding);
          }
       }
 
@@ -836,7 +874,8 @@ public class WSDL11Reader
       }
    }
 
-   private void processOperationFaults(Operation srcOperation, WSDLInterfaceOperation destOperation, WSDLInterface destInterface) throws WSDLException
+   private void processPortTypeOperationFaults(Operation srcOperation, WSDLInterfaceOperation destOperation, WSDLInterface destInterface, WSDLBinding destBinding)
+         throws WSDLException
    {
 
       Map faults = srcOperation.getFaults();
@@ -884,18 +923,23 @@ public class WSDL11Reader
    }
 
    /** Translate the message part name into an XML element name.
-    * @throws WSDLException
     */
-   private QName messagePartToElementName(Message srcMessage, Part srcPart, WSDLInterfaceOperation destOperation) throws WSDLException
+   private QName messagePartToElementName(Message srcMessage, Part srcPart, WSDLInterfaceOperation destOperation, WSDLBinding destBinding) throws WSDLException
    {
-      QName xmlName;
+      QName xmlName = null;
 
       // R2306 A wsdl:message in a DESCRIPTION MUST NOT specify both type and element attributes on the same wsdl:part
       if (srcPart.getTypeName() != null && srcPart.getElementName() != null)
          throw new WSDLException(WSDLException.INVALID_WSDL, "Message parts must not define an element name and type name: " + srcMessage.getQName());
 
+      String bindingType = destBinding.getType();
+      if (Constants.NS_HTTP.equals(bindingType))
+      {
+         xmlName = new QName(srcPart.getName());
+      }
+         
       String style = destOperation.getStyle();
-      if (Constants.URI_STYLE_RPC.equals(style))
+      if (xmlName == null && Constants.URI_STYLE_RPC.equals(style))
       {
          // R2203 An rpc-literal binding in a DESCRIPTION MUST refer, in its soapbind:body element(s),
          // only to wsdl:part element(s) that have been defined using the type attribute.
@@ -910,7 +954,8 @@ public class WSDL11Reader
          if (xmlName == null)
             xmlName = new QName(srcPart.getName());
       }
-      else
+
+      if (xmlName == null && Constants.URI_STYLE_DOCUMENT.equals(style))
       {
          // R2204 A document-literal binding in a DESCRIPTION MUST refer, in each of its soapbind:body element(s),
          // only to wsdl:part element(s) that have been defined using the element attribute
@@ -922,6 +967,9 @@ public class WSDL11Reader
          xmlName = srcPart.getElementName();
       }
 
+      if (xmlName == null)
+         throw new IllegalStateException("Cannot name for wsdl part: " + srcPart);
+
       xmlName = destWsdl.registerQName(xmlName);
       String key = srcMessage.getQName() + "->" + srcPart.getName();
       messagePartToElementMap.put(key, xmlName);
@@ -986,16 +1034,16 @@ public class WSDL11Reader
       return ("rpc".equals(operationStyle)) ? Constants.URI_STYLE_RPC : Constants.URI_STYLE_DOCUMENT;
    }
 
-   private boolean processBinding(Definition srcWsdl, Binding srcBinding) throws WSDLException
+   private WSDLBinding processBinding(Definition srcWsdl, Binding srcBinding) throws WSDLException
    {
       QName srcBindingQName = srcBinding.getQName();
       log.trace("processBinding: " + srcBindingQName);
 
-      if (destWsdl.getBinding(srcBindingQName) == null)
+      WSDLBinding destBinding = destWsdl.getBinding(srcBindingQName);
+      if (destBinding == null)
       {
          PortType srcPortType = getDefinedPortType(srcBinding);
 
-         // Get binding type
          String bindingType = null;
          List<ExtensibilityElement> extList = srcBinding.getExtensibilityElements();
          for (ExtensibilityElement extElement : extList)
@@ -1009,6 +1057,10 @@ public class WSDL11Reader
             {
                bindingType = Constants.NS_SOAP12;
             }
+            else if (extElement instanceof HTTPBinding)
+            {
+               bindingType = Constants.NS_HTTP;
+            }
             else if ("binding".equals(elementType.getLocalPart()))
             {
                log.warn("Unsupported binding: " + elementType);
@@ -1019,41 +1071,37 @@ public class WSDL11Reader
          if (bindingType == null)
             throw new WSDLException(WSDLException.INVALID_WSDL, "Cannot obtain binding type for: " + srcBindingQName);
 
-         // Ignore unknown bindings
-         if (Constants.NS_SOAP11.equals(bindingType) == false && Constants.NS_SOAP12.equals(bindingType) == false)
-            return false;
-
-         WSDLBinding destBinding = new WSDLBinding(destWsdl, srcBindingQName);
-         destBinding.setInterfaceName(srcPortType.getQName());
-         destBinding.setType(bindingType);
-         processUnknownExtensibilityElements(srcBinding, destBinding);
-         destWsdl.addBinding(destBinding);
-
-         // mark SWA Parts upfront
-         preProcessSWAParts(srcBinding, srcWsdl);
-
-         processPortType(srcWsdl, srcPortType);
-
-         String bindingStyle = Style.getDefaultStyle().toString();
-         for (ExtensibilityElement extElement : extList)
+         if (Constants.NS_SOAP11.equals(bindingType) || Constants.NS_SOAP12.equals(bindingType) || Constants.NS_HTTP.equals(bindingType))
          {
-            QName elementType = extElement.getElementType();
-            if (extElement instanceof SOAPBinding)
-            {
-               SOAPBinding soapBinding = (SOAPBinding)extElement;
-               bindingStyle = soapBinding.getStyle();
-            }
-            else if (extElement instanceof SOAP12Binding)
-            {
-               SOAP12Binding soapBinding = (SOAP12Binding)extElement;
-               bindingStyle = soapBinding.getStyle();
-            }
-         }
+            destBinding = new WSDLBinding(destWsdl, srcBindingQName);
+            destBinding.setInterfaceName(srcPortType.getQName());
+            destBinding.setType(bindingType);
+            processUnknownExtensibilityElements(srcBinding, destBinding);
+            destWsdl.addBinding(destBinding);
 
-         processBindingOperations(srcWsdl, destBinding, srcBinding, bindingStyle);
+            preProcessSWAParts(srcBinding, srcWsdl);
+            processPortType(srcWsdl, srcPortType, destBinding);
+
+            String bindingStyle = Style.getDefaultStyle().toString();
+            for (ExtensibilityElement extElement : extList)
+            {
+               if (extElement instanceof SOAPBinding)
+               {
+                  SOAPBinding soapBinding = (SOAPBinding)extElement;
+                  bindingStyle = soapBinding.getStyle();
+               }
+               else if (extElement instanceof SOAP12Binding)
+               {
+                  SOAP12Binding soapBinding = (SOAP12Binding)extElement;
+                  bindingStyle = soapBinding.getStyle();
+               }
+            }
+
+            processBindingOperations(srcWsdl, destBinding, srcBinding, bindingStyle);
+         }
       }
 
-      return true;
+      return destBinding;
    }
 
    /** The port might reference a binding which is defined in another wsdl
@@ -1071,7 +1119,7 @@ public class WSDL11Reader
          List<Binding> bindings = bindingsByNamespace.get(nsURI);
          if (bindings == null)
             throw new WSDLException(WSDLException.INVALID_WSDL, "Cannot find bindings for namespace: " + nsURI);
-         
+
          for (Binding auxBinding : bindings)
          {
             if (srcBindingName.equals(auxBinding.getQName()))
@@ -1081,7 +1129,7 @@ public class WSDL11Reader
             }
          }
       }
-      
+
       return srcBinding;
    }
 
@@ -1557,7 +1605,8 @@ public class WSDL11Reader
       destEndpoint.setAddress(getSOAPAddress(srcPort));
       processUnknownExtensibilityElements(srcPort, destEndpoint);
 
-      if (processBinding(srcWsdl, srcBinding))
+      WSDLBinding destBinding = processBinding(srcWsdl, srcBinding);
+      if (destBinding != null)
          destService.addEndpoint(destEndpoint);
    }
 
