@@ -52,7 +52,7 @@ import EDU.oswego.cs.dl.util.concurrent.ConcurrentReaderHashMap;
  * A RepositoryClassLoader.
  * 
  * @author <a href="adrian@jboss.com">Adrian Brock</a>
- * @version $Revision: 63960 $
+ * @version $Revision: 68461 $
  */
 public abstract class RepositoryClassLoader extends URLClassLoader
 {
@@ -617,6 +617,14 @@ public abstract class RepositoryClassLoader extends URLClassLoader
      */
    protected Class findClass(String name) throws ClassNotFoundException
    {
+      if( repository == null )
+      {
+         String msg = "Invalid use of destroyed classloader for " + name + ", UCL destroyed at:";
+         ClassNotFoundException e = new ClassNotFoundException(msg);
+         e.initCause(this.unregisterTrace);
+         throw e;
+      }
+
       boolean trace = log.isTraceEnabled();
       if( trace )
          log.trace("findClass, name="+name);
