@@ -44,7 +44,7 @@ import EDU.oswego.cs.dl.util.concurrent.SynchronizedBoolean;
  * @author Hiram Chirino (Cojonudo14@hotmail.com)
  * @author David Maplesden (David.Maplesden@orion.co.nz)
  * @author <a href="mailto:adrian@jboss.org">Adrian Brock</a>
- * @version $Revision: 58406 $
+ * @version $Revision: 74987 $
  */
 public class SpyMessageConsumer implements MessageConsumer, SpyConsumer, Runnable
 {
@@ -525,6 +525,21 @@ public class SpyMessageConsumer implements MessageConsumer, SpyConsumer, Runnabl
 
          if (trace)      
             log.trace("Message consumer closing. " + this);
+         
+         while (messages.isEmpty() == false)
+         {
+            SpyMessage mes = (SpyMessage) messages.removeFirst();
+            if (trace)
+               log.trace("close() nacking undelivered message mes=" + mes.getJMSMessageID() + " " + this); 
+            try
+            {
+               session.connection.send(mes.getAcknowledgementRequest(false));
+            }
+            catch (Exception e)
+            {
+               log.debug("Error nacking message: " + mes.getJMSMessageID(), e);
+            }
+         }
          messages.notifyAll();
       }
       
