@@ -54,6 +54,7 @@ import org.jboss.mq.SpyJMSException;
 import org.jboss.mq.SpyMessage;
 import org.jboss.mq.SpyTopic;
 import org.jboss.mq.pm.CacheStore;
+import org.jboss.mq.pm.PersistenceManagerExt;
 import org.jboss.mq.pm.Tx;
 import org.jboss.mq.pm.TxManager;
 import org.jboss.mq.server.JMSDestination;
@@ -73,10 +74,10 @@ import EDU.oswego.cs.dl.util.concurrent.SynchronizedLong;
  * @author Jayesh Parayali (jayeshpk1@yahoo.com)
  * @author Hiram Chirino (cojonudo14@hotmail.com)
  * @author Adrian Brock (adrian@jboss.com)
- * @version $Revision: 64570 $
+ * @version $Revision: 73847 $
  */
 public class PersistenceManager extends ServiceMBeanSupport
-   implements PersistenceManagerMBean, org.jboss.mq.pm.PersistenceManager, CacheStore
+   implements PersistenceManagerMBean, org.jboss.mq.pm.PersistenceManager, CacheStore, PersistenceManagerExt
 {
    /** FAQ about concurrency problems */
    private static String CONCURRENCY_WARNING = "\nCommon reasons for missing messages are \n1) You have multiple JBossMQs running over the same database.\n2) You are using a replicating database that is not keeping up with replication.";
@@ -646,8 +647,8 @@ public class PersistenceManager extends ServiceMBeanSupport
                      message = extractMessage(rs2);
                      if (xaRecovery)
                      {
-                        txid = rs.getLong(3);
-                        txop = rs.getString(4);
+                        txid = rs2.getLong(3);
+                        txop = rs2.getString(4);
                      }
                   }
                   else
@@ -822,6 +823,45 @@ public class PersistenceManager extends ServiceMBeanSupport
          throw new IOException("Could not load the message: " + e);
       }
    }
+   
+   public void forcePersistentTx(Tx txId) throws javax.jms.JMSException
+   {
+      // No need if already done or not doing xa recovery
+      if (txId.wasPersisted() || xaRecovery == false)
+         return;
+      
+      TransactionManagerStrategy tms = new TransactionManagerStrategy();
+      tms.startTX();
+      Connection c = null;
+      boolean threadWasInterrupted = Thread.interrupted();
+      try
+      {
+
+         c = this.getConnection();
+         insertPersistentTx(tms, c, txId);
+      }
+      catch (SQLException e)
+      {
+         tms.setRollbackOnly();
+         throw new SpyJMSException("Could not commit tx: " + txId, e);
+      }
+      finally
+      {
+         try
+         {
+            if (c != null)
+               c.close();
+         }
+         catch (Throwable ignore)
+         {
+         }
+         tms.endTX();
+
+         // Restore the interrupted state of the thread
+         if (threadWasInterrupted)
+            Thread.currentThread().interrupt();
+      }
+   }
 
    /////////////////////////////////////////////////////////////////////////////////
    //
