@@ -71,7 +71,7 @@ import EDU.oswego.cs.dl.util.concurrent.CopyOnWriteArraySet;
  * @author     David Maplesden (David.Maplesden@orion.co.nz)
  * @author     Adrian Brock (Adrian@jboss.org)
  * @created    August 16, 2001
- * @version    $Revision: 61446 $
+ * @version    $Revision: 69770 $
  */
 public class BasicQueue
 {
@@ -211,40 +211,48 @@ public class BasicQueue
       if (trace)
          log.trace("addReceiver " + sub + " " + this);
       
-      MessageReference found = null;
-      synchronized (messages)
+      synchronized(receivers)
       {
-         if (messages.size() != 0)
+         MessageReference found = null;
+         synchronized (messages)
          {
-            for (Iterator it = messages.iterator(); it.hasNext();)
+            if (messages.size() != 0)
             {
-               MessageReference message = (MessageReference) it.next();
-               try
+               for (Iterator it = messages.iterator(); it.hasNext();)
                {
-                  if (message.isExpired())
+                  MessageReference message = (MessageReference) it.next();
+                  try
                   {
-                     it.remove();
-                     expireMessageAsync(message);
+                     if (message.isExpired())
+                     {
+                        it.remove();
+                        expireMessageAsync(message);
+                     }
+                     else if (sub.accepts(message.getHeaders()))
+                     {
+                        //queue message for sending to this sub
+                        it.remove();
+                        found = message;
+                        break;
+                     }
                   }
-                  else if (sub.accepts(message.getHeaders()))
+                  catch (JMSException ignore)
                   {
-                     //queue message for sending to this sub
-                     it.remove();
-                     found = message;
-                     break;
+                     log.info("Caught unusual exception in addToReceivers.", ignore);
                   }
                }
-               catch (JMSException ignore)
-               {
-                  log.info("Caught unusual exception in addToReceivers.", ignore);
-               }
             }
+            if (found != null)
+               setupMessageAcknowledgement(sub, found);
          }
+         if (found != null)
+         {
+            updateRedeliveryFlags(found);
+            queueMessageForSending(sub, found);
+         }
+         else
+            addToReceivers(sub);
       }
-      if (found != null)
-         queueMessageForSending(sub, found);
-      else
-         addToReceivers(sub);
    }
 
    /**
@@ -473,13 +481,17 @@ public class BasicQueue
 
       try
       {
-         message.redelivered();
          // Set redelivered, vendor-specific flags
-         message.invalidate();
-         // Update the persistent message outside the transaction
-         // We want to know the message might have been delivered regardless
-         if (message.isPersistent())
-            server.getPersistenceManager().update(message, null);
+         message.redelivered();
+         // Old style update - doesn't survive a crash
+         if (server.isLazyRedeliveryUpdate())
+         {
+            message.invalidate();
+            // Update the persistent message outside the transaction
+            // We want to know the message might have been delivered regardless
+            if (message.isPersistent())
+               server.getPersistenceManager().update(message, null);
+         }
       }
       catch (JMSException e)
       {
@@ -664,6 +676,9 @@ public class BasicQueue
                   else
                      break;
                }
+               
+               if (messageRef != null)
+                  setupMessageAcknowledgement(sub, messageRef);
             }
          }
          else
@@ -687,6 +702,9 @@ public class BasicQueue
                      break;
                   }
                }
+               
+               if (messageRef != null)
+                  setupMessageAcknowledgement(sub, messageRef);
             }
          }
 
@@ -695,14 +713,11 @@ public class BasicQueue
             if (wait)
                addToReceivers(sub);
          }
-         else
-         {
-            setupMessageAcknowledgement(sub, messageRef);
-         }
       }
 
       if (messageRef == null)
          return null;
+      updateRedeliveryFlags(messageRef);
       return messageRef.getMessageForDelivery();
    }
 
@@ -1090,9 +1105,9 @@ public class BasicQueue
                }
             }
 
-            if (found == null)
+            synchronized (messages)
             {
-               synchronized (messages)
+               if (found == null)
                {
                   messages.add(message);
 
@@ -1103,12 +1118,19 @@ public class BasicQueue
                      addTimeout(message, new ExpireMessageTask(message), message.messageExpiration);
                   }
                }
+               else
+               {
+                  setupMessageAcknowledgement(found, message);
+               }
             }
          }
          
          // Queue to the receiver
          if (found != null)
+         {
+            updateRedeliveryFlags(message);
             queueMessageForSending(found, message);
+         }
       }
       catch (JMSException e)
       {
@@ -1133,7 +1155,6 @@ public class BasicQueue
 
       try
       {
-         setupMessageAcknowledgement(sub, message);
          RoutedMessage r = new RoutedMessage();
          r.message = message;
          r.subscriptionId = new Integer(sub.subscriptionId);
@@ -1159,19 +1180,38 @@ public class BasicQueue
       nack.destination = message.getJMSDestination();
       nack.messageID = message.getJMSMessageID();
       nack.subscriberId = sub.subscriptionId;
-
-      synchronized (messages)
+      
+      UnackedMessageInfo unacked = new UnackedMessageInfo(messageRef, sub);
+      unacknowledgedMessages.put(nack, unacked);
+      unackedByMessageRef.put(messageRef, nack);
+      HashMap map = (HashMap) unackedBySubscription.get(sub);
+      if (map == null)
       {
-         UnackedMessageInfo unacked = new UnackedMessageInfo(messageRef, sub);
-         unacknowledgedMessages.put(nack, unacked);
-         unackedByMessageRef.put(messageRef, nack);
-         HashMap map = (HashMap) unackedBySubscription.get(sub);
-         if (map == null)
-         {
-            map = new HashMap();
-            unackedBySubscription.put(sub, map);
-         }
-         map.put(messageRef, nack);
+         map = new HashMap();
+         unackedBySubscription.put(sub, map);
+      }
+      map.put(messageRef, nack);
+   }
+
+   /**
+    * Update redelivery flags
+    *
+    * @param messageRef the message to be acknowledged
+    * @throws JMSException for any error
+    */
+   protected void updateRedeliveryFlags(MessageReference messageRef) throws JMSException
+   {
+      // Need to update the message so it recovers from crash with redelivery=true 
+      if (server.isLazyRedeliveryUpdate() == false && messageRef.isPersistent())
+      {
+         // Temporarily set the flags
+         messageRef.redelivered();
+         // Update the persistent message outside the transaction
+         // We want to know the message might have been delivered regardless
+         messageRef.invalidate();
+         server.getPersistenceManager().update(messageRef, null);
+         // Now revert the flags back again (but only in memory)
+         messageRef.revertRedelivered();
       }
    }
 
