@@ -65,7 +65,7 @@ import EDU.oswego.cs.dl.util.concurrent.ConcurrentReaderHashMap;
  * @author    Hiram Chirino (Cojonudo14@hotmail.com)
  * @author    David Maplesden (David.Maplesden@orion.co.nz)
  * @author <a href="mailto:pra@tim.se">Peter Antman</a>
- * @version   $Revision: 57198 $
+ * @version   $Revision: 66791 $
  */
 public class JMSDestinationManager extends JMSServerInterceptorSupport
 {
@@ -102,7 +102,7 @@ public class JMSDestinationManager extends JMSServerInterceptorSupport
    private int lastTemporaryQueue = 1;
 
    private Object lastTemporaryQueueLock = new Object();
-
+   
    /** The security manager */
    private StateManager stateManager;
 
@@ -112,6 +112,9 @@ public class JMSDestinationManager extends JMSServerInterceptorSupport
    /** The Cache Used to hold messages */
    private MessageCache messageCache;
 
+   /** Whether to do the redelivery update lazily */
+   private boolean lazyRedeliveryUpdate;
+
    private Object stateLock = new Object();
 
    private Object idLock = new Object();
@@ -167,6 +170,26 @@ public class JMSDestinationManager extends JMSServerInterceptorSupport
       persistenceManager = newPersistenceManager;
    }
 
+   /**
+    * Get the lazyRedeliveryUpdate.
+    * 
+    * @return the lazyRedeliveryUpdate.
+    */
+   public boolean isLazyRedeliveryUpdate()
+   {
+      return lazyRedeliveryUpdate;
+   }
+
+   /**
+    * Set the lazyRedeliveryUpdate.
+    * 
+    * @param lazyRedeliveryUpdate the lazyRedeliveryUpdate.
+    */
+   public void setLazyRedeliveryUpdate(boolean lazyRedeliveryUpdate)
+   {
+      this.lazyRedeliveryUpdate = lazyRedeliveryUpdate;
+   }
+
    /**
     * Returns <code>false</code> if the JMS server is currently running and
     * handling requests, <code>true</code> otherwise.
@@ -409,6 +432,7 @@ public class JMSDestinationManager extends JMSServerInterceptorSupport
       // Reset any redelivered information
       val.setJMSRedelivered(false);
       val.header.jmsProperties.remove(SpyMessage.PROPERTY_REDELIVERY_COUNT);
+      val.header.jmsProperties.remove(SpyMessage.PROPERTY_DELIVERY_COUNT);
 
       //Add the message to the queue
       val.setReadOnlyMode();
