@@ -57,7 +57,7 @@ import org.jboss.util.threadpool.ThreadPool;
  * @author     Vincent Sheffer (vsheffer@telkel.com)
  * @author     <a href="mailto:jplindfo@helsinki.fi">Juha Lindfors</a>
  * @author     <a href="hiram.chirino@jboss.org">Hiram Chirino</a>
- * @version    $Revision: 57198 $
+ * @version    $Revision: 66418 $
  */
 public class DestinationManager extends InterceptorMBeanSupport implements DestinationManagerMBean
 {
@@ -81,6 +81,8 @@ public class DestinationManager extends InterceptorMBeanSupport implements Desti
    private ObjectName threadPool;
    /** Default expiry destination */
    private ObjectName expiryDestination;
+   /** Whether to do the redelivery update lazily */
+   private boolean lazyRedeliveryUpdate = false;
 
    /**
     * @jmx:managed-attribute
@@ -263,6 +265,26 @@ public class DestinationManager extends InterceptorMBeanSupport implements Desti
       this.expiryDestination = expiryDestination;
    }
 
+   /**
+    * Get the lazyRedeliveryUpdate.
+    * 
+    * @return the lazyRedeliveryUpdate.
+    */
+   public boolean isLazyRedeliveryUpdate()
+   {
+      return lazyRedeliveryUpdate;
+   }
+
+   /**
+    * Set the lazyRedeliveryUpdate.
+    * 
+    * @param lazyRedeliveryUpdate the lazyRedeliveryUpdate.
+    */
+   public void setLazyRedeliveryUpdate(boolean lazyRedeliveryUpdate)
+   {
+      this.lazyRedeliveryUpdate = lazyRedeliveryUpdate;
+   }
+
    /**
     * @jmx:managed-operation
     */
@@ -384,6 +406,7 @@ public class DestinationManager extends InterceptorMBeanSupport implements Desti
    {
       super.createService();
       jmsServer = new JMSDestinationManager(tempParameters);
+      jmsServer.setLazyRedeliveryUpdate(lazyRedeliveryUpdate);
    }
 
    protected void startService() throws Exception
@@ -394,7 +417,7 @@ public class DestinationManager extends InterceptorMBeanSupport implements Desti
             ServiceControllerMBean.class,
             ServiceControllerMBean.OBJECT_NAME,
             server);
-
+      
       PersistenceManager pm = (PersistenceManager) server.getAttribute(persistenceManager, "Instance");
       jmsServer.setPersistenceManager(pm);
 
