@@ -883,14 +883,28 @@ public class ConcurrentHashMap
     }
     public Object[] toArray() {
       Collection c = new ArrayList();
+      Object o=null;
       for (Iterator i = iterator(); i.hasNext(); )
-          c.add(i.next());
+      {
+    	  o = i.next();
+    	  if(o!=null)
+    	  {
+    		  c.add(o);
+    	  } 
+      }
       return c.toArray();
     }
     public Object[] toArray(Object[] a) {
       Collection c = new ArrayList();
+      Object o=null;
       for (Iterator i = iterator(); i.hasNext(); )
-          c.add(i.next());
+      {
+    	  o = i.next();
+    	  if(o!=null)
+    	  {
+    		  c.add(o);
+    	  } 
+      }
       return c.toArray(a);
     }
   }
@@ -927,14 +941,28 @@ public class ConcurrentHashMap
     }
     public Object[] toArray() {
       Collection c = new ArrayList();
+      Object o=null;
       for (Iterator i = iterator(); i.hasNext(); )
-          c.add(i.next());
+      {
+    	  o = i.next();
+    	  if(o!=null)
+    	  {
+    		  c.add(o);
+    	  } 
+      }
       return c.toArray();
     }
     public Object[] toArray(Object[] a) {
       Collection c = new ArrayList();
+      Object o=null;
       for (Iterator i = iterator(); i.hasNext(); )
-          c.add(i.next());
+      {
+    	  o = i.next();
+    	  if(o!=null)
+    	  {
+    		  c.add(o);
+    	  } 
+      }
       return c.toArray(a);
     }
   }
@@ -982,14 +1010,28 @@ public class ConcurrentHashMap
     }
     public Object[] toArray() {
       Collection c = new ArrayList();
+      Object o=null;
       for (Iterator i = iterator(); i.hasNext(); )
-          c.add(i.next());
+      {
+    	  o = i.next();
+    	  if(o!=null)
+    	  {
+    		  c.add(o);
+    	  } 
+      }
       return c.toArray();
     }
     public Object[] toArray(Object[] a) {
       Collection c = new ArrayList();
+      Object o=null;
       for (Iterator i = iterator(); i.hasNext(); )
-          c.add(i.next());
+      {
+    	  o = i.next();
+    	  if(o!=null)
+    	  {
+    		  c.add(o);
+    	  } 
+      }
       return c.toArray(a);
     }
   }
