@@ -1,24 +1,24 @@
 /*
- * JBoss, Home of Professional Open Source.
- * Copyright 2006, Red Hat Middleware LLC, and individual contributors
- * as indicated by the @author tags. See the copyright.txt file in the
- * distribution for a full listing of individual contributors.
- *
- * This is free software; you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License as
- * published by the Free Software Foundation; either version 2.1 of
- * the License, or (at your option) any later version.
- *
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this software; if not, write to the Free
- * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
- * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
- */
+* JBoss, Home of Professional Open Source
+* Copyright 2005, JBoss Inc., and individual contributors as indicated
+* by the @authors tag. See the copyright.txt in the distribution for a
+* full listing of individual contributors.
+*
+* This is free software; you can redistribute it and/or modify it
+* under the terms of the GNU Lesser General Public License as
+* published by the Free Software Foundation; either version 2.1 of
+* the License, or (at your option) any later version.
+*
+* This software is distributed in the hope that it will be useful,
+* but WITHOUT ANY WARRANTY; without even the implied warranty of
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+* Lesser General Public License for more details.
+*
+* You should have received a copy of the GNU Lesser General Public
+* License along with this software; if not, write to the Free
+* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
+*/
 package org.jboss.ejb.plugins.cmp.jdbc;
 
 import java.io.StringReader;
@@ -90,7 +90,7 @@ import org.jboss.deployment.DeploymentException;
  *
  * @author <a href="mailto:dain@daingroup.com">Dain Sundstrom</a>
  * @author <a href="mailto:alex@jboss.org">Alex Loubyansky</a>
- * @version $Revision: 63343 $
+ * @version $Revision: 67396 $
  */
 public final class JDBCEJBQLCompiler extends BasicVisitor implements QLCompiler
 {
@@ -1041,7 +1041,7 @@ public final class JDBCEJBQLCompiler extends BasicVisitor implements QLCompiler
             selectManager = (JDBCStoreManager) selectEntity.getManager();
             selectObject = selectEntity;
             setTypeFactory(selectManager.getJDBCTypeFactory());
-            selectEntity(path, buf);
+            selectEntity(path, node.distinct, buf);
          }
       }
       else
@@ -1563,7 +1563,7 @@ public final class JDBCEJBQLCompiler extends BasicVisitor implements QLCompiler
          {
             countCompositePk = true;
             forceDistinct = node.distinct.length() > 0;
-            selectEntity(cntPath, buf);
+            selectEntity(cntPath, forceDistinct, buf);
             return buf;
          }
          else
@@ -1642,7 +1642,8 @@ public final class JDBCEJBQLCompiler extends BasicVisitor implements QLCompiler
       {
          case EJBQLTypes.ENTITY_TYPE:
          case EJBQLTypes.VALUE_CLASS_TYPE:
-            if(cmpField.getJDBCType().hasMapper())
+            if(cmpField.getJDBCType().hasMapper() ||
+               cmpField.getJDBCType().getParameterSetter() != null)
             {
                break;
             }
@@ -1862,7 +1863,7 @@ public final class JDBCEJBQLCompiler extends BasicVisitor implements QLCompiler
       return selected;
    }
 
-   private void selectEntity(ASTPath path, StringBuffer buf)
+   private void selectEntity(ASTPath path, boolean distinct, StringBuffer buf)
    {
       JDBCEntityBridge selectEntity = (JDBCEntityBridge) path.getEntity();
 
@@ -1878,7 +1879,10 @@ public final class JDBCEJBQLCompiler extends BasicVisitor implements QLCompiler
       {
          String eagerLoadGroupName = readAhead.getEagerLoadGroup();
          boolean[] loadGroupMask = selectEntity.getLoadGroupMask(eagerLoadGroupName);
-         SQLUtil.appendColumnNamesClause(selectEntity.getTableFields(), loadGroupMask, selectAlias, columnNamesClause);
+         if(distinct)
+            SQLUtil.appendSearchableColumnNamesClause(selectEntity.getTableFields(), loadGroupMask, selectAlias, columnNamesClause);
+         else
+            SQLUtil.appendColumnNamesClause(selectEntity.getTableFields(), loadGroupMask, selectAlias, columnNamesClause);
 
          try
          {
