@@ -1,24 +1,24 @@
 /*
- * JBoss, Home of Professional Open Source.
- * Copyright 2006, Red Hat Middleware LLC, and individual contributors
- * as indicated by the @author tags. See the copyright.txt file in the
- * distribution for a full listing of individual contributors.
- *
- * This is free software; you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License as
- * published by the Free Software Foundation; either version 2.1 of
- * the License, or (at your option) any later version.
- *
- * This software is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this software; if not, write to the Free
- * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
- * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
- */
+* JBoss, Home of Professional Open Source
+* Copyright 2005, JBoss Inc., and individual contributors as indicated
+* by the @authors tag. See the copyright.txt in the distribution for a
+* full listing of individual contributors.
+*
+* This is free software; you can redistribute it and/or modify it
+* under the terms of the GNU Lesser General Public License as
+* published by the Free Software Foundation; either version 2.1 of
+* the License, or (at your option) any later version.
+*
+* This software is distributed in the hope that it will be useful,
+* but WITHOUT ANY WARRANTY; without even the implied warranty of
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+* Lesser General Public License for more details.
+*
+* You should have received a copy of the GNU Lesser General Public
+* License along with this software; if not, write to the Free
+* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
+*/
 package org.jboss.ejb.plugins.cmp.jdbc;
 
 import java.sql.Connection;
@@ -49,7 +49,7 @@ import org.jboss.logging.Logger;
  * JDBCStartCommand creates the table if specified in xml.
  *
  * @author <a href="mailto:dain@daingroup.com">Dain Sundstrom</a>
- * @author <a href="mailto:rickard.oberg@telkel.com">Rickard berg</a>
+ * @author <a href="mailto:rickard.oberg@telkel.com">Rickard Oberg</a>
  * @author <a href="mailto:marc.fleury@telkel.com">Marc Fleury</a>
  * @author <a href="mailto:shevlandj@kpi.com.au">Joe Shevland</a>
  * @author <a href="mailto:justin@j-m-f.demon.co.uk">Justin Forder</a>
@@ -57,7 +57,7 @@ import org.jboss.logging.Logger;
  * @author <a href="mailto:alex@jboss.org">Alex Loubyansky</a>
  * @author <a href="mailto:heiko.rupp@cellent.de">Heiko W.Rupp</a>
  * @author <a href="mailto:joachim@cabsoft.be">Joachim Van der Auwera</a>
- * @version $Revision: 63140 $
+ * @version $Revision: 73362 $
  */
 public final class JDBCStartCommand
 {
@@ -249,13 +249,12 @@ public final class JDBCStartCommand
       {
          JDBCAbstractCMRFieldBridge cmrField = cmrFields[i];
          JDBCRelationMetaData relationMetaData = cmrField.getMetaData().getRelationMetaData();
+         DataSource dataSource = relationMetaData.getDataSource();
 
          // if the table for the related entity has been created
          final EntityBridge relatedEntity = cmrField.getRelatedEntity();
          if(relationMetaData.isTableMappingStyle() && createdTables.contains(relatedEntity.getEntityName()))
          {
-            DataSource dataSource = relationMetaData.getDataSource();
-
             boolean relTableExisted = SQLUtil.tableExists(cmrField.getQualifiedTableName(), entity.getDataSource());
 
             if(relTableExisted)
@@ -323,6 +322,7 @@ public final class JDBCStartCommand
                {
                   log.debug("Relation table not created as requested: " + cmrField.getQualifiedTableName());
                }
+
                // create Indices if needed
                createCMRIndex(dataSource, cmrField);
 
@@ -339,27 +339,21 @@ public final class JDBCStartCommand
 
    public void addForeignKeyConstraints() throws DeploymentException
    {
-      // Create table if necessary
       Set createdTables = getCreatedTables(manager);
 
       JDBCAbstractCMRFieldBridge[] cmrFields = entity.getCMRFields();
       for(int i = 0; i < cmrFields.length; ++i)
       {
          JDBCAbstractCMRFieldBridge cmrField = cmrFields[i];
+         EntityBridge relatedEntity = cmrField.getRelatedEntity();
          JDBCRelationMetaData relationMetaData = cmrField.getMetaData().getRelationMetaData();
 
-         // if the table for the related entity has been created
-         final EntityBridge relatedEntity = cmrField.getRelatedEntity();
-
-         // Only generate indices on foreign key columns if
-         // the table was freshly created. If not, we risk
-         // creating an index twice and get an exception from the DB
-         if(relationMetaData.isForeignKeyMappingStyle() && createdTables.contains(relatedEntity.getEntityName()))
+         if(relationMetaData.isForeignKeyMappingStyle() && (createdTables.contains(relatedEntity.getEntityName())))
          {
             createCMRIndex(((JDBCAbstractEntityBridge)relatedEntity).getDataSource(), cmrField);
          }
 
-         // Create my fk constraint
+         // Create fk constraint
          addForeignKeyConstraint(cmrField);
       }
    }
@@ -424,58 +418,23 @@ public final class JDBCStartCommand
       }
 
       return true;
-/*
-previous implementation
-      ArrayList idxAscDesc = oldIndexes.getColumnAscDesc();
+   }
 
-      // search for for column in index
-      if(idxAscDesc != null)
+   /**
+    * Check whether a required index already exists on a table
+    *
+    * @param oldIndexes list of existing indexes
+    * @param column     column we test the existence of an index for
+    * @return True if the column has an index; otherwise false
+    */
+   private boolean hasIndex(SQLUtil.OldIndexes oldIndexes, String column)
+   {
+      ArrayList idxColumns = oldIndexes.getColumnNames();
+      if(columnIndex(idxColumns, column) == -1)
       {
-         for(int i = 0; i < idxColumns.size(); i++)
-         {
-            // only match ascending columns
-            if("A".equals(idxAscDesc.get(i)))
-            {
-               String name = columns[0];
-               String testCol = (String) idxColumns.get(i);
-               if(testCol.charAt(0) == '\"' &&
-                  testCol.charAt(testCol.charAt(testCol.length() - 1)) == '\"')
-               {
-                  testCol = testCol.substring(1, testCol.length() - 1);
-               }
-
-               if(testCol.equalsIgnoreCase(name))
-               {
-                  // first column matches, now check the others
-                  String idxName = (String) idxNames.get(i);
-                  int j = 1;
-                  for(; j < columns.length; j++)
-                  {
-                     name = columns[j];
-                     testCol = (String) idxColumns.get(i + j);
-                     if(testCol.charAt(0) == '\"' &&
-                        testCol.charAt(testCol.charAt(testCol.length() - 1)) == '\"')
-                     {
-                        testCol = testCol.substring(1, testCol.length() - 1);
-                     }
-
-                     String testName = (String) idxNames.get(i + j);
-                     if(!(testName.equals(idxName)
-                        &&
-                        testCol.equalsIgnoreCase(name)
-                        && idxAscDesc.get(i + j).equals("A")))
-                     {
-                        break;
-                     }
-                  }
-                  // if they all matched -> found
-                  if(j == columns.length) return true;
-               }
-            }
-         }
+         return false;
       }
-      return false;
-*/
+      return true;
    }
 
    private int columnIndex(ArrayList idxColumns, String column)
@@ -483,11 +442,14 @@ previous implementation
       for(int j = 0; j < idxColumns.size(); ++j)
       {
          String idxColumn = (String)idxColumns.get(j);
-         idxColumn = idxColumn.trim();         
-         while(idxColumn.charAt(0) == '\"' &&
-            idxColumn.charAt(idxColumn.charAt(idxColumn.length() - 1)) == '\"')
+         idxColumn = idxColumn.trim();
+         while(idxColumn.startsWith("\""))
          {
-            idxColumn = idxColumn.substring(1, idxColumn.length() - 1);
+            idxColumn = idxColumn.substring(1);
+         }
+         while(idxColumn.endsWith("\""))
+         {
+            idxColumn = idxColumn.substring(0, idxColumn.length() - 1);
          }
 
          if(idxColumn.equalsIgnoreCase(column))
@@ -505,7 +467,8 @@ previous implementation
       mapping.getFunctionSql( new String[]{tableName, fieldName, fieldStructure}, sqlBuf );
       String sql = sqlBuf.toString();
 
-      log.warn( sql );
+      if(log.isDebugEnabled())
+         log.debug("Executing: " + sql);
 
       // suspend the current transaction
       TransactionManager tm = manager.getContainer().getTransactionManager();
