@@ -68,7 +68,7 @@ import java.util.List;
  * todo refactor optimistic locking
  *
  * @author <a href="mailto:alex@jboss.org">Alexey Loubyansky</a>
- * @version <tt>$Revision: 61754 $</tt>
+ * @version <tt>$Revision: 75267 $</tt>
  */
 public class EntityTable
    implements Table
@@ -439,7 +439,7 @@ public class EntityTable
          System.arraycopy(tmp, 0, fkConstraints, 0, tmp.length);
       }
       final int fkindex = fkConstraints.length - 1;
-      final ForeignKeyConstraint fkc = new ForeignKeyConstraint(fkindex, fkFields);
+      final ForeignKeyConstraint fkc = new ForeignKeyConstraint(fkindex, fkFields, referenced.tableId == tableId);
       fkConstraints[fkindex] = fkc;
       return fkc;
    }
@@ -1340,38 +1340,35 @@ public class EntityTable
             for(int i = 0; i < rowsWithNullFks.size(); ++i)
             {
                final Row row = (Row) rowsWithNullFks.get(i);
-               if(row.state != DELETED)
+               final ForeignKeyConstraint[] cons = row.fkUpdates;
+               for (int c = 0; c < fkConstraints.length; ++c)
                {
-                  final ForeignKeyConstraint[] cons = row.fkUpdates;
-                  for(int c = 0; c < fkConstraints.length; ++c)
+                  if (cons[c] == null || row.state == DELETED && !cons[c].selfReference)
+                     continue;
+
+                  PreparedStatement s = ps[c];
+                  if (s == null)
                   {
-                     if(cons[c] != null)
+                     if (log.isDebugEnabled())
                      {
-                        PreparedStatement s = ps[c];
-                        if(s == null)
-                        {
-                           if(log.isDebugEnabled())
-                           {
-                              log.debug("nullifying fk: " + cons[c].nullFkSql);
-                           }
-                           s = con.prepareStatement(cons[c].nullFkSql);
-                           ps[c] = s;
-                        }
-
-                        int paramInd = 1;
-                        for(int fInd = 0; fInd < pkFields.length; ++fInd)
-                        {
-                           JDBCCMPFieldBridge2 pkField = pkFields[fInd];
-                           Object fieldValue = row.fields[pkField.getRowIndex()];
-                           paramInd = pkField.setArgumentParameters(s, paramInd, fieldValue);
-                        }
-
-                        final int affected = s.executeUpdate();
-                        if(affected != 1)
-                        {
-                           throw new EJBException("Affected " + affected + " rows while expected just one");
-                        }
+                        log.debug("nullifying fk: " + cons[c].nullFkSql);
                      }
+                     s = con.prepareStatement(cons[c].nullFkSql);
+                     ps[c] = s;
+                  }
+
+                  int paramInd = 1;
+                  for (int fInd = 0; fInd < pkFields.length; ++fInd)
+                  {
+                     JDBCCMPFieldBridge2 pkField = pkFields[fInd];
+                     Object fieldValue = row.fields[pkField.getRowIndex()];
+                     paramInd = pkField.setArgumentParameters(s, paramInd, fieldValue);
+                  }
+
+                  final int affected = s.executeUpdate();
+                  if (affected != 1)
+                  {
+                     throw new EJBException("Affected " + affected + " rows while expected just one");
                   }
                }
             }
@@ -1531,6 +1528,7 @@ public class EntityTable
          if(state == CREATED || state == DIRTY)
          {
             updateState(CLEAN);
+            fkUpdates = null;
          }
          else if(state == DELETED)
          {
@@ -1539,6 +1537,7 @@ public class EntityTable
          else if(state == DIRTY_RELATIONS)
          {
             updateState(CLEAN);
+            fkUpdates = null;
          }
 
          scheduleCacheUpdate();
@@ -1859,10 +1858,12 @@ public class EntityTable
    {
       public final int index;
       private final String nullFkSql;
+      private final boolean selfReference;
 
-      public ForeignKeyConstraint(int index, JDBCCMPFieldBridge2[] fkFields)
+      public ForeignKeyConstraint(int index, JDBCCMPFieldBridge2[] fkFields, boolean selfReference)
       {
          this.index = index;
+         this.selfReference = selfReference;
 
          StringBuffer buf = new StringBuffer();
          buf.append("update ").append(tableName).append(" set ")
