@@ -76,7 +76,7 @@ import org.w3c.dom.Element;
  * @author <a href="mailto:marc.fleury@telkel.com">Marc Fleury</a>
  * @author <a href="mailto:jason@planet57.com">Jason Dillon</a>
  * @author <a href="mailto:adrian@jboss.com">Adrian Brock</a>
- * @version <tt>$Revision: 57506 $</tt>
+ * @version <tt>$Revision: 66841 $</tt>
  */
 public class JMSContainerInvoker extends ServiceMBeanSupport
    implements EJBProxyFactory, JMSContainerInvokerMBean
@@ -145,6 +145,15 @@ public class JMSContainerInvoker extends ServiceMBeanSupport
 
    /** If Dead letter queue should be used or not. */
    protected boolean useDLQ = false;
+   
+   /** The forceClearOnShutdown */
+   protected boolean forceClearOnShutdown = false;
+   
+   /** The clearPoolInterval */
+   protected long forceClearInterval = 30000;
+   
+   /** The forceClearAttempts */
+   protected int forceClearAttempts = 5;
 
    /**
     * JNDI name of the provider adapter.
@@ -258,6 +267,48 @@ public class JMSContainerInvoker extends ServiceMBeanSupport
    {
       this.maxMessagesNr = maxMessages;
    }
+   
+   public int getNumActiveSessions()
+   {
+      if (pool instanceof org.jboss.jms.asf.StdServerSessionPool)
+      {
+         org.jboss.jms.asf.StdServerSessionPool stdPool = (org.jboss.jms.asf.StdServerSessionPool)pool;
+         return stdPool.getNumActiveSessions();
+      }
+      
+      return -1;
+   }
+   
+   public void setForceClearOnShutdown(boolean forceClearOnShutdown)
+   {
+      this.forceClearOnShutdown = forceClearOnShutdown;
+   }   
+   
+   public boolean getForceClearOnShutdown()
+   {
+      return this.forceClearOnShutdown;
+   }
+   
+   public long getForceClearOnShutdownInterval()
+   {
+      return this.forceClearInterval;
+   }
+   
+   public void setForceClearOnShutdownInterval(long forceClearOnShutdownInterval)
+   {
+      this.forceClearInterval = forceClearOnShutdownInterval;
+   }
+   
+   public int getForceClearAttempts()
+   {
+      return forceClearAttempts;
+   }
+   
+   public void setForceClearAttempts(int forceClearAttempts)
+   {
+      this.forceClearAttempts = forceClearAttempts;
+      
+   }
 
    public MessageDrivenMetaData getMetaData()
    {
@@ -389,6 +440,60 @@ public class JMSContainerInvoker extends ServiceMBeanSupport
       catch (Exception ignore)
       {
       }
+      
+      try
+      {
+         String maxSize = MetaData.getElementContent
+            (MetaData.getUniqueChild(element, "MaximumSize"));
+         maxPoolSize = Integer.parseInt(maxSize);
+      }
+      catch (Exception ignore)
+      {
+      }
+
+      try
+      {
+         String forceClear = MetaData.getElementContent(MetaData.getUniqueChild(element, "ForceClearOnShutdown"));
+         if(forceClear != null)
+         {
+            forceClearOnShutdown = ("false".equalsIgnoreCase(forceClear)) ? false : true;
+            
+         }
+         
+      }catch(Exception ignore)
+      {
+         
+      }
+
+      try
+      {
+         String clearInterval = MetaData.getElementContent(MetaData.getUniqueChild(element, "ForceClearInterval"));
+         
+         if(clearInterval != null)
+         {
+            forceClearInterval = new Long(clearInterval).longValue();
+            
+         }
+               
+      }catch(Exception ignore)
+      {
+         
+      }
+
+      try
+      {
+         String attempts = MetaData.getElementContent(MetaData.getUniqueChild(element, "ForceClearAttempts"));
+         
+         if(attempts != null)
+         {
+            forceClearAttempts = new Integer(attempts).intValue();
+            
+         }
+      
+      }catch(Exception ignore)
+      {
+         
+      }
 
       try
       {
@@ -1168,7 +1273,7 @@ public class JMSContainerInvoker extends ServiceMBeanSupport
             context.lookup(serverSessionPoolFactoryJNDI);
          
          // the create the pool
-         pool = factory.getServerSessionPool(destination, connection, minSession, maxSession, keepAlive, isTransacted, ack, !isContainerManagedTx || isNotSupportedTx, listener);
+         pool = factory.getServerSessionPool(destination, connection, minSession, maxSession, keepAlive, forceClearOnShutdown, forceClearInterval, forceClearAttempts, isTransacted, ack, !isContainerManagedTx || isNotSupportedTx, listener);
       }
       finally
       {
