@@ -21,6 +21,8 @@
  */
 package org.jboss.metadata;
 
+// $Id: WebMetaData.java 75558 2008-07-09 16:50:17Z bstansberry@jboss.com $
+
 import java.net.URLClassLoader;
 import java.util.ArrayList;
 import java.util.Collection;
@@ -52,7 +54,7 @@ import org.w3c.dom.Element;
  * @see org.jboss.web.AbstractWebContainer
  *
  * @author Scott.Stark@jboss.org
- * @version $Revision: 65717 $
+ * @version $Revision: 75558 $
  */
 public class WebMetaData extends MetaData
 {
@@ -60,6 +62,10 @@ public class WebMetaData extends MetaData
 
    /** The web.xml servlet <String, String> */
    private HashMap servletClassNames = new HashMap();
+   /** The web.xml servlet-param <String,String> */
+   private HashMap servletParams = new HashMap();
+   /** The web.xml context-param <String,String> */
+   private HashMap contextParams = new HashMap();
    /** The web.xml servlet-mapping <String, String> */
    private HashMap servletMappings = new HashMap();
    /** The web.xml resource-refs <String, String> */
@@ -153,7 +159,11 @@ public class WebMetaData extends MetaData
     * for pojo replication or not.
     */
    private boolean replicationFieldBatchMode = true;
-
+   
+   /** By default replicate clustered session metadata at least every 60 seconds */
+   public static final int DEFAULT_MAX_UNREPLICATED_INTERVAL = 60;   
+   private Integer maxUnreplicatedInterval = null;
+   
    /** Should the context use session cookies or use default */
    private int sessionCookies = SESSION_COOKIES_DEFAULT;
 
@@ -177,10 +187,11 @@ public class WebMetaData extends MetaData
    {
       return environmentEntries.iterator();
    }
+
    /**
     * 
     * @param environmentEntries - Collection<EnvEntryMetaData>
-    */ 
+    */
    public void setEnvironmentEntries(Collection environmentEntries)
    {
       this.environmentEntries.clear();
@@ -194,10 +205,11 @@ public class WebMetaData extends MetaData
    {
       return ejbReferences.values().iterator();
    }
+
    /**
     * 
     * @param ejbReferences - Map<String, EjbRefMetaData>
-    */ 
+    */
    public void setEjbReferences(Map ejbReferences)
    {
       this.ejbReferences.clear();
@@ -211,10 +223,11 @@ public class WebMetaData extends MetaData
    {
       return ejbLocalReferences.values().iterator();
    }
+
    /**
     * 
     * @param ejbReferences - Map<String, EjbRefMetaData>
-    */ 
+    */
    public void setEjbLocalReferences(Map ejbReferences)
    {
       this.ejbLocalReferences.clear();
@@ -228,10 +241,11 @@ public class WebMetaData extends MetaData
    {
       return resourceReferences.values().iterator();
    }
+
    /**
     * 
     * @param resourceReferences - Map<String, ResourceRefMetaData>
-    */ 
+    */
    public void setResourceReferences(Map resourceReferences)
    {
       this.resourceReferences.clear();
@@ -245,10 +259,11 @@ public class WebMetaData extends MetaData
    {
       return resourceEnvReferences.values().iterator();
    }
+
    /**
     * 
     * @param resourceReferences - Map<String, ResourceEnvRefMetaData>
-    */ 
+    */
    public void setResourceEnvReferences(Map resourceReferences)
    {
       this.resourceEnvReferences.clear();
@@ -264,10 +279,11 @@ public class WebMetaData extends MetaData
    {
       return messageDestinationReferences.values().iterator();
    }
+
    /**
     * 
     * @param messageDestinationReferences - Map<String, MessageDestinationRefMetaData>
-    */ 
+    */
    public void setMessageDestinationReferences(Map messageDestinationReferences)
    {
       this.messageDestinationReferences.clear();
@@ -284,21 +300,22 @@ public class WebMetaData extends MetaData
    {
       return (MessageDestinationMetaData)messageDestinations.get(name);
    }
+
    /**
     * 
     * @param messageDestinations - Map<String, MessageDestinationMetaData>
-    */ 
+    */
    public void setMessageDestination(Map messageDestinations)
    {
       this.messageDestinations.clear();
       this.messageDestinations.putAll(messageDestinations);
    }
 
-   public Map<String,ServiceRefMetaData> getServiceReferences()
+   public Map<String, ServiceRefMetaData> getServiceReferences()
    {
       return serviceReferences;
    }
-   
+
    public void setServiceReferences(Map<String, ServiceRefMetaData> serviceReferences)
    {
       this.serviceReferences.clear();
@@ -343,10 +360,11 @@ public class WebMetaData extends MetaData
    {
       return (String)wsdlPublishLocationMap.get(name);
    }
+
    /**
     * 
     * @param wsdlPublishLocationMap - Map<String, String>
-    */ 
+    */
    public void setWsdlPublishLocationMap(Map wsdlPublishLocationMap)
    {
       this.wsdlPublishLocationMap.clear();
@@ -390,15 +408,15 @@ public class WebMetaData extends MetaData
    {
       this.securityDomain = securityDomain;
    }
-   
+
    /**
     * JBAS-1824: Specify whether a WebResourcePermission(url,null)
     * needs to be generated by the container
     * @return
     */
-   public boolean isJaccRoleNameStar() 
+   public boolean isJaccRoleNameStar()
    {
-	  return jaccRoleNameStar;
+      return jaccRoleNameStar;
    }
 
    /**
@@ -406,12 +424,12 @@ public class WebMetaData extends MetaData
     * needs to be generated by the container
     * @return
     */
-   public void setJaccRoleNameStar(boolean jaccRoleNameStar) 
+   public void setJaccRoleNameStar(boolean jaccRoleNameStar)
    {
-	  this.jaccRoleNameStar = jaccRoleNameStar;
+      this.jaccRoleNameStar = jaccRoleNameStar;
    }
 
-/** The flag indicating whether the associated security domain cache
+   /** The flag indicating whether the associated security domain cache
     * should be flushed when the session is invalidated.
     * @return true if the flush should occur, false otherwise.
     */
@@ -435,10 +453,11 @@ public class WebMetaData extends MetaData
    {
       return securityContraints.iterator();
    }
+
    /**
     * 
     * @param securityContraints - Collection<WebSecurityMetaData>
-    */ 
+    */
    public void setSecurityConstraints(Collection securityContraints)
    {
       this.securityContraints.clear();
@@ -452,20 +471,22 @@ public class WebMetaData extends MetaData
    {
       return this.securityRoleReferences;
    }
+
    /**
     * 
     * @param servletName
     * @return List<SecurityRoleRefMetaData> for the given servlet name
-    */ 
+    */
    public List getSecurityRoleRefs(String servletName)
    {
-      List roles = (List) this.securityRoleReferences.get(servletName);
+      List roles = (List)this.securityRoleReferences.get(servletName);
       return roles;
    }
+
    /**
     * 
     * @param securityRoleReferences - <String servlet-name, ArrayList<SecurityRoleRefMetaData>>
-    */ 
+    */
    public void setSecurityRoleReferences(Map securityRoleReferences)
    {
       this.securityRoleReferences.clear();
@@ -488,10 +509,11 @@ public class WebMetaData extends MetaData
    {
       return new HashMap(securityRoles);
    }
+
    /**
     * 
     * @param securityRoles - Map<String, SecurityRoleMetaData>
-    */ 
+    */
    public void setSecurityRoles(Map securityRoles)
    {
       this.securityRoles.clear();
@@ -539,14 +561,16 @@ public class WebMetaData extends MetaData
       }
       return runAs;
    }
+
    /**
     * 
     * @return servlet/run-as <String servlet-name, RunAsIdentity>
-    */ 
+    */
    public Map getRunAsIdentity()
    {
       return runAsIdentity;
    }
+
    /** The jboss-web.xml servlet/run-as <String servlet-name, RunAsIdentity>
     */
    public void setRunAsIdentity(Map runAsIdentity)
@@ -555,6 +579,26 @@ public class WebMetaData extends MetaData
       this.runAsIdentity.putAll(runAsIdentity);
    }
 
+   /**
+    * Get the context-param values from the web.xml descriptor
+    * @return Map<String,String> of param-name/param-value from the web-xml
+    */
+   public HashMap getContextParams()
+   {
+      return contextParams;
+   }
+
+   /**
+    * Set the context-params
+    * 
+    * @param contextParams - Map<String,String>
+    */
+   public void setContextParams(Map contextParams)
+   {
+      this.contextParams.clear();
+      this.contextParams.putAll(contextParams);
+   }
+
    /**
     * Get the servlet-name values from the web.xml descriptor
     * @return Set<String> of the servlet-names from the servlet-mappings
@@ -563,11 +607,12 @@ public class WebMetaData extends MetaData
    {
       return servletMappings;
    }
+
    /** The web.xml servlet-mapping  */
    /**
     * servlet-mapping/serlvet-name to url-pattern mapping
     * @param servletMappings - Map<String, String>
-    */ 
+    */
    public void setServletMappings(Map servletMappings)
    {
       this.servletMappings.clear();
@@ -583,6 +628,20 @@ public class WebMetaData extends MetaData
       return new HashSet(servletMappings.keySet());
    }
 
+   /**
+    * Get the init parameter map for a servlet or an empty map if there are none.
+    */
+   public Map getServletParams(String servletName)
+   {
+      Map params = (Map)servletParams.get(servletName);
+      if (params == null)
+      {
+         params = new HashMap();
+         servletParams.put(servletName, params);
+      }
+      return params;
+   }
+
    /**
     * Get the servlet-name/servlet-class mapping from the web.xml descriptor
     * @return Map<String, String> of the servlet-name/servlet-class mapping from web.xml
@@ -624,10 +683,11 @@ public class WebMetaData extends MetaData
    {
       return virtualHosts.iterator();
    }
+
    /**
     * 
     * @param virtualHosts - Collection<String>
-    */ 
+    */
    public void setVirtualHosts(Collection virtualHosts)
    {
       this.virtualHosts.clear();
@@ -642,10 +702,11 @@ public class WebMetaData extends MetaData
    {
       return distributable;
    }
+
    /**
     * Mark the web-app as distributable
     * @param distributable - true for distributable
-    */ 
+    */
    public void setDistributable(boolean distributable)
    {
       this.distributable = distributable;
@@ -658,9 +719,10 @@ public class WebMetaData extends MetaData
    {
       return depends;
    }
+
    /**
     @param depends - Collection<ObjectName> of the web app dependencies
-    */ 
+    */
    public void setDepends(Collection depends)
    {
       this.depends.clear();
@@ -685,6 +747,7 @@ public class WebMetaData extends MetaData
    {
       return useJBossWebLoader;
    }
+
    public void setUseJBossWebLoader(boolean flag)
    {
       useJBossWebLoader = flag;
@@ -725,6 +788,7 @@ public class WebMetaData extends MetaData
    {
       return this.sessionCookies;
    }
+
    public void setSessionCookies(int sessionCookies)
    {
       this.sessionCookies = sessionCookies;
@@ -734,6 +798,7 @@ public class WebMetaData extends MetaData
    {
       return this.invalidateSessionPolicy;
    }
+
    public void setInvalidateSessionPolicy(int invalidateSessionPolicy)
    {
       this.invalidateSessionPolicy = invalidateSessionPolicy;
@@ -748,6 +813,7 @@ public class WebMetaData extends MetaData
    {
       return replicationGranularity;
    }
+
    public void setReplicationGranularity(int replicationGranularity)
    {
       this.replicationGranularity = replicationGranularity;
@@ -757,6 +823,21 @@ public class WebMetaData extends MetaData
    {
       return replicationFieldBatchMode;
    }
+   
+   public void setReplicationFieldBatchMode(boolean batchMode)
+   {
+      this.replicationFieldBatchMode = batchMode;
+   }
+
+   public Integer getMaxUnreplicatedInterval()
+   {
+      return maxUnreplicatedInterval;
+   }
+
+   public void setMaxUnreplicatedInterval(Integer maxUnreplicatedInterval)
+   {
+      this.maxUnreplicatedInterval = maxUnreplicatedInterval;
+   }
 
    public void importXml(Element element) throws DeploymentException
    {
@@ -786,7 +867,22 @@ public class WebMetaData extends MetaData
          {
             servletClassNames.put(servletName, servletClass);
          }
-         
+
+         Iterator initParams = getChildrenByTagName(servlet, "init-param");
+         while (initParams.hasNext())
+         {
+            Element param = (Element)initParams.next();
+            String paramName = getElementContent(getUniqueChild(param, "param-name"));
+            String paramValue = getElementContent(getUniqueChild(param, "param-value"));
+
+            if (null == servletParams.get(servletName))
+            {
+               servletParams.put(servletName, new HashMap());
+            }
+
+            ((Map)servletParams.get(servletName)).put(paramName, paramValue);
+         }
+
          Iterator roleRefs = getChildrenByTagName(servlet, "security-role-ref");
          ArrayList roleNames = new ArrayList();
          while (roleRefs.hasNext())
@@ -807,6 +903,16 @@ public class WebMetaData extends MetaData
          }
       }
 
+      // Parse the web-app/context-param elements
+      iterator = getChildrenByTagName(webApp, "context-param");
+      while (iterator.hasNext())
+      {
+         Element contextParam = (Element)iterator.next();
+         String paramName = getElementContent(getUniqueChild(contextParam, "param-name"));
+         String paramValue = getElementContent(getUniqueChild(contextParam, "param-value"));
+         contextParams.put(paramName, paramValue);
+      }
+
       // Parse the web-app/servlet-mapping elements
       iterator = getChildrenByTagName(webApp, "servlet-mapping");
       while (iterator.hasNext())
@@ -860,8 +966,7 @@ public class WebMetaData extends MetaData
          }
          catch (Throwable t)
          {
-            throw new DeploymentException("Error in web.xml " +
-                    "for message destination: " + t.getMessage());
+            throw new DeploymentException("Error in web.xml " + "for message destination: " + t.getMessage());
          }
       }
 
@@ -1001,17 +1106,16 @@ public class WebMetaData extends MetaData
          Boolean flag = Boolean.valueOf(securityDomainElement.getAttribute("flushOnSessionInvalidation"));
          flushOnSessionInvalidation = flag.booleanValue();
       }
-      
+
       //Parse the jboss-web/jacc-star-role-allow element
       Element jaccStarRoleElement = getOptionalChild(jbossWeb, "jacc-star-role-allow");
       if (jaccStarRoleElement != null)
       {
-         jaccRoleNameStar = "true".equalsIgnoreCase(getElementContent(jaccStarRoleElement)); 
+         jaccRoleNameStar = "true".equalsIgnoreCase(getElementContent(jaccStarRoleElement));
       }
 
       // Parse the jboss-web/virtual-host elements
-      for (Iterator virtualHostElements = getChildrenByTagName(jbossWeb, "virtual-host");
-           virtualHostElements.hasNext();)
+      for (Iterator virtualHostElements = getChildrenByTagName(jbossWeb, "virtual-host"); virtualHostElements.hasNext();)
       {
          Element virtualHostElement = (Element)virtualHostElements.next();
          String virtualHostName = getElementContent(virtualHostElement);
@@ -1027,8 +1131,7 @@ public class WebMetaData extends MetaData
          ResourceRefMetaData refMetaData = (ResourceRefMetaData)resourceReferences.get(resRefName);
          if (refMetaData == null)
          {
-            throw new DeploymentException("resource-ref " + resRefName
-                    + " found in jboss-web.xml but not in web.xml");
+            throw new DeploymentException("resource-ref " + resRefName + " found in jboss-web.xml but not in web.xml");
          }
          refMetaData.importJbossXml(resourceRef);
       }
@@ -1042,8 +1145,7 @@ public class WebMetaData extends MetaData
          ResourceEnvRefMetaData refMetaData = (ResourceEnvRefMetaData)resourceEnvReferences.get(resRefName);
          if (refMetaData == null)
          {
-            throw new DeploymentException("resource-env-ref " + resRefName
-                    + " found in jboss-web.xml but not in web.xml");
+            throw new DeploymentException("resource-env-ref " + resRefName + " found in jboss-web.xml but not in web.xml");
          }
          refMetaData.importJbossXml(resourceRef);
       }
@@ -1075,8 +1177,7 @@ public class WebMetaData extends MetaData
          }
          catch (Throwable t)
          {
-            throw new DeploymentException("Error in web.xml " +
-                    "for message destination: " + t.getMessage());
+            throw new DeploymentException("Error in web.xml " + "for message destination: " + t.getMessage());
          }
       }
 
@@ -1088,8 +1189,7 @@ public class WebMetaData extends MetaData
          String roleName = getElementContent(getUniqueChild(securityRole, "role-name"));
          SecurityRoleMetaData securityRoleMetaData = (SecurityRoleMetaData)securityRoles.get(roleName);
          if (securityRoleMetaData == null)
-            throw new DeploymentException("Security role '" + roleName + "' defined in jboss-web.xml" +
-                    " is not defined in web.xml");
+            throw new DeploymentException("Security role '" + roleName + "' defined in jboss-web.xml" + " is not defined in web.xml");
 
          Iterator itPrincipalNames = getChildrenByTagName(securityRole, "principal-name");
          while (itPrincipalNames.hasNext())
@@ -1108,8 +1208,7 @@ public class WebMetaData extends MetaData
          EjbRefMetaData ejbRefMetaData = (EjbRefMetaData)ejbReferences.get(ejbRefName);
          if (ejbRefMetaData == null)
          {
-            throw new DeploymentException("ejb-ref " + ejbRefName
-                    + " found in jboss-web.xml but not in web.xml");
+            throw new DeploymentException("ejb-ref " + ejbRefName + " found in jboss-web.xml but not in web.xml");
          }
          ejbRefMetaData.importJbossXml(ejbRef);
       }
@@ -1123,8 +1222,7 @@ public class WebMetaData extends MetaData
          EjbLocalRefMetaData ejbLocalRefMetaData = (EjbLocalRefMetaData)ejbLocalReferences.get(ejbLocalRefName);
          if (ejbLocalRefMetaData == null)
          {
-            throw new DeploymentException("ejb-local-ref " + ejbLocalRefName
-                    + " found in jboss-web.xml but not in web.xml");
+            throw new DeploymentException("ejb-local-ref " + ejbLocalRefName + " found in jboss-web.xml but not in web.xml");
          }
          ejbLocalRefMetaData.importJbossXml(ejbLocalRef);
       }
@@ -1138,8 +1236,7 @@ public class WebMetaData extends MetaData
          ServiceRefMetaData refMetaData = (ServiceRefMetaData)serviceReferences.get(serviceRefName);
          if (refMetaData == null)
          {
-            throw new DeploymentException("service-ref " + serviceRefName
-                    + " found in jboss-web.xml but not in web.xml");
+            throw new DeploymentException("service-ref " + serviceRefName + " found in jboss-web.xml but not in web.xml");
          }
          refMetaData.importJBossXml(serviceRef);
       }
@@ -1157,8 +1254,7 @@ public class WebMetaData extends MetaData
       }
 
       // Parse the jboss-web/depends elements
-      for (Iterator dependsElements = getChildrenByTagName(jbossWeb, "depends");
-           dependsElements.hasNext();)
+      for (Iterator dependsElements = getChildrenByTagName(jbossWeb, "depends"); dependsElements.hasNext();)
       {
          Element dependsElement = (Element)dependsElements.next();
          String dependsName = getElementContent(dependsElement);
@@ -1202,7 +1298,7 @@ public class WebMetaData extends MetaData
                this.invalidateSessionPolicy = SESSION_INVALIDATE_SET;
             else
                throw new DeploymentException("replication-trigger value set to a non-valid value: '" + repMethod
-                       + "' (should be ['SET_AND_GET', 'SET_AND_NON_PRIMITIVE_GET', 'SET']) in jboss-web.xml");
+                     + "' (should be ['SET_AND_GET', 'SET_AND_NON_PRIMITIVE_GET', 'SET']) in jboss-web.xml");
          }
 
          // ... then manage "replication-type".
@@ -1216,11 +1312,10 @@ public class WebMetaData extends MetaData
             else if ("ASYNC".equalsIgnoreCase(repType))
                this.replicationType = REPLICATION_TYPE_ASYNC;
             else
-               throw new DeploymentException("replication-type value set to a non-valid value: '" + repType
-                       + "' (should be ['SYNC', 'ASYNC']) in jboss-web.xml");
+               throw new DeploymentException("replication-type value set to a non-valid value: '" + repType + "' (should be ['SYNC', 'ASYNC']) in jboss-web.xml");
          }
 
-         // ... then manage "replication-type".
+         // ... then manage "replication-granularity".
          //
          Element replicationGranularityElement = MetaData.getOptionalChild(sessionReplicationRootElement, "replication-granularity");
          if (replicationGranularityElement != null)
@@ -1234,7 +1329,7 @@ public class WebMetaData extends MetaData
                this.replicationGranularity = REPLICATION_GRANULARITY_FIELD;
             else
                throw new DeploymentException("replication-granularity value set to a non-valid value: '" + repType
-                  + "' (should be ['SESSION', 'ATTRIBUTE', or 'FIELD'']) in jboss-web.xml");
+                     + "' (should be ['SESSION', 'ATTRIBUTE', or 'FIELD'']) in jboss-web.xml");
          }
 
          Element batchModeElement = MetaData.getOptionalChild(sessionReplicationRootElement, "replication-field-batch-mode");
@@ -1243,6 +1338,21 @@ public class WebMetaData extends MetaData
             Boolean flag = Boolean.valueOf(MetaData.getElementContent(batchModeElement));
             replicationFieldBatchMode = flag.booleanValue();
          }
+
+         Element maxUnreplicatedIntervalElement = MetaData.getOptionalChild(sessionReplicationRootElement, "max-unreplicated-interval");
+         if (maxUnreplicatedIntervalElement != null)
+         {
+            String maxUnrep =  MetaData.getElementContent(maxUnreplicatedIntervalElement);
+            try
+            {
+               maxUnreplicatedInterval = Integer.valueOf(maxUnrep);
+            }
+            catch (NumberFormatException e)
+            {
+               throw new DeploymentException("max-unreplicated-interval value set to a non-integer value: '" + maxUnrep
+                     + " in jboss-web.xml");
+            }
+         }
       }
 
       // Check for a war level class loading config
@@ -1283,8 +1393,7 @@ public class WebMetaData extends MetaData
          {
             if (webXmlRunAs == null)
             {
-               throw new DeploymentException("run-as-principal: " + principalName
-                       + " found in jboss-web.xml but there was no run-as in web.xml");
+               throw new DeploymentException("run-as-principal: " + principalName + " found in jboss-web.xml but there was no run-as in web.xml");
             }
             // See if there are any additional roles for this principal
             Set extraRoles = getSecurityRoleNamesByPrincipal(principalName);
@@ -1299,4 +1408,5 @@ public class WebMetaData extends MetaData
       }
 
    }
+
 }
