@@ -23,7 +23,6 @@ package org.jboss.verifier.strategy;
 
 // standard imports
 
-import org.gjt.lindfors.pattern.StrategyContext;
 import org.jboss.logging.Logger;
 import org.jboss.metadata.BeanMetaData;
 import org.jboss.metadata.EntityMetaData;
@@ -63,7 +62,7 @@ import javax.jms.Message;
  * @author <a href="mailto:jwalters@computer.org">Jay Walters</a>
  * @author Scott.Stark@jboss.org
  *
- * @version $Revision: 57209 $
+ * @version $Revision: 68139 $
  * @since   JDK 1.3
  */
 public abstract class AbstractVerifier
@@ -1165,7 +1164,7 @@ public abstract class AbstractVerifier
     *
     * @return  the client object using this algorithm implementation
     */
-   public StrategyContext getContext()
+   public VerificationContext getContext()
    {
       return context;
    }
