@@ -54,7 +54,7 @@ import org.jboss.system.server.ServerLoader;
  * @author <a href="mailto:jason@planet57.com">Jason Dillon</a>
  * @author <a href="mailto:adrian.brock@happeningtimes.com">Adrian Brock</a>
  * @author <a href="mailto:scott.stark@jboss.org">Scott Stark</a>
- * @version $Revision: 61145 $
+ * @version $Revision: 66869 $
  */
 public class Main
 {
@@ -482,12 +482,12 @@ public class Main
          }
       }
 
-      // JBAS-4119, set the java.rmi.server.hostname if not set
+      // Fix up other bind addresses
+      String bindAddress = System.getProperty(ServerConfig.SERVER_BIND_ADDRESS);
       if (System.getProperty("java.rmi.server.hostname") == null)
-      {
-         System.setProperty("java.rmi.server.hostname", "127.0.0.1");
-      }
-   }
+         System.setProperty("java.rmi.server.hostname", bindAddress);
+      if (System.getProperty("jgroups.bind_addr") == null)
+         System.setProperty("jgroups.bind_addr", bindAddress);   }
 
    /**
     * This is where the magic begins.
