@@ -74,7 +74,7 @@ import EDU.oswego.cs.dl.util.concurrent.ConcurrentReaderHashMap;
  * @author <a href="mailto:sacha.labourey@cogito-info.ch">Sacha Labourey</a>
  * @author <a href="mailto:scott.stark@jboss.org">Scott Stark</a>
  * @author <a href="mailto:dimitris@jboss.org">Dimitris Andreadis<a/>
- * @version $Revision: 57205 $
+ * @version $Revision: 69570 $
  */
 public class SARDeployer extends SubDeployerSupport
    implements SARDeployerMBean
@@ -184,11 +184,13 @@ public class SARDeployer extends SubDeployerSupport
          NodeList loaders = di.document.getElementsByTagName("loader-repository");
          if( loaders.getLength() > 0 )
          {
-            Element loader = (Element) loaders.item(0);
-            LoaderRepositoryConfig config = LoaderRepositoryFactory.parseRepositoryConfig(loader);
-            di.setRepositoryInfo(config);
+        	 if(loaders.getLength() > 1)
+        		 throw new DeploymentException("SAR Deployment cannot have more than one loader-repository entry.");
+        	 Element loader = (Element) loaders.item(0);
+             LoaderRepositoryConfig config = LoaderRepositoryFactory.parseRepositoryConfig(loader);
+             di.setRepositoryInfo(config);              
          }
-
+         
          // In case there is a dependent classpath defined parse it
          parseXMLClasspath(di);
 
