@@ -23,6 +23,7 @@ package org.jboss.test.cmp2.commerce;
 
 import java.util.Collection;
 import java.util.Set;
+import java.util.Properties;
 import java.lang.reflect.Method;
 import javax.management.MBeanServer;
 import javax.management.MBeanServerFactory;
@@ -38,13 +39,14 @@ import org.jboss.mx.server.ServerConstants;
 import org.jboss.mx.server.registry.MBeanEntry;
 import org.jboss.mx.server.registry.MBeanRegistry;
 import org.jboss.test.JBossTestCase;
+import org.jboss.test.util.ejb.EJBTestCase;
 import org.jboss.mx.util.MBeanProxyExt;
 import org.jboss.util.UnreachableStatementException;
+import org.jboss.util.platform.Java;
 
-public class QueryTest extends org.jboss.test.util.ejb.EJBTestCase
+public class QueryTest extends EJBTestCase
 {
    private JDBCEJBQLCompiler compiler;
-   private static final String javaVersion = System.getProperty("java.specification.version");
    private static final Class[] NO_PARAMS = new Class[]{};
 
    public static Test suite() throws Exception
@@ -57,7 +59,7 @@ public class QueryTest extends org.jboss.test.util.ejb.EJBTestCase
       super(name);
    }
 
-   public void setUpEJB() throws Exception
+   public void setUpEJB(Properties props) throws Exception
    {
       MBeanServer server = (MBeanServer) MBeanServerFactory.findMBeanServer(null).get(0);
       ObjectName name = new ObjectName("jboss.j2ee:jndiName=commerce/Order,service=EJB");
@@ -164,17 +166,11 @@ public class QueryTest extends org.jboss.test.util.ejb.EJBTestCase
                    compileJBossQL("SELECT OBJECT(u) FROM user u WHERE LCASE(u.userName) = ?1",
                                   Collection.class, new Class[]{String.class}));
 
-      String expected = "1.4".equals(javaVersion) ?
-            "SELECT t0_o1.ORDER_NUMBER FROM ORDER_DATA t0_o1, ORDER_DATA t3_o2, CUSTOMEREJB t2_o2_customer, CUSTOMEREJB t1_o1_customer WHERE (( NOT (t1_o1_customer.id=t2_o2_customer.id)) AND (t0_o1.CC_TYPE=t3_o2.CC_TYPE AND t0_o1.CC_FIRST_NAME=t3_o2.CC_FIRST_NAME AND t0_o1.CC_MI=t3_o2.CC_MI AND t0_o1.CC_LAST_NAME=t3_o2.CC_LAST_NAME AND t0_o1.CC_BILLING_ZIP=t3_o2.CC_BILLING_ZIP AND t0_o1.CC_CARD_NUMBER=t3_o2.CC_CARD_NUMBER) AND t3_o2.customer=t2_o2_customer.id AND t0_o1.customer=t1_o1_customer.id)" :
-            "SELECT t0_o1.ORDER_NUMBER FROM ORDER_DATA t0_o1, ORDER_DATA t3_o2, CUSTOMEREJB t1_o1_customer, CUSTOMEREJB t2_o2_customer WHERE (( NOT (t1_o1_customer.id=t2_o2_customer.id)) AND (t0_o1.CC_TYPE=t3_o2.CC_TYPE AND t0_o1.CC_FIRST_NAME=t3_o2.CC_FIRST_NAME AND t0_o1.CC_MI=t3_o2.CC_MI AND t0_o1.CC_LAST_NAME=t3_o2.CC_LAST_NAME AND t0_o1.CC_BILLING_ZIP=t3_o2.CC_BILLING_ZIP AND t0_o1.CC_CARD_NUMBER=t3_o2.CC_CARD_NUMBER) AND t0_o1.customer=t1_o1_customer.id AND t3_o2.customer=t2_o2_customer.id)";
+      String expected = "SELECT t0_o1.ORDER_NUMBER FROM ORDER_DATA t0_o1, ORDER_DATA t3_o2, CUSTOMEREJB t2_o2_customer, CUSTOMEREJB t1_o1_customer WHERE (( NOT (t1_o1_customer.id=t2_o2_customer.id)) AND (t0_o1.CC_TYPE=t3_o2.CC_TYPE AND t0_o1.CC_FIRST_NAME=t3_o2.CC_FIRST_NAME AND t0_o1.CC_MI=t3_o2.CC_MI AND t0_o1.CC_LAST_NAME=t3_o2.CC_LAST_NAME AND t0_o1.CC_BILLING_ZIP=t3_o2.CC_BILLING_ZIP AND t0_o1.CC_CARD_NUMBER=t3_o2.CC_CARD_NUMBER) AND t3_o2.customer=t2_o2_customer.id AND t0_o1.customer=t1_o1_customer.id)";
       String compiled = compileJBossQL(
          "SELECT OBJECT(o1) FROM OrderX o1, OrderX o2 WHERE o1.customer <> o2.customer AND o1.creditCard = o2.creditCard",
          Collection.class, NO_PARAMS);
-      /*
       assertTrue("Expected: " + expected + " but got: " + compiled, expected.equals(compiled));
-      */
-      if( expected.equals(compiled) == false )
-         System.err.println("Expected: " + expected + " but got: " + compiled);
 
       assertEquals("SELECT t0_o.ORDER_NUMBER " +
                    "FROM ORDER_DATA t0_o " +
@@ -307,18 +303,21 @@ public class QueryTest extends org.jboss.test.util.ejb.EJBTestCase
          )
       );
 
-      String expected = "1.4".equals(javaVersion) ?
+      String expected =
          "SELECT t0_o.ORDER_NUMBER " +
          "FROM ORDER_DATA t0_o, LINEITEMEJB t4_l, PRODUCTCATEGORYEJB t1_pc, PRODUCT_PRODUCT_CATEGORY t5_l_product_productCategories_R, PRODUCT t6_l_product " +
          "WHERE (((t0_o.ORDER_NUMBER = ? AND t1_pc.name = ?))) " +
          "AND t6_l_product.id=t5_l_product_productCategories_R.PRODUCT_ID " +
          "AND t1_pc.id=t5_l_product_productCategories_R.PRODUCT_CATEGORY_ID " +
          "AND t1_pc.subId=t5_l_product_productCategories_R.PRODUCT_CATEGORY_SUBID " +
-         "AND t4_l.product=t6_l_product.id AND t0_o.ORDER_NUMBER=t4_l.ORDER_NUMBER" :
+         "AND t4_l.product=t6_l_product.id AND t0_o.ORDER_NUMBER=t4_l.ORDER_NUMBER";
+
+      //JBAS-5031
+      String expectedJDK6 =
          "SELECT t0_o.ORDER_NUMBER " +
          "FROM ORDER_DATA t0_o, LINEITEMEJB t4_l, PRODUCTCATEGORYEJB t1_pc, PRODUCT_PRODUCT_CATEGORY t5_l_product_productCategories_R, PRODUCT t6_l_product " +
          "WHERE (((t0_o.ORDER_NUMBER = ? AND t1_pc.name = ?))) " +
-         "AND t0_o.ORDER_NUMBER=t4_l.ORDER_NUMBER " +
+	 "AND t0_o.ORDER_NUMBER=t4_l.ORDER_NUMBER " +
          "AND t6_l_product.id=t5_l_product_productCategories_R.PRODUCT_ID " +
          "AND t1_pc.id=t5_l_product_productCategories_R.PRODUCT_CATEGORY_ID " +
          "AND t1_pc.subId=t5_l_product_productCategories_R.PRODUCT_CATEGORY_SUBID " +
@@ -329,11 +328,12 @@ public class QueryTest extends org.jboss.test.util.ejb.EJBTestCase
                                       "IN(l.product.productCategories) pc " +
                                       "WHERE (o.ordernumber = ?1 and pc.name=?2)",
                                       Collection.class, new Class[]{Long.class, String.class});
-      /*
-      assertEquals(expected, compiled);
-      */
-      if( expected.equals(compiled) == false )
-         System.err.println("Expected: " + expected + " but got: " + compiled);
+
+       if (Java.isVersion(Java.VERSION_1_5) == false) {
+      		assertEquals(expectedJDK6, compiled);
+       } else {
+      		assertEquals(expected, compiled);
+       }
 
       expected = "SELECT DISTINCT t0_o.ORDER_NUMBER " +
          "FROM ORDER_DATA t0_o, LINEITEMEJB t3_l " +
@@ -380,8 +380,14 @@ public class QueryTest extends org.jboss.test.util.ejb.EJBTestCase
                    compileEJBQL("SELECT OBJECT(o) FROM OrderX o WHERE o.ordernumber = ?1 OR o.lineItems IS NOT EMPTY",
                                 Set.class, new Class[]{Long.class}));
 
+
+       if (Java.isVersion(Java.VERSION_1_5) == false) {
+      assertEquals("SELECT t0_l.id FROM CUSTOMEREJB t1_c, ORDER_DATA t3_o, LINEITEMEJB t0_l WHERE ((t1_c.id = 1)) AND t3_o.ORDER_NUMBER=t0_l.ORDER_NUMBER AND t1_c.id=t3_o.customer",
+        compileEJBQL("SELECT OBJECT(l) FROM Customer c, IN(c.orders) o, IN(o.lineItems) l WHERE c.id=1"));
+       } else {
       assertEquals("SELECT t0_l.id FROM CUSTOMEREJB t1_c, ORDER_DATA t3_o, LINEITEMEJB t0_l WHERE ((t1_c.id = 1)) AND t1_c.id=t3_o.customer AND t3_o.ORDER_NUMBER=t0_l.ORDER_NUMBER",
-                   compileEJBQL("SELECT OBJECT(l) FROM Customer c, IN(c.orders) o, IN(o.lineItems) l WHERE c.id=1"));
+        compileEJBQL("SELECT OBJECT(l) FROM Customer c, IN(c.orders) o, IN(o.lineItems) l WHERE c.id=1"));
+	}
 
       // customer query was SELECT OBJECT(s) FROM Service AS s, Platform AS p WHERE p.id = ?1 AND s.server MEMBER OF p.servers
       assertEquals("SELECT t0_l.id FROM LINEITEMEJB t0_l, CUSTOMEREJB t1_c, ORDER_DATA t3_l_order WHERE (t1_c.id = 1 AND EXISTS (SELECT t2_c_orders.ORDER_NUMBER FROM ORDER_DATA t2_c_orders WHERE t1_c.id=t2_c_orders.customer AND t2_c_orders.ORDER_NUMBER=t3_l_order.ORDER_NUMBER) AND t0_l.ORDER_NUMBER=t3_l_order.ORDER_NUMBER)",
