@@ -1,8 +1,8 @@
 /*
- * JBoss, Home of Professional Open Source.
- * Copyright 2006, Red Hat Middleware LLC, and individual contributors
- * as indicated by the @author tags. See the copyright.txt file in the
- * distribution for a full listing of individual contributors.
+ * JBoss, Home of Professional Open Source
+ * Copyright 2005, JBoss Inc., and individual contributors as indicated
+ * by the @authors tag. See the copyright.txt in the distribution for a
+ * full listing of individual contributors.
  *
  * This is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as
@@ -28,23 +28,22 @@ import java.util.Arrays;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
+import java.util.Properties;
 
 import javax.naming.InitialContext;
 import javax.rmi.PortableRemoteObject;
 
 import junit.framework.Test;
 import junit.framework.TestSuite;
-import org.jboss.test.util.ejb.EJBTestCase;
 
 import org.jboss.logging.Logger;
 import org.jboss.test.JBossTestCase;
+import org.jboss.test.util.ejb.EJBTestCase;
 
 /**
  * A test suite to check JBoss data mapping to/from Large Binary Objects (LOBs).
  *
- * @see org.jboss.test.util.ejb.EJBTestCase
- *
- * @version <tt>$Revision: 61780 $</tt>
+ * @version <tt>$Revision: 67396 $</tt>
  * @author  <a href="mailto:steve@resolvesw.com">Steve Coy</a>.
  * @author  <a href="mailto:alex@jboss.org">Alexey Loubyansky</a>
  */
@@ -98,14 +97,14 @@ public class LOBUnitTestCase extends EJBTestCase
       suite.addTest(new LOBUnitTestCase("testValueHolder"));
       suite.addTest(new LOBUnitTestCase("testCleanGetValueHolder"));
       suite.addTest(new LOBUnitTestCase("testStateFactoryValueHolder"));
-      
+      suite.addTest(new LOBUnitTestCase("testJBAS3717"));
+
       return JBossTestCase.getDeploySetup(suite, "cmp2-lob.jar");
    }
 
    // Constructors --------------------------------------------------
 
    public LOBUnitTestCase(String name)
-      throws java.io.IOException
    {
       super(name);
    }
@@ -293,7 +292,6 @@ public class LOBUnitTestCase extends EJBTestCase
 
    public void testMapCMPField() throws Exception
    {
-      log.debug("testMapCMPField> start");
       Facade facade = facadeHome.create();
       Integer id = new Integer(111);
       try
@@ -398,7 +396,6 @@ public class LOBUnitTestCase extends EJBTestCase
 
    public void testValueHolder() throws Exception
    {
-      log.debug("testValueHolder> start");
       Facade facade = facadeHome.create();
       Integer id = new Integer(555);
       try
@@ -471,7 +468,7 @@ public class LOBUnitTestCase extends EJBTestCase
     * Lookup the LOB lobHome and cache it.
     * Load the test data.
     */
-   public void setUpEJB()
+   public void setUpEJB(Properties props)
       throws Exception
    {
       log.debug("setupEJB");
@@ -492,13 +489,27 @@ public class LOBUnitTestCase extends EJBTestCase
       }
    }
 
+   public void testJBAS3717() throws Exception
+   {
+      Facade facade = facadeHome.create();
+      Integer id = new Integer(12);
+      try
+      {
+         facade.createLOB(id);
+         assertTrue(facade.executeDynamicQuery("select distinct object(o) from lob o", null));
+      }
+      finally
+      {
+         try { facade.removeLOB(id); } catch(Exception e) {}
+      }
+   }
+
    /**
     * Remove data references so that they can be garbage collected if needed.
     */
-   public void tearDownEJB()
+   public void tearDownEJB(Properties props)
       throws Exception
    {
-      log.debug("tearDownEJB");
    }
 
    // Protected -------------------------------------------------------
