@@ -42,7 +42,7 @@ import org.jboss.tools.SerialVersionUID;
  * org.jboss.tools.SerialVersionUID utility.
  *
  * @author Scott.Stark@jboss.org
- * @version $Revision: 66320 $
+ * @version $Revision: 74467 $
  */
 public class SerialVersionUIDUnitTestCase extends TestCase
 {
@@ -91,6 +91,8 @@ public class SerialVersionUIDUnitTestCase extends TestCase
          "org.jboss.invocation.iiop.IIOPInvoker$TransientServantLocator",
          // jboss remoting upgrade to 2.0.0.GA
          "org.jboss.remoting",
+         // jboss resource adaptor jdbc vendor 
+         "org.jboss.resource.adapter.jdbc.vendor",
          // Ignore the JacORB IDL compiler generated stubs/ties
          "org.jboss.tm.iiop._CoordinatorExtStub",
          "org.jboss.tm.iiop._TransactionFactoryExtStub",
@@ -98,6 +100,8 @@ public class SerialVersionUIDUnitTestCase extends TestCase
          "org.jboss.iiop.csiv2.SASCurrentLocalTie",
          // Ignore org.jboss.ws
          "org.jboss.ws",
+         // JBAS-4148, ignore org.jfree classes
+         "org.jfree",
          // Ignore jgroups issues after the upgrade to 2.4.1
          "org.jgroups"
       };
@@ -162,6 +166,8 @@ public class SerialVersionUIDUnitTestCase extends TestCase
          "org.jboss.invocation.iiop.IIOPInvoker$TransientServantLocator",
          // jboss remoting upgrade to 2.0.0.GA
          "org.jboss.remoting",
+         // jboss resource adaptor jdbc vendor 
+         "org.jboss.resource.adapter.jdbc.vendor",
          // jboss-serialization 1.0.3.GA
          "org.jboss.serial.objectmetamodel.DataContainer",
          // Ignore the JacORB IDL compiler generated stubs/ties
@@ -171,6 +177,8 @@ public class SerialVersionUIDUnitTestCase extends TestCase
          "org.jboss.iiop.csiv2.SASCurrentLocalTie",
          // Ignore org.jboss.ws
          "org.jboss.ws",
+         // JBAS-4148, ignore org.jfree classes
+         "org.jfree",
          // Ignore jgroups issues after the upgrade to 2.4.1
          "org.jgroups",
          // Ignore incompatible classes from the sun-javamail upgrade, JBAS-3488
@@ -238,11 +246,15 @@ public class SerialVersionUIDUnitTestCase extends TestCase
          "org.jboss.invocation.unified.marshall.InvocationUnMarshaller",
          // jboss remoting upgrade to 2.0.0.GA
          "org.jboss.remoting",
+         // jboss resource adaptor jdbc vendor 
+         "org.jboss.resource.adapter.jdbc.vendor",
          // Ignore the JacORB IDL compiler generated stubs/ties
          "org.jboss.tm.iiop._CoordinatorExtStub",
          "org.jboss.tm.iiop._TransactionFactoryExtStub",
          "org.jboss.tm.iiop._TransactionServiceStub",
          "org.jboss.iiop.csiv2.SASCurrentLocalTie",
+         // JBAS-4148, ignore org.jfree classes
+         "org.jfree",
          // Ignore jgroups issues after the upgrade to 2.4.1
          "org.jgroups",         
          // The aop classes were not finalized until 4.0.3
@@ -312,6 +324,8 @@ public class SerialVersionUIDUnitTestCase extends TestCase
          "org.jboss.invocation.iiop.IIOPInvoker$TransientServantLocator",
          // jboss remoting upgrade to 2.0.0.GA
          "org.jboss.remoting",
+         // jboss resource adaptor jdbc vendor 
+         "org.jboss.resource.adapter.jdbc.vendor",
          // Ignore the JacORB IDL compiler generated stubs/ties
          "org.jboss.tm.iiop._CoordinatorExtStub",
          "org.jboss.tm.iiop._TransactionFactoryExtStub",
@@ -320,6 +334,8 @@ public class SerialVersionUIDUnitTestCase extends TestCase
          // Ignore org.jboss.webservice for 4.0.1 since the org.apache.axis to
          // org.jboss.axis package name change breaks serialization
          "org.jboss.webservice",
+         // JBAS-4148, ignore org.jfree classes
+         "org.jfree",
          // Ignore org.hsqldb as there are some utility classes that changed
          // in the upgrade to 1_8_0
          "org.hsqldb",
@@ -395,6 +411,8 @@ public class SerialVersionUIDUnitTestCase extends TestCase
          "org.jboss.invocation.iiop.IIOPInvoker$TransientServantLocator",
          // jboss remoting upgrade to 2.0.0.GA
          "org.jboss.remoting",
+         // jboss resource adaptor jdbc vendor 
+         "org.jboss.resource.adapter.jdbc.vendor",
          // serialVersionUIDs were added at v4.0.4
          "org.jboss.monitor.alarm.AlarmNotification",
          "org.jboss.monitor.alarm.AlarmTableNotification",
@@ -406,6 +424,8 @@ public class SerialVersionUIDUnitTestCase extends TestCase
          // Ignore org.jboss.webservice for 4.0.1 since the org.apache.axis to
          // org.jboss.axis package name change breaks serialization
          "org.jboss.webservice",
+         // JBAS-4148, ignore org.jfree classes
+         "org.jfree",
          // Ignore org.hsqldb as there are some utility classes that changed
          // in the upgrade to 1_8_0
          "org.hsqldb",
