@@ -71,7 +71,7 @@ import org.jboss.test.JBossTestCase;
 /** Tests of the LoginModule classes.
 
  @author Scott.Stark@jboss.org
- @version $Revision: 65634 $
+ @version $Revision: 75772 $
  */
 public class LoginModulesUnitTestCase extends JBossTestCase
 {
@@ -365,6 +365,17 @@ public class LoginModulesUnitTestCase extends JBossTestCase
       {
          return getConnection("sa", "");
       }
+
+      public boolean isWrapperFor(Class<?> iface) throws SQLException
+      {
+        return false;
+      }
+
+      public <T> T unwrap(Class<T> iface) throws SQLException
+      {
+         return null;
+      }
+
       public java.sql.Connection getConnection(String user, String pass) throws java.sql.SQLException
       {
 			java.sql.Connection con = null;
