@@ -54,7 +54,7 @@ import org.w3c.dom.NodeList;
  * 
  * @author Scott.Stark@jboss.org
  * @author Dimitris.Andreadis@jboss.org
- * @version $Revision: 59596 $
+ * @version $Revision: 74690 $
  */
 public class PropertyEditorsUnitTestCase extends JBossTestCase
 {
@@ -358,11 +358,21 @@ public class PropertyEditorsUnitTestCase extends JBossTestCase
          getLog().debug("Current Locale: " + Locale.getDefault());
          DateEditor.initialize();
          
-         // An important date
-         text = "ven., 25 juin 1971 00:30:00 +0200";
-         format = new SimpleDateFormat("EEE, d MMM yyyy HH:mm:ss Z");
-         date = format.parse(text);      
-         
+         try
+         {
+            // An important date
+            text = "ven., 25 juin 1971 00:30:00 +0200";
+            format = new SimpleDateFormat("EEE, d MMM yyyy HH:mm:ss Z");
+            date = format.parse(text);
+         }
+         catch (java.text.ParseException pe)
+         {
+            // "ver." doesn't work correctly with JDK on Z-systems, wherefore using ven - see JBAS-5639
+            text = "ven, 25 juin 1971 00:30:00 +0200";
+            format = new SimpleDateFormat("EEE, d MMM yyyy HH:mm:ss Z");
+            date = format.parse(text);
+         }
+
          editor = new DateEditor();
          editor.setAsText(text);
          getLog().debug("setAsText('" + text + "') --> getValue() = '" + editor.getValue() + "'");
