@@ -41,6 +41,7 @@ import org.apache.catalina.connector.Connector;
 import org.apache.tomcat.util.modeler.Registry;
 import org.jboss.deployment.DeploymentInfo;
 import org.jboss.deployment.SubDeployerExt;
+import org.jboss.metadata.WebMetaData;
 import org.jboss.mx.util.MBeanProxyExt;
 import org.jboss.security.plugins.JaasSecurityManagerServiceMBean;
 import org.jboss.system.ServiceControllerMBean;
@@ -130,6 +131,12 @@ public class JBossWeb extends AbstractWebContainer
     * Whether we are using Apache MOD_JK(2) module or not
     */
    private boolean useJK = false;
+   
+   /**
+    * Maximum interval a clustered session should allow request to not
+    * replicate the session timestamp. 
+    */
+   private int maxUnreplicatedInterval = WebMetaData.DEFAULT_MAX_UNREPLICATED_INTERVAL;
 
    /**
     * A flag indicating if the JBoss Loader should be used
@@ -301,7 +308,17 @@ public class JBossWeb extends AbstractWebContainer
    public void setUseJK(boolean useJK)
    {
       this.useJK = useJK;
-   } 
+   }
+
+   public int getMaxUnreplicatedInterval()
+   {
+      return maxUnreplicatedInterval;
+   }
+
+   public void setMaxUnreplicatedInterval(int maxUnreplicatedInterval)
+   {
+      this.maxUnreplicatedInterval = maxUnreplicatedInterval;
+   }
 
    public boolean getDeleteWorkDirOnContextDestroy()
    {
@@ -647,6 +664,7 @@ public class JBossWeb extends AbstractWebContainer
       config.setSnapshotMode(this.snapshotMode);
       config.setUseLocalCache(this.useLocalCache);
       config.setUseJK(this.useJK);
+      config.setMaxUnreplicatedInterval(this.maxUnreplicatedInterval);
       config.setSubjectAttributeName(this.subjectAttributeName);
       config.setUseJBossWebLoader(this.useJBossWebLoader);
       config.setAllowSelfPrivilegedWebApps(this.allowSelfPrivilegedWebApps);
