@@ -60,7 +60,7 @@ import org.jboss.metadata.WebMetaData;
  * 
  * @version $Revision: 57206 $
  */
-abstract class ClusteredSession
+public abstract class ClusteredSession
    extends StandardSession 
    implements Externalizable
 {
@@ -151,10 +151,7 @@ abstract class ClusteredSession
    protected transient long lastReplicated;
 
    /**
-    * Maximum percentage of the inactive interval this session
-    * should be allowed to go unreplicated if access to the
-    * session doesn't mark it as dirty. Drives the calculation
-    * of maxUnreplicatedInterval.
+    * @deprecated Not used
     */
    protected transient int maxUnreplicatedFactor = 80;
    
@@ -163,7 +160,10 @@ abstract class ClusteredSession
     * should be allowed to go unreplicated if access to the
     * session doesn't mark it as dirty. 
     */
-   protected transient long maxUnreplicatedInterval;
+   protected transient long maxUnreplicatedInterval = 0;
+   
+   /** True if maxUnreplicatedInterval is 0 or less than maxInactiveInterval */
+   protected transient boolean alwaysReplicateMetadata = true;
    
    /**
     * Whether any of this session's attributes implement
@@ -200,7 +200,7 @@ abstract class ClusteredSession
       invalidationPolicy = manager.getInvalidateSessionPolicy();
       this.useJK = useJK;
       this.firstAccess = true;
-      calcMaxUnreplicatedInterval();
+      checkAlwaysReplicateMetadata();
    }
 
    /**
@@ -342,40 +342,18 @@ abstract class ClusteredSession
    }
 
    /**
-    * Gets the maximum percentage of the <code>maxInactiveInterval</code>
-    * beyond which a session should be replicated upon access even if it 
-    * isn't dirty.  Used to ensure that even a read-only session gets 
-    * replicated before it expires, so that it isn't removed from other
-    * nodes.
-    * 
-    * @return  an int between 1 and 100, or -1 if replicating on access is
-    *          disabled
+    * @deprecated Returns a meaningless value; use {@link #setMaxUnreplicatedInterval(int)}
     */
    public int getMaxUnreplicatedFactor()
    {
-      return maxUnreplicatedFactor;
+      return 80;
    }
 
    /**
-    * Sets the maximum percentage of the <code>maxInactiveInterval</code>
-    * beyond which a session should be replicated upon access even if it 
-    * isn't dirty.  Used to ensure that even a read-only session gets 
-    * replicated before it expires, so that it isn't removed from other
-    * nodes.
-    * 
-    * @param maxUnreplicatedFactor  an int between 1 and 100, or -1 to
-    *                               disable replicating on access
-    * 
-    * @throws IllegalArgumentException if the factor isn't -1 or between
-    *                                  1 and 100
+    * @deprecated Ignored; use {@link #setMaxUnreplicatedInterval(int)}
     */
    public void setMaxUnreplicatedFactor(int factor)
-   {
-      if ((factor != -1 && factor < 1) || factor > 100)
-         throw new IllegalArgumentException("Invalid factor " + factor +
-                                   " -- must be between 1 and 100 or -1");
-      this.maxUnreplicatedFactor = factor;
-      calcMaxUnreplicatedInterval();
+   {      
    }
    
 
@@ -386,7 +364,7 @@ abstract class ClusteredSession
    public void setMaxInactiveInterval(int interval)
    {
       super.setMaxInactiveInterval(interval);
-      calcMaxUnreplicatedInterval();
+      checkAlwaysReplicateMetadata();
       sessionMetadataDirty();
    }
 
@@ -408,32 +386,74 @@ abstract class ClusteredSession
       lastReplicated = System.currentTimeMillis();
    }
 
+   /**
+    * Get the maximum interval between requests, in <strong>milliseconds</strong>, 
+    * after which a request will trigger replication of the session's metadata 
+    * regardless of whether the request has otherwise made the session dirty. 
+    * <p/>
+    * <strong>NOTE:</strong> This value is in milliseconds while the equivalent
+    * property in JBossCacheManager is in seconds.
+    * 
+    * @return the maximum interval since last replication after which a request
+    *         will trigger session metadata replication. A value of 
+    *         <code>0</code> means replicate metadata on every request; a value 
+    *         of <code>-1</code> means never replicate metadata unless the 
+    *         session is otherwise dirty.
+    */
    public long getMaxUnreplicatedInterval()
    {
       return maxUnreplicatedInterval;
    }
+
+   /**
+    * Sets the maximum interval between requests, in <strong>milliseconds</strong>, 
+    * after which a request will trigger replication of the session's metadata 
+    * regardless of whether the request has otherwise made the session dirty. 
+    * <p/>
+    * <strong>NOTE:</strong> This value is in milliseconds while the equivalent
+    * property in JBossCacheManager is in seconds.
+    * 
+    * @param  maxUnreplicatedInterval  
+    *         the maximum interval since last replication after which a request
+    *         will trigger session metadata replication. A value of 
+    *         <code>0</code> means replicate metadata on every request; a value 
+    *         of <code>-1</code> means never replicate metadata unless the 
+    *         session is otherwise dirty. A value less than <code>-1</code> is
+    *         treated as <code>-1</code>.
+    */
+   public void setMaxUnreplicatedInterval(long interval)
+   {
+      this.maxUnreplicatedInterval = Math.max(interval, -1);
+      checkAlwaysReplicateMetadata();
+   }
    
    public boolean getExceedsMaxUnreplicatedInterval()
    {
-      boolean result = false;
+      boolean exceeds = alwaysReplicateMetadata;
       
-      if (maxUnreplicatedInterval > 0) // -1 means ignore; 0 means expire now
+      if (!exceeds && maxUnreplicatedInterval > 0) // -1 means ignore
       {
-         result = ((System.currentTimeMillis() - lastReplicated) >= maxUnreplicatedInterval);
+         long unrepl = System.currentTimeMillis() - lastReplicated;
+         exceeds = (unrepl >= maxUnreplicatedInterval);
       }      
       
-      return result;
+      return exceeds;
    }
    
-   private void calcMaxUnreplicatedInterval()
+   private void checkAlwaysReplicateMetadata()
    {
-      if (maxInactiveInterval < 0 || maxUnreplicatedFactor < 0)
-         maxUnreplicatedInterval = -1;
-      else
+      boolean was = this.alwaysReplicateMetadata;
+      this.alwaysReplicateMetadata = 
+         (maxUnreplicatedInterval == 0 
+            || (maxUnreplicatedInterval > 0 && maxInactiveInterval >= 0 
+                  && maxUnreplicatedInterval > (maxInactiveInterval * 1000)));
+      
+      if (this.alwaysReplicateMetadata && !was && log.isTraceEnabled())
       {
-         // Ignoring casting would be (mII * 1000) * (mUF / 100)
-         maxUnreplicatedInterval = maxInactiveInterval * maxUnreplicatedFactor * 10;
-      }
+         log.trace(id + " will always replicate metadata; maxUnreplicatedInterval=" 
+                   + maxUnreplicatedInterval
+                   + ", maxInactiveInterval=" + maxInactiveInterval);
+      }  
    }
 
    /**
@@ -1012,8 +1032,8 @@ abstract class ClusteredSession
       version = 0;
       hasActivationListener = null;
       lastReplicated = 0;
-      maxUnreplicatedFactor = 80;
-      calcMaxUnreplicatedInterval();
+      maxUnreplicatedInterval = 0;
+      this.alwaysReplicateMetadata = true;
    }
    
    /**
@@ -1132,6 +1152,12 @@ abstract class ClusteredSession
          // access cannot be the first.
          this.firstAccess = false;
          
+         // Assume deserialization means replication and use thisAccessedTime
+         // as a proxy for when replication occurred
+         this.lastReplicated = this.thisAccessedTime;
+         
+         checkAlwaysReplicateMetadata();
+         
          // TODO uncomment when work on JBAS-1900 is completed      
 //         // Session notes -- for FORM auth apps, allow replicated session 
 //         // to be used without requiring a new login
@@ -1426,7 +1452,8 @@ abstract class ClusteredSession
    
    public boolean getReplicateSessionBody()
    {
-      return sessionMetadataDirty || getExceedsMaxUnreplicatedInterval();
+      return sessionMetadataDirty || getExceedsMaxUnreplicatedInterval()
+                || (maxUnreplicatedInterval == -1 && sessionAttributesDirty);
    }
 
    protected boolean isGetDirty(Object attribute)
