@@ -45,6 +45,7 @@ import org.apache.catalina.Session;
 import org.apache.catalina.Valve;
 import org.apache.catalina.core.ContainerBase;
 import org.jboss.cache.CacheException;
+import org.jboss.cache.aop.PojoCacheMBean;
 import org.jboss.metadata.WebMetaData;
 import org.jboss.mx.util.MBeanServerLocator;
 import org.jboss.web.tomcat.service.JBossWeb;
@@ -61,9 +62,8 @@ import org.jboss.web.tomcat.service.JBossWeb;
 public class JBossCacheManager
    extends JBossManager
    implements JBossCacheManagerMBean
-{
-
-  /**
+{  
+   /**
     * Informational name for this Catalina component
     */
    static final String info_ = "JBossCacheManager/1.0";
@@ -87,7 +87,7 @@ public class JBossCacheManager
 
    /** Our TreeCache's ObjectName */
    private String cacheObjectNameString_ = JBossWeb.DEFAULT_CACHE_NAME;
-
+   
    /**
     * If set to true, will add a JvmRouteFilter to the request.
     */
@@ -130,6 +130,8 @@ public class JBossCacheManager
     * Whether we are doing trace level logging
     */
    private boolean trace;
+   
+   private int maxUnreplicatedInterval_ = WebMetaData.DEFAULT_MAX_UNREPLICATED_INTERVAL;
 
    //  ----------------------------------------------------------  Constructors
 
@@ -137,6 +139,17 @@ public class JBossCacheManager
    {
       super();
    }
+   
+   /** 
+    * Create a new JBossCacheManager using the given cache. For use in unit testing.
+    * 
+    * @param pojoCache
+    */
+   public JBossCacheManager(PojoCacheMBean pojoCache)
+   {
+      super();
+      this.proxy_ = new JBossCacheService(pojoCache);
+   }
 
    /**
     * Initializes this Manager when running in embedded mode.
@@ -154,7 +167,14 @@ public class JBossCacheManager
       this.replicationFieldBatchMode_ =
          webMetaData.getReplicationFieldBatchMode() ? Boolean.TRUE : Boolean.FALSE;
       
-      proxy_ = new JBossCacheService(cacheObjectNameString_);
+      Integer maxUnrep = webMetaData.getMaxUnreplicatedInterval();
+      if (maxUnrep != null)
+      {
+         this.maxUnreplicatedInterval_ = maxUnrep.intValue();
+      }
+      
+      if (proxy_ == null)
+         proxy_ = new JBossCacheService(cacheObjectNameString_);
 
       // Confirm our replication granularity is compatible with the cache
       // Throws ISE if not
@@ -377,6 +397,16 @@ public class JBossCacheManager
    {
       this.useLocalCache_ = useLocalCache;
    }
+   
+   public int getMaxUnreplicatedInterval()
+   {
+      return maxUnreplicatedInterval_;
+   }
+   
+   public void setMaxUnreplicatedInterval(int maxUnreplicatedInterval)
+   {
+      this.maxUnreplicatedInterval_ = maxUnreplicatedInterval;
+   }
 
    // JBossCacheManagerMBean-methods -------------------------------------
 
@@ -741,6 +771,7 @@ public class JBossCacheManager
             session = new SessionBasedClusteredSession(this);
             break;
       }
+      
       return session;
    }
 
@@ -1019,15 +1050,14 @@ public class JBossCacheManager
             }
             catch (Exception exn)
             {
-               exn.printStackTrace();
+               log_.error("Problem rolling back session mgmt transaction", exn);
             }
             
             // We will need to alert Tomcat of this exception.
             if (ex instanceof RuntimeException)
                throw (RuntimeException) ex;
             
-            throw new RuntimeException("loadSession(): failed to load session " +
-                                       realId, ex);
+            throw new RuntimeException("Failed to load session " + realId, ex);
          }
          finally
          {
@@ -1248,12 +1278,18 @@ public class JBossCacheManager
          long now = System.currentTimeMillis();
          Map unloaded = new HashMap(unloadedSessions_);
          Set entries = unloaded.entrySet();
+         // We may have not gotten replication of a timestamp for requests 
+         // that occurred w/in maxUnreplicatedInterval_ of the previous
+         // request. So we add a grace period to avoid flushing a session early
+         // and permanently losing part of its node structure in JBoss Cache.
+         long maxUnrep = maxUnreplicatedInterval_ < 0 ? 60 : maxUnreplicatedInterval_;
+         long maxUnused = maxInactiveInterval_ + maxUnrep;
          for (Iterator it = entries.iterator(); it.hasNext(); )
          {
             Map.Entry entry = (Map.Entry) it.next();
             OwnedSessionUpdate osu = (OwnedSessionUpdate) entry.getValue();
             int elapsed = (int) ((now - osu.updateTime) / 1000L);
-            if (elapsed >= maxInactiveInterval_)
+            if (elapsed >= maxUnused)
             {
                String realId = (String) entry.getKey();
                try
@@ -1699,7 +1735,7 @@ public class JBossCacheManager
       }
 
       // Add clustered session valve
-      ClusteredSessionValve valve = new ClusteredSessionValve();
+      ClusteredSessionValve valve = new ClusteredSessionValve(this);
       installContextValve(valve);
    }
 
