@@ -21,7 +21,7 @@
  */
 package javax.xml.ws.http;
 
-// $Id: HTTPBinding.java 2897 2007-04-23 06:12:12Z thomas.diesler@jboss.com $
+// $Id: HTTPBinding.java 5983 2008-03-15 03:42:44Z thomas.diesler@jboss.com $
 
 import javax.xml.ws.Binding;
 
@@ -30,8 +30,8 @@ import javax.xml.ws.Binding;
  * 
  *  @since JAX-WS 2.0
 **/
-public interface HTTPBinding extends Binding {
-
+public interface HTTPBinding extends Binding
+{
   /**
    * A constant representing the identity of the XML/HTTP binding.
    */
